#include "StdAfx.h"
#include "AndModel.h"
#include "DetectModel.h"

CAndModel::CAndModel(void)
{
	ChildSet.clear( );
	HoG_W.clear( );
	Shp_W.clear( );

	NodeID		= -1;
	Level		= -1;
	PosX		= 0;
	PosY		= 0;
	SizeX		= 0;
	SizeY		= 0;
	Bias		= 0;
	ViewID		= -1;
	Parent		= -1;
	Partner		= -1;
}

CAndModel::~CAndModel(void)
{
}

int CAndModel::ReadModel(ifstream &inModel)
{
	int		ChdCnt;

	inModel>>NodeID;
	inModel>>ViewID;
	inModel>>ChdCnt;
	inModel>>Level;
	inModel>>Partner;
	inModel>>Bias;

	inModel>>PosX;
	inModel>>PosY;
	inModel>>SizeX;
	inModel>>SizeY;

	inModel>>Dim_HoG;
	inModel>>Dim_Shp;

	if ((SizeX > MaxSizeX) || (SizeY > MaxSizeY))
	{
		cout<<"Node "<<setw(4)<<NodeID<<" : ("<<setw(4)<<SizeX<<", "<<setw(4)<<SizeY<<") "<<endl;
		exit(0);
	}

	ChildSet.resize(ChdCnt);
	for ( int t = 0; t < ChdCnt; t++ )
	{
		inModel>>ChildSet[t];
	}

	HoG_W.resize(Dim_HoG);
	for ( int t = 0; t < Dim_HoG; t++ )
	{
		inModel>>HoG_W[t];
	}

	Shp_W.resize(Dim_Shp);
	for ( int t = 0; t < Dim_Shp; t++ )
	{
		inModel>>Shp_W[t];
	}

	return	1;
}

int CAndModel::WriteModel(ofstream &outModel)
{
	int		ChdCnt	= (int) ChildSet.size( );
	int		Dim_HoG	= (int) HoG_W.size( );
	int		Dim_Shp	= (int) Shp_W.size( );

	outModel.precision(8);
	outModel<<setw(8)<<NodeID;
	outModel<<setw(8)<<ViewID;
	outModel<<setw(8)<<ChdCnt;
	outModel<<setw(8)<<Level;
	outModel<<setw(8)<<Partner;
	outModel<<setw(16)<<fixed<<Bias<<endl;

	outModel<<setw(8)<<PosX<<setw(8)<<PosY;
	outModel<<setw(8)<<SizeX<<setw(8)<<SizeY;
	outModel<<endl;

	outModel<<setw(8)<<Dim_HoG;
	outModel<<setw(8)<<Dim_Shp;
	outModel<<endl;

	for ( int t = 0; t < ChdCnt; t++ )
	{
		outModel<<setw(8)<<ChildSet[t];
	}
	outModel<<endl;

	outModel.precision(8);
	for ( int t = 0; t < Dim_HoG; t++ )
	{
		outModel<<setw(16)<<fixed<<HoG_W[t];
		if ((t+1) % (SizeX * SizeY) == 0)
		{
			outModel<<endl;
		}
	}

	for ( int t = 0; t < Dim_Shp; t++ )
	{
		outModel<<setw(16)<<fixed<<Shp_W[t];
	}
	outModel<<endl;

	outModel<<endl;
	return	1;
}

int CAndModel::GetSymIdx( )
{
	vector<int>		SimIdx;
	int				Dim_HoG	= 32;
	int				Dim		= (int) HoG_W.size( );
	int				Idx1, Idx2;

	// left-right symmetry of 32D features
	SimIdx.assign(Dim_HoG, 0);
	SimIdx[0]	= 9;	SimIdx[1]	= 8;	SimIdx[2]	= 7;	SimIdx[3]	= 6;	SimIdx[4]	= 5;	SimIdx[5]	= 4;	SimIdx[6]	= 3;	SimIdx[7]	= 2;	SimIdx[8]	= 1;
	SimIdx[9]	= 0;	SimIdx[10]	= 17;	SimIdx[11]	= 16;	SimIdx[12]	= 15;	SimIdx[13]	= 14;	SimIdx[14]	= 13;	SimIdx[15]	= 12;	SimIdx[16]	= 11;	SimIdx[17]	= 10;
	SimIdx[18]	= 18;	SimIdx[19]	= 26;	SimIdx[20]	= 25;	SimIdx[21]	= 24;	SimIdx[22]	= 23;	SimIdx[23]	= 22;	SimIdx[24]	= 21;	SimIdx[25]	= 20;	SimIdx[26]	= 19;
	SimIdx[27]	= 29;	SimIdx[28]	= 30;	SimIdx[29]	= 27;	SimIdx[30]	= 28;	SimIdx[31]	= 31;

	// left-right symmetry of cells
	SymIdx.resize(Dim);
	for ( int y = 0; y < SizeY; y++ )
	{
		for ( int x = 0; x < SizeX; x++ )
		{
			for ( int t = 0; t < Dim_HoG; t++ )
			{
				Idx1	= SizeX * SizeY * t + SizeX * y + x;
				Idx2	= SizeX * SizeY * SimIdx[t] + SizeX * y + (SizeX - x - 1);
				SymIdx[Idx1]	= Idx2;
			}
		}
	}

	return	1;
}

int CAndModel::InitParts(CDetectModel &DetModel, int XCnt, int YCnt, int &AndCnt, int &OrCnt)
{
	int		CCnt	= (int) ChildSet.size( );
	int		Cnt		= XCnt * YCnt;
	CRect	CurRect;
	int		NodeID, Cx, Cy;
	int		CSizeX, CSizeY;
	int		ret;

	if (CCnt > 0)	// whether this node has children nodes
	{
		// Initialize children nodes if this node has children nodes
		for ( int t = 0; t < CCnt; t++ )
		{
			NodeID	= ChildSet[t];
			ret		= DetModel.AndModels[NodeID].InitParts(DetModel, XCnt, YCnt, AndCnt, OrCnt);
		}
	}
	else
	{
		if (Level + 1 < DetModel.MaxL)
		{
			// number of nodes in each level (Level 2: 3*3; Level 3: 6*6, 6=3*2)
			XCnt	= (Level == 0) ? 3 : 2;
			YCnt	= (Level == 0) ? 3 : 2;
			Cnt		= XCnt * YCnt;

			// the size of node in current scale
			CSizeX	= SizeX;
			CSizeY	= SizeY;
			for ( int t = DetModel.HoG_S[Level]; t < DetModel.HoG_S[Level + 1]; t++ )
			{
				CSizeX	*= 2;
				CSizeY	*= 2;
			}

			// Initial children nodes
			for ( int t = 0; t < Cnt; t++ )
			{
				Cx	= t % XCnt;
				Cy	= t / XCnt;

				// left
				if (Cx * 2 + 1 < XCnt)
				{
					int		BSizeX	= int(1.0 * CSizeX / XCnt + 0.5);
					float	BSizeY	= 1.0f * CSizeY / YCnt;

					CurRect.left	= Cx * BSizeX;
					CurRect.right	= Cx * BSizeX + BSizeX;
					CurRect.top		= int(Cy * BSizeY + 0.5);
					CurRect.bottom	= int(Cy * BSizeY + 1.0 * BSizeY + 0.5);
				}

				// right
				if (Cx * 2 + 1 > XCnt)
				{
					int		BSizeX	= int(1.0 * CSizeX / XCnt + 0.5);
					float	BSizeY	= 1.0f * CSizeY / YCnt;

					CurRect.left	= CSizeX - (XCnt - Cx) * BSizeX;
					CurRect.right	= CSizeX - (XCnt - Cx - 1) * BSizeX;
					CurRect.top		= int(Cy * BSizeY + 0.5);
					CurRect.bottom	= int(Cy * BSizeY + 1.0 * BSizeY + 0.5);
				}

				// center
				if (Cx * 2 + 1 == XCnt)
				{
					int		BSizeX	= int(0.5 * CSizeX / XCnt + 0.5);
					float	BSizeY	= 1.0f * CSizeY / YCnt;

					CurRect.left	= CSizeX / 2 - BSizeX;
					CurRect.right	= (CSizeX + 1) / 2 + BSizeX;
					CurRect.top		= int(Cy * BSizeY + 0.5);
					CurRect.bottom	= int(Cy * BSizeY + 1.0 * BSizeY + 0.5);
				}

				CAndModel	&NxtModel	= DetModel.AndModels[AndCnt];

				// information of nodes
				NxtModel.ViewID		= ViewID;
				NxtModel.NodeID		= AndCnt;
				NxtModel.SizeX		= CurRect.Width( );
				NxtModel.SizeY		= CurRect.Height( );
				NxtModel.PosX		= CurRect.left;
				NxtModel.PosY		= CurRect.top;
				NxtModel.Bias		= 0.0;
				NxtModel.Level		= Level + 1;
				NxtModel.Partner	= -1;

				NxtModel.ChildSet.clear( );
				NxtModel.Dim_HoG	= NxtModel.SizeX * NxtModel.SizeY * g_Parameters.Dim_HoG;
				NxtModel.Dim_Shp	= g_Parameters.Dim_Shape;
				NxtModel.HoG_W.assign(NxtModel.SizeX * NxtModel.SizeY * g_Parameters.Dim_HoG, 0);
				NxtModel.Shp_W.assign(g_Parameters.Dim_Shape, 0);
				NxtModel.Shp_W[0]	= 0.1f;
				NxtModel.Shp_W[2]	= 0.1f;

				ChildSet.push_back(AndCnt);
				AndCnt++;
				ret	= NxtModel.InitParts(DetModel, XCnt, YCnt, AndCnt, OrCnt);
			}
		}
	}

	return	1;
}

int CAndModel::GetNodePos(CDetectModel &DetModel, int cx, int cy, int View)
{
	int		ret;

	if (Level > 0)
	{
		// calculate the position of parent node in current resolution
		for ( int t = DetModel.HoG_S[Level-1]; t < DetModel.HoG_S[Level]; t++ )
		{
			cx	*= 2;
			cy	*= 2;
		}
	}

	// add offset of position
	RPosX	= cx + PosX;
	RPosY	= cy + PosY;
	RSizeX	= SizeX;
	RSizeY	= SizeY;
	ViewID	= View;
	for ( int t = 0; t < (int) ChildSet.size( ); t++ )
	{
		ret	= DetModel.AndModels[ChildSet[t]].GetNodePos(DetModel, RPosX, RPosY, ViewID);
	}

	return	1;
}