#include "StdAfx.h"
#include "AndNode.h"
#include "OrNode.h"
#include "Data.h"
#include "InstRect.h"
#include "InstTree.h"
#include "DetectModel.h"
#include "FilterLib/ExternalTools.h"

CAndNode::CAndNode(void)
{
}

CAndNode::~CAndNode(void)
{
}

// Get ratio between two resolution
int CAndNode::GetStepRate(int Pos_S1, int Pos_S2, int &Step)
{
	Step	= 1;
	for ( int p = 0; p < Pos_S1 - Pos_S2; p++ )
	{
		Step	*= 2;
	}

	return	1;
}

// calculate the bounding boxes of current node 
int CAndNode::GetNodeRect(CDetectModel &DetModel, CData &Data, int x, int y, CRect &CurRect)
{
	CAndModel	&CurModel	= DetModel.AndModels[NodeID];
	int			BinSize		= Data.BinSize_HoG;
	double		RScale, PosX, PosY;

	RScale			= Data.FeaPyd[HoG_L].Scale;
	PosX			= (x + InitX) * BinSize / RScale;
	PosY			= (y + InitY) * BinSize / RScale;
	CurRect.left	= int(PosX + 0.5);
	CurRect.top		= int(PosY + 0.5);
	CurRect.right	= int(PosX + CurModel.SizeX * BinSize / RScale + 0.5 + 1);
	CurRect.bottom	= int(PosY + CurModel.SizeY * BinSize / RScale + 0.5 + 1);

	return	1;
}

// Crop the bounding box according to the image size
int CAndNode::CropNodeRect(CData &Data, CRect &CurRect)
{
	CurRect.left	= __max(0, __min(Data.ImW, CurRect.left));
	CurRect.right	= __max(0, __min(Data.ImW, CurRect.right));
	CurRect.top		= __max(0, __min(Data.ImH, CurRect.top));
	CurRect.bottom	= __max(0, __min(Data.ImH, CurRect.bottom));
	return	1;
}

// Calculate HoG score of one position (dot product)
double CAndNode::GetScore_HoG(CData &Data, vector<double> &HoG_W, int PosX, int PosY, int SizeX, int SizeY)
{
	FeaMapType	&FeaMap	= Data.FeaPyd[HoG_L];
	int			Dim		= FeaMap.Map_HoG.dimv( );
	int			DimX	= FeaMap.Map_HoG.dimx( );
	int			DimY	= FeaMap.Map_HoG.dimy( );
	int			DimSize	= DimX * DimY;
	double		*pW		= &HoG_W[0];
	double		*pS1	= FeaMap.Map_HoG.data + PosY * DimX + PosX;
	double		*pS2	= NULL;
	double		Score	= 0;

	for ( int t = 0; t < Dim; t++ )
	{
		pS2	= pS1 + t * DimSize;
		for ( int y = 0; y < SizeY; y++ )
		{
			for ( int x = 0; x < SizeX; x++ )
			{
				Score	+= (*pS2++) * (*pW++);
			}
			pS2	+= DimX - SizeX;
		}
	}

	return	Score;
}

// Calculate appearance score (HoG) of current node (all positions)
int CAndNode::MakeAppScore(CDetectModel &DetModel, CData &Data, CInstTree &InstTree)
{
	CAndModel	&CurModel	= DetModel.AndModels[NodeID];
	int			MarX		= g_Parameters.MarX;
	int			MarY		= g_Parameters.MarY;
	double		Bias		= CurModel.Bias * g_Parameters.Bias_Node;
	int			PosX, PosY;
	double		Sre_HoG;

	for ( int y = 0; y < SizeY; y++ )
	{
		for ( int x = 0; x < SizeX; x++ )
		{
			PosX	= x + InitX + MarX;
			PosY	= y + InitY + MarY;
			Sre_HoG	= GetScore_HoG(Data, CurModel.HoG_W, PosX, PosY, CurModel.SizeX, CurModel.SizeY);
			matSre[y][x]	= Sre_HoG + Bias;
		}
	}

	return	1;
}

// Add scores of children nodes
int CAndNode::AddChdScore(CAndNode &NxtNode, int Step)
{
	for ( int y = 0; y < SizeY; y++ )
	{
		for ( int x = 0; x < SizeX; x++ )
		{
			matSre[y][x]	+= NxtNode.matSre[y*Step][x*Step];
		}
	}

	return	1;
}

// Calculate scores of current node
int CAndNode::MakeNodeScore(CDetectModel &DetModel, CData &Data, CInstTree &InstTree)
{
	CAndModel	&CurModel	= DetModel.AndModels[NodeID];
	int			CurL		= CurModel.Level;
	int			ret;

	// Add scores of children nodes
	for ( int t = 0; t < (int) CurModel.ChildSet.size( ); t++ )
	{
		int			NxtID		= CurModel.ChildSet[t];
		CAndNode	&NxtNode	= InstTree.AndNodes[NxtID];
		int			Step;

		ret	= GetStepRate(DetModel.HoG_S[CurL+1], DetModel.HoG_S[CurL], Step);
		ret	= NxtNode.MakeNodeScore(DetModel, Data, InstTree);
		ret	= AddChdScore(NxtNode, Step);
	}

	// calculate shape scores
	ret	= MakeShpScore(DetModel, InstTree);
	return	1;
}

// calculate shape scores
int CAndNode::MakeShpScore(CDetectModel &DetModel, CInstTree &InstTree)
{
	CAndModel			&CurModel	= DetModel.AndModels[NodeID];
	TMatrix<double>		matTrans	= matSre;
	int					PosX		= CurModel.PosX;
	int					PosY		= CurModel.PosY;
	int					ret;

	g_Parameters.timer2.Start( );
	if (CurModel.Shp_W.size( ) == 4)		// [-dx*dx, -dx, -dy*dy, -dy]
	{
		double	ax, bx, ay, by;
		int		MovX	= CurModel.SizeX;
		int		MovY	= CurModel.SizeY;

		ax	= CurModel.Shp_W[0];
		bx	= CurModel.Shp_W[1];
		ay	= CurModel.Shp_W[2];
		by	= CurModel.Shp_W[3];
		ret	= CExternalTools::DistanceTransform(matTrans, matSre, matSrcX, matSrcY, SizeX, SizeY, PosX, PosY, ax, bx, ay, by, MovX, MovY);
	}
	g_Parameters.timer2.Stop( );

	return	1;
}

// Get the bounding box given the position (just for root node)
int CAndNode::GetInstRects(CDetectModel &DetModel, CData &Data, CInstTree &InstTree, CInstRect &CurRet, int x, int y)
{
	CAndModel	&CurModel	= DetModel.AndModels[NodeID];
	int			MarX		= g_Parameters.MarX;
	int			MarY		= g_Parameters.MarY;
	int			ret;

	CurRet.Lvl		= HoG_L;
	CurRet.PosX		= x + MarX + InitX;
	CurRet.PosY		= y + MarY + InitY;
	CurRet.Score	= matSre[y][x];
	CurRet.ViewID	= CurModel.ViewID;
	ret	= GetNodeRect(DetModel, Data, x, y, CurRet.RootRect);
	ret	= CropNodeRect(Data, CurRet.RootRect);

	return	1;
}

// Get the instance with maximum overlap given the position (for positive image)
int CAndNode::GetInst_Overlap(CDetectModel &DetModel, CData &Data, CInstTree &InstTree, CSVMInst &CurInst, double &MaxS, int x, int y)
{
	CAndModel	&CurModel	= DetModel.AndModels[NodeID];
	float		Overlap		= g_Parameters.Overlap_Root;
	CRect		DetRet;
	float		CurS;
	int			ret;

	// As a positive data, overlap should be large than 0.7 with ground-truth
	ret		= GetNodeRect(DetModel, Data, x, y, DetRet);
	ret		= CropNodeRect(Data, DetRet);
	CurS	= CExternalTools::GetRectOverlap(DetRet, Data.PosRect, 0);
	if (CurS < Overlap)
	{
		return	1;
	}

	// check whether the instance is the one with maximum overlap
	ret		= GetNodeRect(DetModel, Data, x, y, DetRet);
	CurS	= CExternalTools::GetRectOverlap(DetRet, Data.PosRect, 0);
	if ((CurS < MaxS) || ((fabs(CurS - MaxS) < 1e-5) && (CurInst.LRCmp < 0)))
	{
		return	1;
	}

	// Get information
	MaxS	= CurS;
	CurInst.Score_SVM	= matSre[y][x];
	CurInst.Score_KKT	= matSre[y][x];
	CurInst.DetRect		= DetRet;
	CurInst.ViewID		= CurModel.ViewID;
	CurInst.SymID		= DetModel.ViewToSym[CurModel.ViewID];
	CurInst.Lvl			= HoG_L;
	CurInst.PosX		= x;
	CurInst.PosY		= y;

	// Get features of current instances
	ret	= CurInst.GetShpFea(DetModel, InstTree, NodeID, x, y);
	ret	= CurInst.GetAppFea(DetModel, Data, InstTree);
	ret	= CurInst.GetLRCmp(DetModel);

	return	1;
}

// Get the instance with maximum score given the position (for positive image)
int CAndNode::GetInst_Score(CDetectModel &DetModel, CData &Data, CInstTree &InstTree, CSVMInst &CurInst, int x, int y)
{
	CAndModel	&CurModel	= DetModel.AndModels[NodeID];
	float		Overlap		= g_Parameters.Overlap_Root;
	CRect		DetRet;
	float		CurS;
	int			ret;

	// As a positive data, overlap should be large than 0.7 with ground-truth
	ret		= GetNodeRect(DetModel, Data, x, y, DetRet);
	ret		= CropNodeRect(Data, DetRet);
	CurS	= CExternalTools::GetRectOverlap(DetRet, Data.PosRect, 0);
	if (CurS < Overlap)
	{
		return	1;
	}

	// Get information
	CurInst.Score_SVM	= matSre[y][x];
	CurInst.Score_KKT	= matSre[y][x];
	CurInst.DetRect		= DetRet;
	CurInst.ViewID		= CurModel.ViewID;
	CurInst.SymID		= DetModel.ViewToSym[CurModel.ViewID];
	CurInst.Lvl			= HoG_L;
	CurInst.PosX		= x;
	CurInst.PosY		= y;

	// Get features of current instances
	ret	= CurInst.GetShpFea(DetModel, InstTree, NodeID, x, y);
	ret	= CurInst.GetAppFea(DetModel, Data, InstTree);

	return	1;
}

// Get the instance given the position (for negative image)
int CAndNode::GetTopInsts(CDetectModel &DetModel, CData &Data, CInstTree &InstTree, CSVMInst &CurInst, int x, int y)
{
	CAndModel	&CurModel	= DetModel.AndModels[NodeID];
	CRect		DetRect;
	int			ret;

	// Get information
	ret	= GetNodeRect(DetModel, Data, x, y, DetRect);
	ret	= CurInst.Init(DetModel);

	CurInst.Alpha		= 0.0f;
	CurInst.DataID		= Data.DataID;
	CurInst.Label		= 1;
	CurInst.Score_Loss	= 1.0f;
	CurInst.Score_SVM	= matSre[y][x];
	CurInst.Score_KKT	= matSre[y][x] + CurInst.Score_Loss;
	CurInst.DetRect		= DetRect;
	CurInst.ViewID		= CurModel.ViewID;
	CurInst.SymID		= DetModel.ViewToSym[CurModel.ViewID];
	CurInst.Lvl			= HoG_L;
	CurInst.PosX		= x;
	CurInst.PosY		= y;

	// Get features of current instances
	ret	= CurInst.GetShpFea(DetModel, InstTree, NodeID, x, y);
	ret	= CurInst.GetAppFea(DetModel, Data, InstTree);

	return	1;
}