#include "StdAfx.h"
#include "Data.h"

CData::CData(void)
{
	DataID		= 0;
	Label		= 0;
	InVal		= 0;
	BinSize_HoG	= 0;

	ImW			= 0;
	ImH			= 0;

	SrcImg.clear( );
	strFileName.clear( );
	FeaPyd.clear( );

	PosRect.SetRect(0, 0, 0, 0);
}

CData::~CData(void)
{
	FreeImageSpace( );
	FreePyramid( );
}

int CData::ReadSourceImage( )
{
	CExternalTools::myLoadImage(strFileName, SrcImg);
	ImW	= SrcImg.dimx( );
	ImH	= SrcImg.dimy( );
	return	1;
}

int CData::ReadSourceImage(string strFileName)
{
	this->strFileName	= strFileName;
	CExternalTools::myLoadImage(strFileName, SrcImg);
	ImW	= SrcImg.dimx( );
	ImH	= SrcImg.dimy( );
	return	1;
}

int CData::ReadSrcImgWithRect(string strFileName)
{
	char	fname[MAX_PATH];
	int		ret;

	sprintf_s(fname, MAX_PATH, "%s.jpg", strFileName.c_str( ));
	CExternalTools::myLoadImage(fname, SrcImg);
	this->strFileName	= fname;
	ImW	= SrcImg.dimx( );
	ImH	= SrcImg.dimy( );

	sprintf_s(fname, MAX_PATH, "%s.txt", strFileName.c_str( ));
	ret	= ReadPosRect(fname);
	return	1;
}

int CData::WriteSrcImgWithRect(string strFileName)
{
	char	fname[MAX_PATH];
	int		ret;

	sprintf_s(fname, MAX_PATH, "%s.jpg", strFileName.c_str( ));
	CExternalTools::mySaveImage(fname, SrcImg);

	sprintf_s(fname, MAX_PATH, "%s.txt", strFileName.c_str( ));
	ret	= WritePosRect(fname);
	return	1;
}

void CData::FreeImageSpace( )
{
	SrcImg.clear( );
}

int CData::ReadPosRect(string strRectFileName)
{
	ifstream	inRect(strRectFileName.c_str( ));

	inRect>>PosRect.left;
	inRect>>PosRect.right;
	inRect>>PosRect.top;
	inRect>>PosRect.bottom;
	inRect.close( );
	return	1;
}

int CData::WritePosRect(string strRectFileName)
{
	ofstream	outRect(strRectFileName.c_str( ));

	outRect<<setw(6)<<PosRect.left;
	outRect<<setw(6)<<PosRect.right;
	outRect<<setw(6)<<PosRect.top;
	outRect<<setw(6)<<PosRect.bottom;
	outRect.close( );
	return	1;
}

int CData::ExtractFeaturePyramid(int Interval, int InitScale)
{
	CHoGExtractor		Extractor_HoG;
	int					MinCelCnt	= g_Parameters.CellNumber_LowBound;
	double				SclStp, MinS;
	double				Scale;
	int					SclCnt, MaxS, Index;
	int					ret;

	if ((InitScale != 0) && (InitScale != 1))
	{
		return	0;
	}

	BinSize_HoG	= g_Parameters.BinSize_HoG;
	this->InVal	= Interval;

	// HoG - determine the height of pyramid
	SclStp	= pow(2, 1.0 / Interval);
	MinS	= 1.0f * __min(ImW, ImH) / (MinCelCnt * BinSize_HoG);
	SclCnt	= __max(0, Interval * InitScale + 1 + int(log(MinS) / log(SclStp)));
	MaxS	= __max(0, __min(SclCnt - Interval * InitScale, Interval));

	FreePyramid( );
	FeaPyd.resize(SclCnt);

	// extract HoG features
	for ( int t = 0; t < MaxS; t++ )
	{
		CImg<double>	Im1, Im2;
		int				CurH, CurW, PreH, PreW;
		int				HoG_H, HoG_W;

		Index	= 0;
		Scale	= 1.0f / pow(SclStp, t);
		CExternalTools::ResizeImage(Im1, SrcImg, Scale);
		CurH	= Im1.dimy( );
		CurW	= Im1.dimx( );

		if (InitScale >= 1)
		{
			ret	= Extractor_HoG.GetHoGMap(Im1, BinSize_HoG / 2, FeaPyd[Index+t].Map_HoG, HoG_W, HoG_H);
			FeaPyd[Index+t].Scale	= Scale * 2;
			FeaPyd[Index+t].Map_H	= HoG_H;
			FeaPyd[Index+t].Map_W	= HoG_W;
			FeaPyd[Index+t].Img_H	= CurH * 2;
			FeaPyd[Index+t].Img_W	= CurW * 2;
			Index	+= Interval;
		}

		if (InitScale >= 0)
		{
			ret	= Extractor_HoG.GetHoGMap(Im1, BinSize_HoG, FeaPyd[Index+t].Map_HoG, HoG_W, HoG_H);
			FeaPyd[Index+t].Scale	= Scale;
			FeaPyd[Index+t].Map_H	= HoG_H;
			FeaPyd[Index+t].Map_W	= HoG_W;
			FeaPyd[Index+t].Img_H	= CurH;
			FeaPyd[Index+t].Img_W	= CurW;
			Index	+= Interval;
		}

		for ( int p = Index + t; p < SclCnt; p += Interval )
		{
			PreH	= CurH;
			PreW	= CurW;
			CExternalTools::ResizeImage(Im2, Im1, 0.5);
			swap(Im1, Im2);

			ret	= Extractor_HoG.GetHoGMap(Im1, BinSize_HoG, FeaPyd[p].Map_HoG, HoG_W, HoG_H);
			FeaPyd[p].Scale	= FeaPyd[p-Interval].Scale * 0.5f;
			FeaPyd[p].Map_H	= HoG_H;
			FeaPyd[p].Map_W	= HoG_W;
			FeaPyd[p].Img_H	= CurH;
			FeaPyd[p].Img_W	= CurW;
		}
	}

	return	1;
}


void CData::FreePyramid( )
{
	vector<FeaMapType>	NulPyd;

	NulPyd.swap(FeaPyd);
}

int CData::DrawPosData( )
{
	vector<Color>	&Colors	= g_Parameters.Colors;
	CImageEx		*pImage	= new CImageEx( );
	char			fname[MAX_PATH];
	int				ret;

	ret	= pImage->Load(strFileName.c_str( ));
	ret	= pImage->ConvertToARGB32Bits( );

	Graphics		graphics(pImage->m_pBitmap);
	SolidBrush		myBrush(Color(255, 0, 0));
	Pen				myPen(&myBrush, 2);
	Rect			rect;
	PointF			point;

	rect.X		= PosRect.left;
	rect.Y		= PosRect.top;
	rect.Width	= PosRect.Width( );
	rect.Height	= PosRect.Height( );
	ret	= graphics.DrawRectangle(&myPen, rect);

	sprintf_s(fname, MAX_PATH, "PosSample\\%06d.jpg", DataID);
	ret	= pImage->Save(fname);
	delete	pImage;
	return	1;
}

