#include "StdAfx.h"
#include "DataSet.h"

CDataSet::CDataSet(void)
{
}

CDataSet::~CDataSet(void)
{
}

void CDataSet::Clear( )
{
	PosCnt	= 0;
	NegCnt	= 0;
	PosSet.clear( );
	NegSet.clear( );
}

int CDataSet::SplitByAspect(vector<vector<int> > &AspectIdx, int AspectNumber)
{
	vector<double>			Aspects, Aspects_Rank;
	vector<double>			AspectThres;
	double					Aspect;
	int						Index;

	// calculate aspects of all positive data
	Aspects.clear( );
	for ( int t = 0; t < PosCnt; t++ )
	{
		Aspect	= 1.0 * PosSet[t].PosRect.Height( ) / PosSet[t].PosRect.Width( );
		Aspects.push_back(Aspect);
	}

	// sort the aspects, and get the threshold of each view. The number of data in each view is equal.
	Aspects_Rank	= Aspects;
	AspectThres.assign(AspectNumber, 0);
	sort(Aspects_Rank.begin( ), Aspects_Rank.end( ), less<double>( ));
	for ( int p = 0; p < AspectNumber; p++ )
	{
		Index			= (int) Aspects_Rank.size( ) * p / AspectNumber;
		AspectThres[p]	= Aspects_Rank[Index];
	}

	AspectIdx.resize(AspectNumber);
	for ( int t = 0; t < AspectNumber; t++ )
	{
		AspectIdx[t].clear( );
	}

	// Assign data to each view
	for ( int t = 0; t < PosCnt; t++ )
	{
		Index	= AspectNumber - 1;
		for ( int p = 1; p < AspectNumber; p++ )
		{
			if (Aspects[t] < AspectThres[p])
			{
				Index	= p - 1;
				break;
			}
		}

		AspectIdx[Index].push_back(t);
	}

	return	1;
}

// calculate aspect of each view. The method is to find the aspect which occurs most frequently.
int CDataSet::GetAspect(vector<int> &AspectIdx, double &Aspect)
{
	vector<double>		AspectHist1, AspectHist2;
	CKernel1D<double>	Kernel;
	int					BinCount	=  200;
	double				MaxAspect	=  2.0;
	double				MinAspect	= -2.0;
	double				MaxScore	= 0;
	int					DataID, BinID;

	AspectHist1.assign(BinCount+1+2*BinCount, 0);
	AspectHist2.assign(BinCount+1+2*BinCount, 0);
	for ( int t = 0; t < (int) AspectIdx.size( ); t++ )
	{
		DataID	= AspectIdx[t];
		Aspect	= 1.0 * PosSet[DataID].PosRect.Height( ) / PosSet[DataID].PosRect.Width( );
		Aspect	= log(Aspect);
		BinID	= int(BinCount * (Aspect - MinAspect) / (MaxAspect - MinAspect) + 0.5);
		BinID	= __max(0, __min(BinCount, BinID)) + BinCount;

		AspectHist1[BinID]	+= 1;
	}

	CKernel1D<double>::GenerateGaussian(Kernel, 10);
	Kernel._convolve(BinCount + 1 + 2 * BinCount, (&AspectHist2[0]), (&AspectHist1[0]), 1, 1, 1);

	for ( int t = 0; t <= BinCount; t++ )
	{
		if (AspectHist2[t+BinCount] > MaxScore)
		{
			MaxScore	= AspectHist2[t+BinCount];
			Aspect		= exp(MinAspect + t * (MaxAspect - MinAspect) / BinCount);
		}
	}

	return	1;
}

// Get the area of each group. The area is large then 80% of data in the view.
int CDataSet::GetAreaSize(vector<int> &AspectIdx, double &AreaSize)
{
	vector<double>		Areas;
	int					DataID;
	int					Index	= int(0.8 * AspectIdx.size( ) + 0.5);

	Areas.assign((int) AspectIdx.size( ), 0);
	for ( int t = 0; t < (int) AspectIdx.size( ); t++ )
	{
		DataID		= AspectIdx[t];
		Areas[t]	= 1.0 * PosSet[DataID].PosRect.Height( ) * PosSet[DataID].PosRect.Width( );
	}

	sort(Areas.begin( ), Areas.end( ), greater<double>( ));
	AreaSize	= Areas[Index];
	return	1;
}