#include "StdAfx.h"
#include "DetectModel.h"
#include "DataSet.h"

CDetectModel::CDetectModel(void)
{
	OrModels.clear( );
	AndModels.clear( );
}

CDetectModel::~CDetectModel(void)
{
}

int CDetectModel::ReadModel(string strModelName)
{
	ifstream	inModel(strModelName.c_str( ));
	int			AndCnt, OrCnt;
	int			ret;

	inModel>>Root;
	inModel>>MaxL;
	inModel>>ViewCnt;
	inModel>>AndCnt;
	inModel>>OrCnt;
	inModel>>Bias_Neg;
	inModel>>Thres;

	ViewToSym.assign(ViewCnt, 0);
	for ( int t = 0; t < ViewCnt; t++ )
	{
		inModel>>ViewToSym[t];
	}

	HoG_S.assign(MaxL, 0);
	for ( int t = 0; t < MaxL; t++ )
	{
		inModel>>HoG_S[t];
		InitS	= __max(InitS, HoG_S[t]);
	}

	AndModels.resize(AndCnt);
	for ( int t = 0; t < AndCnt; t++ )
	{
		ret	= AndModels[t].ReadModel(inModel);
		AndModels[t].Parent	= -1;
	}

	OrModels.resize(OrCnt);
	for ( int t = 0; t < OrCnt; t++ )
	{
		ret	= OrModels[t].ReadModel(inModel);
	}

	for ( int t = 0; t < AndCnt; t++ )
	{
		for ( int p = 0; p < (int) AndModels[t].ChildSet.size( ); p++ )
		{
			AndModels[AndModels[t].ChildSet[p]].Parent	= t;
		}
	}

	Thres	= __min(Thres, Bias_Neg * g_Parameters.Bias_Neg -1.1f);
	NodeCnt	= AndCnt;
	inModel.close( );
	inModel.clear( );
	return	1;
}

int CDetectModel::WriteModel(string strDetectModelFileName)
{
	ofstream	outModel(strDetectModelFileName.c_str( ));
	int			AndCnt	= (int) AndModels.size( );
	int			OrCnt	= (int) OrModels.size( );
	int			ret;

	outModel.precision(8);
	outModel<<setw(8)<<Root;
	outModel<<setw(8)<<MaxL;
	outModel<<setw(8)<<ViewCnt;
	outModel<<setw(8)<<AndCnt;
	outModel<<setw(8)<<OrCnt;
	outModel<<setw(16)<<fixed<<Bias_Neg;
	outModel<<setw(16)<<fixed<<Thres;
	outModel<<endl;

	for ( int t = 0; t < ViewCnt; t++ )
	{
		outModel<<setw(8)<<ViewToSym[t];
	}
	outModel<<endl;

	for ( int t = 0; t < MaxL; t++ )
	{
		outModel<<setw(8)<<HoG_S[t];
	}
	outModel<<endl;
	outModel<<endl;

	for ( int t = 0; t < AndCnt; t++ )
	{
		ret	= AndModels[t].WriteModel(outModel);
	}

	for ( int t = 0; t < OrCnt; t++ )
	{
		ret	= OrModels[t].WriteModel(outModel);
	}
	outModel<<endl;

	outModel.close( );
	outModel.clear( );
	return	1;
}

// Get the minimum area among views
int CDetectModel::GetAreaSize_Min( )
{
	COrModel	&RootModel	= OrModels[Root];
	int			Area		= INT_MAX;
	int			SizeX		= INT_MAX;
	int			SizeY		= INT_MAX;
	int			BinSize		= g_Parameters.BinSize_HoG;

	for ( int t = 0; t < (int) RootModel.ChildSet.size( ); t++ )
	{
		int			NxtID		= RootModel.ChildSet[t];
		CAndModel	&NxtModel	= AndModels[NxtID];

		SizeX	= __min(SizeX, NxtModel.SizeX);
		SizeY	= __min(SizeY, NxtModel.SizeY);
	}

	Area	= SizeX * SizeY * BinSize * BinSize;
	return	Area;
}

// Get the maximum area among views
int CDetectModel::GetAreaSize_Max( )
{
	COrModel	&RootModel	= OrModels[Root];
	int			MaxArea		= 0;
	int			BinSize		= g_Parameters.BinSize_HoG;

	for ( int t = 0; t < (int) RootModel.ChildSet.size( ); t++ )
	{
		int			NxtID		= RootModel.ChildSet[t];
		CAndModel	&NxtModel	= AndModels[NxtID];
		int			Area		= NxtModel.SizeX * NxtModel.SizeY;

		MaxArea	= __max(Area, MaxArea);
	}

	MaxArea	*= BinSize * BinSize;
	return	MaxArea;
}


//*******************************************************************************************************************************************//
//***************************************************************     Init     **************************************************************//
//*******************************************************************************************************************************************//

// Initial model only with root node
int CDetectModel::InitRoots(CDataSet &DataSet, vector<vector<int> > &AspIdx)
{
	int			AspectCnt	= (int) AspIdx.size( );
	int			BinSize		= g_Parameters.BinSize_HoG;
	int			MaxSize		= g_Parameters.MaxDetectSize;
	int			MinSize		= g_Parameters.MinDetectSize;
	double		Aspect, AreaSize;
	int			NodeID;
	int			ret;

	Root		= 0;
	MaxL		= 1;
	InitS		= 0;
	Bias_Neg	= 0;
	Thres		= 0.0;
	ViewCnt		= AspectCnt * 2;
	OrModels.resize(1);
	AndModels.resize(ViewCnt);
	HoG_S.clear( );
	HoG_S.push_back(0);

	// views
	ViewToSym.clear( );
	for ( int t = 0; t < AspectCnt; t++ )
	{
		ViewToSym.push_back(t);
		ViewToSym.push_back(t);
	}

	// or nodes (only root is or node)
	NodeID	= 0;
	OrModels[NodeID].ChildSet.clear( );
	OrModels[NodeID].NodeID	= NodeID;
	for ( int t = 0; t < ViewCnt; t++ )
	{
		OrModels[NodeID].ChildSet.push_back(t);
	}

	for ( int t = 0; t < AspectCnt; t++ )
	{
		NodeID	= 2 * t;

		// current view
		AndModels[NodeID].ViewID	= NodeID;
		AndModels[NodeID].PosX		= 0;
		AndModels[NodeID].PosY		= 0;
		AndModels[NodeID].Level		= 0;
		AndModels[NodeID].NodeID	= NodeID;
		AndModels[NodeID].Bias		= 0;
		AndModels[NodeID].Partner	= -1;
		AndModels[NodeID].Shp_W.clear( );
		AndModels[NodeID].HoG_W.clear( );
		AndModels[NodeID].ChildSet.clear( );

		ret	= DataSet.GetAspect(AspIdx[t], Aspect);
		ret	= DataSet.GetAreaSize(AspIdx[t], AreaSize);
		AreaSize	= __max(MinSize, __min(MaxSize, AreaSize));

		AndModels[NodeID].SizeX		= __max(MinSizeX, __min(MaxSizeX, int(sqrt(AreaSize / Aspect) / BinSize + 0.5)));
		AndModels[NodeID].SizeY		= __max(MinSizeY, __min(MaxSizeY, int(sqrt(AreaSize * Aspect) / BinSize + 0.5)));
		AndModels[NodeID].Dim_HoG	= AndModels[NodeID].SizeX * AndModels[NodeID].SizeY * g_Parameters.Dim_HoG;
		AndModels[NodeID].Dim_Shp	= 0;
		AndModels[NodeID].HoG_W.assign(AndModels[NodeID].SizeX * AndModels[NodeID].SizeY * g_Parameters.Dim_HoG, 0);

		// Symmetrical view
		AndModels[NodeID+1]			= AndModels[NodeID];
		AndModels[NodeID+1].ViewID	= NodeID + 1;
		AndModels[NodeID+1].NodeID	= NodeID + 1;
	}

	return	1;
}

// initial model whose height is large than 1
int CDetectModel::InitParts(int MaxL, int XCnt, int YCnt)
{
	int			AndCnt, OrCnt, Cnt;
	int			CurCnt;
	int			ret;

	this->MaxL	= MaxL;
	this->InitS	= 0;
	AndCnt		= (int) AndModels.size( );
	OrCnt		= (int) OrModels.size( );

	// resolution of each level
	for ( int L = 1; L < MaxL; L++ )
	{
		HoG_S.push_back(1);
	}

	Cnt		= AndCnt;
	CurCnt	= 1;
	for ( int L = 0; L < MaxL; L++ )
	{
		CurCnt	= CurCnt * XCnt * YCnt;
		Cnt		= Cnt + AndCnt * CurCnt;
	}

	// for Roots
	AndModels.resize(Cnt);
	OrModels.resize(Cnt - AndCnt + 1);
	ret = OrModels[Root].InitParts((*this), XCnt, YCnt, AndCnt, OrCnt);
	AndModels.resize(AndCnt);
	OrModels.resize(OrCnt);

	// find symmetrical one for each node
	ret	= OrModels[Root].GetNodePos((*this));
	ret	= MatchPartner( );

	return	1;
}

// find the symmetrical nodes
int CDetectModel::MatchPartner( )
{
	int		ViewCnt, SizeX, SizeY;
	int		MaxL;

	ViewCnt	= 0;
	MaxL	= 0;
	for ( int t = 0; t < (int) AndModels.size( ); t++ )
	{
		ViewCnt	= __max(ViewCnt, AndModels[t].ViewID + 1);
		MaxL	= __max(MaxL, AndModels[t].Level + 1);
		AndModels[t].Partner	= -1;
	}

	for ( int V = 0; V < ViewCnt; V++ )
	{
		for ( int L = 0; L < MaxL; L++ )
		{
			SizeX	= 0;
			SizeY	= 0;
			for ( int t = 0; t < (int) AndModels.size( ); t++ )
			{
				if ((AndModels[t].ViewID == V) && (AndModels[t].Level == L))
				{
					SizeX	= __max(SizeX, AndModels[t].RPosX + AndModels[t].RSizeX);
					SizeY	= __max(SizeY, AndModels[t].RPosY + AndModels[t].RSizeY);
				}
			}

			for ( int t = 0; t < (int) AndModels.size( ); t++ )
			{
				// in the same view and in the same level
				if ((AndModels[t].ViewID != V) || (AndModels[t].Level != L) || (AndModels[t].Partner >= 0))
				{
					continue;
				}

				for ( int p = 0; p < (int) AndModels.size( ); p++ )
				{
					// with the same size
					if ((AndModels[p].ViewID != V) || (AndModels[p].Level != L) || (AndModels[p].Partner >= 0) || 
						(AndModels[t].SizeX != AndModels[p].SizeX) || (AndModels[t].SizeY != AndModels[p].SizeY))
					{
						continue;
					}

					// the positions are left-right symmetrical
					if ((AndModels[t].RPosY == AndModels[p].RPosY) && (AndModels[t].RPosX == SizeX - (AndModels[p].RPosX + AndModels[p].RSizeX)))
					{
						AndModels[t].Partner	= AndModels[p].NodeID;
						AndModels[p].Partner	= AndModels[t].NodeID;
					}
				}
			}
		}
	}

	return	1;
}

//*******************************************************************************************************************************************//
//***************************************************************    Train    ***************************************************************//
//*******************************************************************************************************************************************//

// calculate some information of nodes
int CDetectModel::GetNodeInfo( )
{
	int				ViewID;
	int				MarX	= g_Parameters.MarX;
	int				MarY	= g_Parameters.MarY;

	// the dimensions of features 
	MaxChdCnt	= 0;
	LCnts.assign(MaxL, 0);
	HoGStep.assign(MaxL, 1);
	LShp.assign(ViewCnt, 0);
	LHoG.assign(ViewCnt, 0);

	for ( int t = 0; t < MaxL; t++ )
	{
		if (t < MaxL - 1)
		{
			for ( int p = HoG_S[t]; p < HoG_S[t+1]; p++ )
			{
				HoGStep[p]	*= 2;
			}
		}

		LCnts[t]	= 0;
		for ( int p = 0; p < (int) AndModels.size( ); p++ )
		{
			if (AndModels[p].Level == t)
			{
				ViewID					= AndModels[p].ViewID;
				MaxChdCnt				= __max(MaxChdCnt, (int) AndModels[p].ChildSet.size( ));
				AndModels[p].Off_HoG	= LHoG[ViewID];
				AndModels[p].Off_Shp	= LShp[ViewID];
				LHoG[ViewID]			+= (int) AndModels[p].HoG_W.size( );
				LShp[ViewID]			+= (int) AndModels[p].Shp_W.size( );
				LCnts[t]				+= 1;
			}
		}
	}

	return	1;
}

// calculate the mappings of features between two symmetrical views
int CDetectModel::GetSymIdx( )
{
	int		ret;

	// calculate the mappings for each node
	for ( int t = 0; t < NodeCnt; t++ )
	{
		ret	= AndModels[t].GetSymIdx( );
	}

	SymCnt	= 0;
	for ( int t = 0; t < ViewCnt; t++ )
	{
		SymCnt	= __max(SymCnt, ViewToSym[t] + 1);
	}

	LHoG_Sym.assign(SymCnt, 0);
	LShp_Sym.assign(SymCnt, 0);
	ViewToPri.assign(SymCnt, -1);
	for ( int t = 0; t < ViewCnt; t++ )
	{
		if (ViewToPri[ViewToSym[t]] == -1)
		{
			ViewToPri[ViewToSym[t]]	= t;
			LHoG_Sym[ViewToSym[t]]	= LHoG[t];
			LShp_Sym[ViewToSym[t]]	= LShp[t];
		}
	}

	SymIdx_HoG.resize(ViewCnt);
	SymIdx_Shp.resize(ViewCnt);
	SymRat_HoG.resize(ViewCnt);
	SymRat_Shp.resize(ViewCnt);
	for ( int t = 0; t < ViewCnt; t++ )
	{
		SymIdx_HoG[t].assign(LHoG[t], 0);
		SymIdx_Shp[t].assign(LShp[t], 0);
		SymRat_HoG[t].assign(LHoG[t], 0);
		SymRat_Shp[t].assign(LShp[t], 0);
	}

	for ( int t = 0; t < NodeCnt; t++ )
	{
		CAndModel	&CurModel	= AndModels[t];
		int			ViewID		= CurModel.ViewID;
		int			Dim_HoG		= CurModel.Dim_HoG;
		int			Dim_Shp		= CurModel.Dim_Shp;
		int			Partner		= CurModel.Partner;
		int			Off_HoG		= CurModel.Off_HoG;
		int			Off_Shp		= CurModel.Off_Shp;
		int			Off_HoG_Sym	= AndModels[Partner].Off_HoG;
		int			Off_Shp_Sym	= AndModels[Partner].Off_Shp;

		// find the mappings in symmetical node
		for ( int p = 0; p < CurModel.Dim_HoG; p++ )
		{
			SymIdx_HoG[ViewID][Off_HoG + p]		= Off_HoG_Sym + CurModel.SymIdx[p];
			SymRat_HoG[ViewID][Off_HoG + p]		= 1.0f;
		}

		for ( int p = 0; p < CurModel.Dim_Shp; p++ )
		{
			SymIdx_Shp[ViewID][Off_Shp + p]		= Off_Shp_Sym + p;
			SymRat_Shp[ViewID][Off_Shp + p]		= (p == 1) ? - 1.0f : 1.0f;
		}
	}

	return	1;
}

// alloc memory for training
int CDetectModel::AllocMemory(CDataSet &DataSet)
{
	int		MaxInstSetCnt	= g_Parameters.MaxInstSetCnt;
	int		MaxPerInstCnt	= g_Parameters.MaxPerInstCnt;
	int		MaxViewInstCnt	= g_Parameters.MaxViewInstCnt;
	int		PosCnt			= DataSet.PosCnt;
	int		ret;

	// Instance Tree
	ret	= InstTree.Init((*this));

	// Instance Set
	InstSet.resize(MaxPerInstCnt * 2);
	for ( int t = 0; t < MaxPerInstCnt * 2; t++ )
	{
		ret	= InstSet[t].AllocMemory((*this));
	}

	// Training Model
	pW_HoG.resize(SymCnt);
	pW_Shp.resize(SymCnt);
	pW_HoG_Sym.resize(SymCnt);
	pW_Shp_Sym.resize(SymCnt);
	Bias.assign(SymCnt, 0);
	Bias_Sym.assign(SymCnt, 0);
	for ( int t = 0; t < SymCnt; t++ )
	{
		pW_HoG[t]		= (double *) malloc(LHoG_Sym[t] * sizeof(double));
		pW_Shp[t]		= (double *) malloc(LShp_Sym[t] * sizeof(double));
		pW_HoG_Sym[t]	= (double *) malloc(LHoG_Sym[t] * sizeof(double));
		pW_Shp_Sym[t]	= (double *) malloc(LShp_Sym[t] * sizeof(double));
	}

	// Feature Set
	FeaSet_HoG.resize(SymCnt);
	FeaSet_Shp.resize(SymCnt);
	for ( int t = 0; t < SymCnt; t++ )
	{
		FeaSet_HoG[t]	= (double *) malloc(MaxViewInstCnt * LHoG_Sym[t] * sizeof(double));
		FeaSet_Shp[t]	= (double *) malloc(MaxViewInstCnt * LShp_Sym[t] * sizeof(double));
	}

	// Kernel
	KerSre.resize(SymCnt);
	for ( int t = 0; t < SymCnt; t++ )
	{
		KerSre[t]	= (double *) malloc(MaxViewInstCnt * MaxViewInstCnt * sizeof(double));
	}

	// Support Instances
	SuppInsts.resize(SymCnt);
	InstSres.resize(SymCnt);
	InstAlpha.resize(SymCnt);
	NoUseInsts.resize(SymCnt);
	NoUseIndex.resize(SymCnt);
	ViewInstCnt.assign(SymCnt, 0);
	for ( int t = 0; t < SymCnt; t++ )
	{
		NoUseInsts[t].clear( );
		SuppInsts[t].resize(MaxViewInstCnt);
		NoUseIndex[t].assign(MaxViewInstCnt, 0);
		InstSres[t].assign(MaxViewInstCnt, 0);
		InstAlpha[t].assign(MaxViewInstCnt, 0);
	}

	// Set of support instances
	InstSetCnt	= 0;
	SuppInstSet.resize(MaxInstSetCnt);
	Alpha_Neg.assign(MaxInstSetCnt, 0);
	Score_Neg.assign(MaxInstSetCnt, 0);
	for ( int t = 0; t < MaxInstSetCnt; t++ )
	{
		ret	= SuppInstSet[t].Init( );
	}

	return	1;
}

// release memory for training
int CDetectModel::FreeMemory(CDataSet &DataSet)
{
	int		MaxPerInstCnt	= g_Parameters.MaxPerInstCnt;
	int		ret;

	// Inst Set
	for ( int t = 0; t < MaxPerInstCnt * 2; t++ )
	{
		ret	= InstSet[t].FreeMemory( );
	}

	for ( int t = 0; t < SymCnt; t++ )
	{
		free(pW_HoG[t]);
		free(pW_Shp[t]);
		free(pW_HoG_Sym[t]);
		free(pW_Shp_Sym[t]);
	}

	// Feature Set
	for ( int t = 0; t < SymCnt; t++ )
	{
		free(FeaSet_HoG[t]);
		free(FeaSet_Shp[t]);
	}

	// Kernel
	for ( int t = 0; t < SymCnt; t++ )
	{
		free(KerSre[t]);
	}

	return	1;
}

// add a instance to the set of support instances
int CDetectModel::AddInstToSet(int SetID, CSVMInst &PosInst, bool bKerSre)
{
	// positive data
	if (PosInst.Label == 1)
	{
		int		SymID	= PosInst.SymID;
		int		ViewID	= PosInst.ViewID;
		int		InstCnt	= ViewInstCnt[SymID];
		int		Index;

		// get the index for the instance
		if (NoUseInsts[SymID].size( ) == 0)
		{
			Index	= ViewInstCnt[SymID];
			ViewInstCnt[SymID]++;
		}
		else
		{
			Index	= NoUseInsts[SymID][0];
			NoUseInsts[SymID].erase(NoUseInsts[SymID].begin( ));
		}

		SuppInsts[SymID][Index]		= PosInst;
		InstSres[SymID][Index]		= PosInst.Score_KKT;
		InstAlpha[SymID][Index]		= PosInst.Alpha;
		NoUseIndex[SymID][Index]	= 1;

		CSVMInst	&CurInst	= SuppInsts[SymID][Index];
		int			ret;

		CurInst.Index	= Index;
		CurInst.pHoG	= NULL;
		CurInst.pShp	= NULL;
		SuppInstSet[SetID].InstIdx.push_back(Index);

		// Feature
		double	*pFea_HoG	= FeaSet_HoG[SymID] + Index * LHoG_Sym[SymID];
		double	*pFea_Shp	= FeaSet_Shp[SymID] + Index * LShp_Sym[SymID];

		// left-right symmetry
		if (CurInst.ViewID == ViewToPri[SymID])
		{
			memcpy(pFea_HoG, PosInst.pHoG, LHoG_Sym[SymID] * sizeof(double));
			memcpy(pFea_Shp, PosInst.pShp, LShp_Sym[SymID] * sizeof(double));
		}
		else
		{
			ret	= GetSymVal(pFea_HoG, PosInst.pHoG, LHoG_Sym[SymID], SymIdx_HoG[ViewID], SymRat_HoG[ViewID]);
			ret	= GetSymVal(pFea_Shp, PosInst.pShp, LShp_Sym[SymID], SymIdx_Shp[ViewID], SymRat_Shp[ViewID]);
		}

		if (bKerSre)
		{
			// calculate similarity of all support vectors in current view
			ret	= MakeKerScore(SymID, Index);
		}
	}
	else
	{
		// negative data
		Alpha_Neg[SetID]	= PosInst.Alpha;
		Score_Neg[SetID]	= PosInst.Score_Loss;
	}

	return	1;
}

// Get instances of negative image (score is large than a threshold)
int CDetectModel::GetTopInsts(CData &Data, double Thres, int &InstCnt)
{
	int			Interval	= g_Parameters.Interval_Train;
	int			InitS		= this->InitS;
	int			ret;

	ret	= Data.ReadSourceImage( );
	ret	= Data.ExtractFeaturePyramid(Interval, InitS);
	ret	= InstTree.GetTopInsts((*this), Data, InstSet, InstCnt, Thres);
	Data.FreePyramid( );
	Data.FreeImageSpace( );
	return	1;
}

// Get instances of positive image (maximum overlap or maximum score)
int CDetectModel::SetPosSet(CDataSet &DataSet, int IterTime)
{
	vector<int>		IVCnt(ViewCnt, 0);
	CSVMInst		UseInst;
	int				SetIdx, PosCnt, ViewID;
	CTimer			timer;
	int				ret;

	timer.Reset( );
	timer.Start( );

	// Init
	ret		= UseInst.AllocMemory((*this));
	ret		= NegInst.Init((*this));
	SetIdx	= 0;
	PosCnt	= 0;
	NegInst.Alpha		= 0.0;
	NegInst.Score_Loss	= 1.0;
	NegInst.Score_KKT	= 1.0;
	NegInst.Label		= -1;

	if (IterTime == 0)
	{
		// for first round of CCCP --- find the instance with maximum overlap
		for ( int t = 0; t < DataSet.PosCnt; t++ )
		{
			ret		= DataSet.PosSet[t].ReadSourceImage( );
			ret		= InstTree.GetInst_Overlap((*this), DataSet.PosSet[t], UseInst);
			ViewID	= UseInst.ViewID;
			DataSet.UseIdx_Pos[t]	= (ViewID < 0) ? 0 : 1;
			DataSet.PosSet[t].FreeImageSpace( );
			cout<<setw(4)<<t+1<<"/"<<setw(4)<<DataSet.PosCnt<<"\r";

			if (ViewID >= 0)
			{
				SuppInstSet[SetIdx].InstIdx.clear( );
				SuppInstSet[SetIdx].Label	= 1;
				SuppInstSet[SetIdx].DataID	= UseInst.DataID;
				SuppInstSet[SetIdx].Lvl		= UseInst.Lvl;
				SuppInstSet[SetIdx].PosX	= UseInst.PosX;
				SuppInstSet[SetIdx].PosY	= UseInst.PosY;	
				SuppInstSet[SetIdx].ViewID	= UseInst.ViewID;
				SuppInstSet[SetIdx].SymID	= UseInst.SymID;
				SuppInstSet[SetIdx].RIdx	= 0;
				SuppInstSet[SetIdx].Use		= 1;

				ret	= AddInstToSet(SetIdx, UseInst, false);
				ret	= AddInstToSet(SetIdx, NegInst, false);
				IVCnt[ViewID]++;
				PosCnt++;
				SetIdx++;
			}
		}

		InstSetCnt	= PosCnt;
	}

	// for non-first round of CCCP --- find the instance with maximum score
	if (IterTime > 0)
	{
		vector<double>	Alphas;
		double			Alpha;
		int				ViewID, SymID;
		int				RIdx, Index;

		Alphas.clear( );
		for ( int t = 0; t < InstSetCnt; t++ )
		{
			if (SuppInstSet[t].Label == 1)
			{
				RIdx	= SuppInstSet[t].RIdx;
				Index	= SuppInstSet[t].InstIdx[RIdx];
				SymID	= SuppInstSet[t].SymID;
				NoUseIndex[SymID][Index]	= 0;
				NoUseInsts[SymID].push_back(Index);
				Alphas.push_back(SuppInsts[SymID][Index].Alpha);
			}
		}

		for ( int t = 0; t < SymCnt; t++ )
		{
			sort(NoUseInsts[t].begin( ), NoUseInsts[t].end( ));
		}

		for ( int t = 0; t < DataSet.PosCnt; t++ )
		{
			if (DataSet.UseIdx_Pos[t] == 0)
			{
				continue;
			}

			ret		= DataSet.PosSet[t].ReadSourceImage( );
			ret		= InstTree.GetInst_Score((*this), DataSet.PosSet[t], UseInst);
			ViewID	= UseInst.ViewID;
			DataSet.PosSet[t].FreeImageSpace( );
			cout<<setw(4)<<t+1<<"/"<<setw(4)<<DataSet.PosCnt<<"\r";

			if (ViewID < 0)
			{
				cout<<"Pos Data "<<setw(6)<<t<<" --- Error !"<<endl;
				continue;
			}

			// replace the instance of same data
			Alpha	= Alphas[SetIdx];
			SymID	= UseInst.SymID;
			ViewID	= UseInst.ViewID;
			if (NoUseInsts[SymID].size( ) == 0)
			{
				Index	= ViewInstCnt[SymID];
				ViewInstCnt[SymID]++;
			}
			else
			{
				Index	= NoUseInsts[SymID][0];
				NoUseInsts[SymID].erase(NoUseInsts[SymID].begin( ));
			}

			SuppInsts[SymID][Index]		= UseInst;
			InstSres[SymID][Index]		= UseInst.Score_KKT;
			InstAlpha[SymID][Index]		= Alpha;
			NoUseIndex[SymID][Index]	= 1;

			CSVMInst	&CurInst	= SuppInsts[SymID][Index];

			CurInst.Index	= Index;
			CurInst.pHoG	= NULL;
			CurInst.pShp	= NULL;
			CurInst.Alpha	= Alpha;

			SuppInstSet[SetIdx].Label			= 1;
			SuppInstSet[SetIdx].DataID			= CurInst.DataID;
			SuppInstSet[SetIdx].Lvl				= CurInst.Lvl;
			SuppInstSet[SetIdx].PosX			= CurInst.PosX;
			SuppInstSet[SetIdx].PosY			= CurInst.PosY;	
			SuppInstSet[SetIdx].ViewID			= CurInst.ViewID;
			SuppInstSet[SetIdx].SymID			= CurInst.SymID;
			SuppInstSet[SetIdx].InstIdx[RIdx]	= Index;

			double	*pFea_HoG	= FeaSet_HoG[SymID] + CurInst.Index * LHoG_Sym[SymID];
			double	*pFea_Shp	= FeaSet_Shp[SymID] + CurInst.Index * LShp_Sym[SymID];

			// left-right symmetry
			if (CurInst.ViewID == ViewToPri[SymID])
			{
				memcpy(pFea_HoG, UseInst.pHoG, LHoG_Sym[SymID] * sizeof(double));
				memcpy(pFea_Shp, UseInst.pShp, LShp_Sym[SymID] * sizeof(double));
			}
			else
			{
				ret	= GetSymVal(pFea_HoG, UseInst.pHoG, LHoG_Sym[SymID], SymIdx_HoG[ViewID], SymRat_HoG[ViewID]);
				ret	= GetSymVal(pFea_Shp, UseInst.pShp, LShp_Sym[SymID], SymIdx_Shp[ViewID], SymRat_Shp[ViewID]);
			}

			IVCnt[ViewID]++;
			PosCnt++;
			SetIdx++;
		}
	}

	// calculate similarity matrix
	ret	= UpdateState( );
	for ( int t = 0; t < PosCnt; t++ )
	{
		int			RIdx		= SuppInstSet[t].RIdx;
		int			Index		= SuppInstSet[t].InstIdx[RIdx];
		int			SymID		= SuppInstSet[t].SymID;
		CSVMInst	&CurInst	= SuppInsts[SymID][Index];

		ret	= MakeKerScore(SymID, CurInst.Index);
	}

	timer.Stop( );
	cout<<"PosCnt: "<<PosCnt<<endl;
	cout<<"TimeCost: "<<timer.GetElpTime( )<<endl;

	g_Parameters.outCCCP.precision(3);
	g_Parameters.outCCCP<<"PosCnt: "<<setw(6)<<PosCnt<<endl;
	g_Parameters.outCCCP<<"Views : ";
	for ( int t = 0; t < ViewCnt; t++ )
	{
		g_Parameters.outCCCP<<setw(6)<<IVCnt[t];
	}
	g_Parameters.outCCCP<<endl;
	g_Parameters.outCCCP<<"TimeCost: "<<setw(12)<<fixed<<timer.GetElpTime( )<<endl;

	return	1;
}

// convert the model for learning to the model for inference. These two models have differnt data structure.
int CDetectModel::GetInfVer( )
{
	double			C_SVM	= g_Parameters.SVM_C;
	int				MaxCnt	= g_Parameters.MaxViewInstCnt;
	double			Alpha;
	int				SymID;
	int				ret;

	Bias.assign(SymCnt, 0);
	for ( int t = 0; t < SymCnt; t++ )
	{
		memset(pW_HoG[t], 0, LHoG_Sym[t] * sizeof(double));
		memset(pW_Shp[t], 0, LShp_Sym[t] * sizeof(double));
	}

	// calculate weights of featrues
	for ( int t = 0; t < InstSetCnt; t++ )
	{
		if (SuppInstSet[t].Use < 0)
		{
			continue;
		}

		SymID	= SuppInstSet[t].SymID;
		for ( int p = 0; p < (int) SuppInstSet[t].InstIdx.size( ); p++ )
		{
			double		*pHoG, *pShp;
			int			Index;

			Index	= SuppInstSet[t].InstIdx[p];
			Alpha	= InstAlpha[SymID][Index];
			Alpha	= (p == SuppInstSet[t].RIdx) ? 1.0f - Alpha : - Alpha;
			Alpha	*= C_SVM;
			pHoG	= FeaSet_HoG[SymID] + Index * LHoG_Sym[SymID];
			pShp	= FeaSet_Shp[SymID] + Index * LShp_Sym[SymID];

			Bias[SymID]	+= Alpha * g_Parameters.Bias_Node;
			for ( int l = 0; l < LHoG_Sym[SymID]; l++ )
			{
				pW_HoG[SymID][l]	+= pHoG[l] * Alpha;
			}
			for ( int l = 0; l < LShp_Sym[SymID]; l++ )
			{
				pW_Shp[SymID][l]	+= pShp[l] * Alpha;
			}
		}
	}

	// get the weights of symmetrical views
	Bias_Sym	= Bias;
	for ( int t = 0; t < SymCnt; t++ )
	{
		ret	= GetSymVal(pW_HoG_Sym[t], pW_HoG[t], LHoG_Sym[t], SymIdx_HoG[ViewToPri[t]], SymRat_HoG[ViewToPri[t]]);
		ret	= GetSymVal(pW_Shp_Sym[t], pW_Shp[t], LShp_Sym[t], SymIdx_Shp[ViewToPri[t]], SymRat_Shp[ViewToPri[t]]);
	}

	// assign the weight vector of each node 
	for ( int t = 0; t < NodeCnt; t++ )
	{
		CAndModel	&CurModel	= AndModels[t];
		int			ViewID		= CurModel.ViewID;
		int			SymID		= ViewToSym[ViewID];
		int			Off_HoG		= CurModel.Off_HoG;
		int			Dim_HoG		= CurModel.Dim_HoG;
		int			Off_Shp		= CurModel.Off_Shp;
		int			Dim_Shp		= CurModel.Dim_Shp;
		double		*pHoG		= (Dim_HoG > 0) ? &AndModels[t].HoG_W[0] : NULL;
		double		*pShp		= (Dim_Shp > 0) ? &AndModels[t].Shp_W[0] : NULL;

		CurModel.Bias	= (CurModel.Level == 0) ? Bias[SymID] : 0.0f;

		if (ViewID == ViewToPri[SymID])
		{
			memcpy(pHoG, pW_HoG[SymID] + Off_HoG, Dim_HoG * sizeof(double));
			memcpy(pShp, pW_Shp[SymID] + Off_Shp, Dim_Shp * sizeof(double));
		}
		else
		{
			memcpy(pHoG, pW_HoG_Sym[SymID] + Off_HoG, Dim_HoG * sizeof(double));
			memcpy(pShp, pW_Shp_Sym[SymID] + Off_Shp, Dim_Shp * sizeof(double));
		}
	}

	// Bias (Negative instance)
	Bias_Neg	= 0;
	for ( int t = 0; t < InstSetCnt; t++ )
	{
		if (SuppInstSet[t].Use < 0)
		{
			continue;
		}

		Alpha		= Alpha_Neg[t];
		Alpha		= (SuppInstSet[t].RIdx == -1) ? 1.0f - Alpha : - Alpha;
		Alpha		*= C_SVM;
		Bias_Neg	+= Alpha * g_Parameters.Bias_Neg;
	}

	return	1;
}

// train detection model
int CDetectModel::TrainSVM(CDataSet &DataSet)
{
	int		ItrCnt	= 0;
	double	PreS	= DBL_MAX;
	double	CurS	= DBL_MAX;
	int		MaxItr;
	int		ret;

	ret	= AllocMemory(DataSet);

	g_Parameters.Time_All	= 0;
	g_Parameters.Time_SMO	= 0;
	g_Parameters.Time_Ker	= 0;
	g_Parameters.Time_Infer	= 0;
	g_Parameters.Time_Other	= 0;
	g_Parameters.Iter_CCCP	= 0;
	g_Parameters.timer1.Reset( );
	g_Parameters.timer2.Reset( );
	g_Parameters.timer3.Reset( );
	g_Parameters.timer4.Reset( );
	g_Parameters.timer5.Reset( );
	g_Parameters.outCCCP.open("Log_Train.txt");
	g_Parameters.outTime.open("Time_Train.txt");
	g_Parameters.outTime.precision(2);

	ret	= NegInst.Init((*this));
	// the number of negative images used this iteration
	DataSet.NegCnt	= __min((int) DataSet.NegSet.size( ), g_Parameters.NegPicCnt_Start);

	// for each iteration of CCCP
	while (ItrCnt < g_Parameters.Max_CCCP_Iter)
	{
		g_Parameters.timer1.Reset( );
		g_Parameters.timer2.Reset( );
		g_Parameters.timer3.Reset( );
		g_Parameters.timer4.Reset( );
		g_Parameters.timer5.Reset( );
		g_Parameters.timer1.Start( );

		// inference positive data and add them to the set of support vector
		g_Parameters.timer3.Start( );
		PreS	= CurS;
		MaxItr	= (ItrCnt == 0) ? g_Parameters.Max_SMO_Iter * 2 : g_Parameters.Max_SMO_Iter;
		ret		= SetPosSet(DataSet, ItrCnt);
		g_Parameters.timer3.Stop( );

		// update the detection models (the positive data is new)
		g_Parameters.timer5.Start( );
		ret	= GetInfVer( );
		ret	= MakeSVMScores(false);
		g_Parameters.timer5.Stop( );

		// add negative data and update weights
		ret	= SMO_LSVM(DataSet, MaxItr);

		// the number of negative images used next iteration
		DataSet.NegCnt	= __min((int) DataSet.NegSet.size( ), int(DataSet.NegCnt * g_Parameters.NegPicCnt_Ratio + 0.5));
		ItrCnt++;

		g_Parameters.timer1.Stop( );
		g_Parameters.outTime<<"Iter: "<<setw(3)<<ItrCnt<<endl;
		g_Parameters.outTime<<"Time - All   : "<<setw(10)<<fixed<<g_Parameters.timer1.GetCumTime( )<<endl;
		g_Parameters.outTime<<"Time - SMO   : "<<setw(10)<<fixed<<g_Parameters.timer2.GetCumTime( )<<endl;
		g_Parameters.outTime<<"Time - Ker   : "<<setw(10)<<fixed<<g_Parameters.timer3.GetCumTime( )<<endl;
		g_Parameters.outTime<<"Time - Infer : "<<setw(10)<<fixed<<g_Parameters.timer4.GetCumTime( )<<endl;
		g_Parameters.outTime<<"Time - Other : "<<setw(10)<<fixed<<g_Parameters.timer5.GetCumTime( )<<endl;
		g_Parameters.outTime<<"*******************************************************"<<endl;
		g_Parameters.Time_All	+= g_Parameters.timer1.GetCumTime( );
		g_Parameters.Time_SMO	+= g_Parameters.timer2.GetCumTime( );
		g_Parameters.Time_Ker	+= g_Parameters.timer3.GetCumTime( );
		g_Parameters.Time_Infer	+= g_Parameters.timer4.GetCumTime( );
		g_Parameters.Time_Other	+= g_Parameters.timer5.GetCumTime( );

		cout.precision(2);
		cout<<"Iter: "<<setw(3)<<ItrCnt<<endl;
		cout<<"Time - All   : "<<setw(10)<<fixed<<g_Parameters.timer1.GetCumTime( )<<endl;
		cout<<"Time - SMO   : "<<setw(10)<<fixed<<g_Parameters.timer2.GetCumTime( )<<endl;
		cout<<"Time - Ker   : "<<setw(10)<<fixed<<g_Parameters.timer3.GetCumTime( )<<endl;
		cout<<"Time - Infer : "<<setw(10)<<fixed<<g_Parameters.timer4.GetCumTime( )<<endl;
		cout<<"Time - Other : "<<setw(10)<<fixed<<g_Parameters.timer5.GetCumTime( )<<endl;
		cout<<"*******************************************************"<<endl;
	}

	g_Parameters.outCCCP.close( );
	g_Parameters.outTime<<"*******************************************************"<<endl;
	g_Parameters.outTime<<"Time - All   : "<<setw(10)<<fixed<<g_Parameters.Time_All<<endl;
	g_Parameters.outTime<<"Time - SMO   : "<<setw(10)<<fixed<<g_Parameters.Time_SMO<<endl;
	g_Parameters.outTime<<"Time - Ker   : "<<setw(10)<<fixed<<g_Parameters.Time_Ker<<endl;
	g_Parameters.outTime<<"Time - Infer : "<<setw(10)<<fixed<<g_Parameters.Time_Infer<<endl;
	g_Parameters.outTime<<"Time - Other : "<<setw(10)<<fixed<<g_Parameters.Time_Other<<endl;
	g_Parameters.outTime<<"*******************************************************"<<endl;
	g_Parameters.outTime.close( );

	cout.precision(2);
	cout<<"*******************************************************"<<endl;
	cout<<"Time - All   : "<<setw(10)<<fixed<<g_Parameters.Time_All<<endl;
	cout<<"Time - SMO   : "<<setw(10)<<fixed<<g_Parameters.Time_SMO<<endl;
	cout<<"Time - Ker   : "<<setw(10)<<fixed<<g_Parameters.Time_Ker<<endl;
	cout<<"Time - Infer : "<<setw(10)<<fixed<<g_Parameters.Time_Infer<<endl;
	cout<<"Time - Other : "<<setw(10)<<fixed<<g_Parameters.Time_Other<<endl;
	cout<<"*******************************************************"<<endl;

	ret	= FreeMemory(DataSet);
	return	1;
}

// update the status of memory
int CDetectModel::UpdateState( )
{
	int		MaxViewInstCnt	= g_Parameters.MaxViewInstCnt;
	int		SymID, Index;

	ViewInstCnt.assign(SymCnt, 0);
	NoUseIndex.resize(SymCnt);
	for ( int v = 0; v < SymCnt; v++ )
	{
		NoUseIndex[v].assign(MaxViewInstCnt, 0);
	}

	for ( int t = 0; t < InstSetCnt; t++ )
	{
		if (SuppInstSet[t].Use == -1)
		{
			continue;
		}

		SymID	= SuppInstSet[t].SymID;
		for ( int p = 0; p < (int) SuppInstSet[t].InstIdx.size( ); p++ )
		{
			Index	= SuppInstSet[t].InstIdx[p];
			NoUseIndex[SymID][Index]	= 1;
			ViewInstCnt[SymID]			= __max(ViewInstCnt[SymID], Index + 1);
		}
	}

	for ( int t = 0; t < SymCnt; t++ )
	{
		for ( int p = 0; p < (int) NoUseInsts[t].size( ); p++ )
		{
			if (NoUseInsts[t][p] >= ViewInstCnt[t])
			{
				NoUseInsts[t].erase(NoUseInsts[t].begin( ) + p);
				p--;
			}
		}
	}

	return	1;
}

// calculate the similarity for one instance
int CDetectModel::MakeKerScore(int SymID, int CurIdx)
{
	int		ICnt		= ViewInstCnt[SymID];
	int		MaxCnt		= g_Parameters.MaxViewInstCnt;
	double	*pSet_HoG	= FeaSet_HoG[SymID];
	double	*pSet_Shp	= FeaSet_Shp[SymID];
	double	*pHoG		= FeaSet_HoG[SymID] + CurIdx * LHoG_Sym[SymID];
	double	*pShp		= FeaSet_Shp[SymID] + CurIdx * LShp_Sym[SymID];
	double	Sre_Bias	= g_Parameters.Bias_Node * g_Parameters.Bias_Node;
	double	Score;

	// for each vector in the set of support vectors
	for ( int t = 0; t < ICnt; t++ )
	{
		// the vector with the index is not used
		if (NoUseIndex[SymID][t] == 0)
		{
			Score	= 0;
		}

		if (NoUseIndex[SymID][t] == 1)
		{
			Score	= Sre_Bias;
			Score	+= GetScore_Lin(pHoG, pSet_HoG + t * LHoG_Sym[SymID], LHoG_Sym[SymID]);
			Score	+= GetScore_Lin(pShp, pSet_Shp + t * LShp_Sym[SymID], LShp_Sym[SymID]);
		}

		// set the score of the matrix
		KerSre[SymID][t * MaxCnt + CurIdx]	= Score;
		KerSre[SymID][CurIdx * MaxCnt + t]	= Score;
	}

	return	1;
}

// Calculate score of each instance in the set of support instances
int CDetectModel::MakeSVMScores(bool bShow)
{
	vector<vector<double> >		Alphas;
	int							MaxInstCnt	= g_Parameters.MaxViewInstCnt;
	double						Alpha;

	Alphas.resize(SymCnt);
	for ( int t = 0; t < SymCnt; t++ )
	{
		Alphas[t].assign(MaxInstCnt, 0);
	}

	// get weights of each instance
	for ( int t = 0; t < InstSetCnt; t++ )
	{
		CSVMInstSet		&CurSet	= SuppInstSet[t];
		int				SymID	= CurSet.SymID;
		int				InstCnt	= (int) CurSet.InstIdx.size( );
		int				Index;

		if (SuppInstSet[t].Use < 0)
		{
			continue;
		}

		for ( int p = 0; p < InstCnt; p++ )
		{
			Index	= CurSet.InstIdx[p];
			Alpha	= SuppInsts[SymID][Index].Alpha;
			Alpha	= (p == CurSet.RIdx) ? (1.0f - Alpha) : - Alpha;
			Alpha	*= g_Parameters.SVM_C;
			Alphas[SymID][Index]	= Alpha;
		}
	}

	// positive instances
	for ( int t = 0; t < InstSetCnt; t++ )
	{
		for ( int p = 0; p < (int) SuppInstSet[t].InstIdx.size( ); p++ )
		{
			int			SymID		= SuppInstSet[t].SymID;
			int			Index		= SuppInstSet[t].InstIdx[p];
			CSVMInst	&CurInst	= SuppInsts[SymID][Index];
			int			ICnt		= ViewInstCnt[SymID];
			int			Off_Ker		= Index * MaxInstCnt;
			double		Score		= 0;

			// calculate score. 
			for ( int i = 0; i < ICnt; i++ )
			{
				Score	+= KerSre[SymID][Off_Ker + i] * Alphas[SymID][i];
			}

			// for debugging
			if ((bShow) && (fabs(CurInst.Score_SVM - Score) > 0.01))
			{
				cout.precision(6);
				cout<<setw(10)<<fixed<<CurInst.Score_SVM<<" - ";
				cout<<setw(10)<<fixed<<Score<<" - ";
				cout<<setw(10)<<fixed<<InstSres[SymID][Index] - (Score + CurInst.Score_Loss);
				cout<<endl;

				g_Parameters.outCCCP.precision(6);
				g_Parameters.outCCCP<<setw(10)<<fixed<<CurInst.Score_SVM<<" - ";
				g_Parameters.outCCCP<<setw(10)<<fixed<<Score<<" - ";
				g_Parameters.outCCCP<<setw(10)<<fixed<<InstSres[SymID][Index] - (Score + CurInst.Score_Loss);
				g_Parameters.outCCCP<<endl;
			}

			CurInst.Score_SVM	= Score;
			CurInst.Score_KKT	= Score + CurInst.Score_Loss;
			InstSres[SymID][Index]	= CurInst.Score_KKT;
		}
	}

	// negative instances
	NegInst.Score_SVM	= 0;
	for ( int t = 0; t < InstSetCnt; t++ )
	{
		if (SuppInstSet[t].Use < 0)
		{
			continue;
		}

		Alpha	= Alpha_Neg[t];
		Alpha	= (SuppInstSet[t].RIdx == -1) ? 1.0f - Alpha : - Alpha;
		Alpha	*= g_Parameters.SVM_C;
		NegInst.Score_SVM	+= Alpha * g_Parameters.Bias_Neg * g_Parameters.Bias_Neg;
	}
	NegInst.Score_KKT	= NegInst.Score_SVM + NegInst.Score_Loss;
	return	1;
}

int CDetectModel::SMO_LSVM(CDataSet &DataSet, int MaxIter_SMO)
{
	int			MaxIter_KKT		= g_Parameters.Max_KKT_Iter;
	double		Score_Loss		= 1.0f;
	double		Epsilon_SMO		= g_Parameters.SMO_epsilon;
	double		Epsilon_Rate	= 10.0;
	int			Itr, InstCnt;
	int			SetID, InstID;
	int			CurIdx;
	int			ret;

	{
		g_Parameters.Iter_CCCP++;
		g_Parameters.outCCCP<<"************************************ ";
		g_Parameters.outCCCP<<setw(4)<<g_Parameters.Iter_CCCP;
		g_Parameters.outCCCP<<" ************************************"<<endl;
	}

	// run SMO in current dataset
	g_Parameters.timer2.Start( );
	ret		= SMO(Epsilon_SMO);
	g_Parameters.timer2.Stop( );
	Itr		= 0;
	ret		= PrintSVM_CCCP(Itr, 0, g_Parameters.timer2.GetElpTime( ));

	while (Itr < MaxIter_SMO)
	{
		Itr		+= 1;
		ret		= NegInst.Init((*this));
		InstCnt	= 0;
		NegInst.Score_SVM	= Bias_Neg * g_Parameters.Bias_Neg;
		NegInst.Score_KKT	= Bias_Neg * g_Parameters.Bias_Neg;

		for ( int t = 0; t < DataSet.NegCnt; t++ )
		{
			// infer negative image and get instances which violate the KKT condition
			g_Parameters.timer4.Start( );
			CurIdx	= DataSet.NegIdx[t];
			ret		= GetTopInsts(DataSet.NegSet[CurIdx], NegInst.Score_KKT - Score_Loss, InstCnt);
			g_Parameters.timer4.Stop( );

			cout.precision(2);
			cout<<setw(6)<<InstCnt<<" - "<<setw(6)<<t+1<<" - "<<setw(6)<<DataSet.NegCnt;
			cout<<" - "<<setw(12)<<g_Parameters.timer4.GetElpTime( )<<"\r";
			if ((InstCnt < g_Parameters.MaxPerInstCnt) && (t + 1 < DataSet.NegCnt))
			{
				continue;
			}

			g_Parameters.timer3.Start( );
			for ( int p = 0; p < InstCnt; p++ )
			{
				// whether the instance is in the set of support vector or not
				ret	= InSuppVecSet(InstSet[p], -1, SetID, InstID);
				if ((InstID >= 0) || (InstID == -2))
				{
					continue;
				}

				// add to the set of support vector
				if (SetID < 0)
				{
					SetID		= InstSetCnt;
					InstSetCnt	+= 1;
					ret			= AddInstToSet(SetID, NegInst, true);
					SuppInstSet[SetID].DataID	= InstSet[p].DataID;
					SuppInstSet[SetID].Label	= -1;
					SuppInstSet[SetID].Lvl		= InstSet[p].Lvl;
					SuppInstSet[SetID].PosX		= InstSet[p].PosX;
					SuppInstSet[SetID].PosY		= InstSet[p].PosY;
					SuppInstSet[SetID].ViewID	= InstSet[p].ViewID;
					SuppInstSet[SetID].SymID	= InstSet[p].SymID;
					SuppInstSet[SetID].RIdx		= -1;
					SuppInstSet[SetID].Use		= 1;
				}

				ret	= AddInstToSet(SetID, InstSet[p], true);
			}
			g_Parameters.timer3.Stop( );
			cout<<"*"<<"\r";

			// run SMO
			g_Parameters.timer2.Start( );
			ret	= SMO(Epsilon_SMO);
			g_Parameters.timer2.Stop( );

			// run SMO with high precision at the end of CCCP
			if (t + 1 == DataSet.NegCnt)
			{
				g_Parameters.timer2.Start( );
				ret	= SMO(Epsilon_SMO / Epsilon_Rate);
				g_Parameters.timer2.Stop( );
			}

			// remove all redundant instances (weight = 0) 
			g_Parameters.timer5.Start( );
			ret	= RemoveNonSuppVec( );
			g_Parameters.timer5.Stop( );

			// print logs
			InstCnt	= 0;
			NegInst.Score_SVM	= Bias_Neg * g_Parameters.Bias_Neg;
			NegInst.Score_KKT	= Bias_Neg * g_Parameters.Bias_Neg;
			ret	= PrintSVM_CCCP(Itr, t + 1, g_Parameters.timer2.GetElpTime( ));
		}
	}

	// save the model after one CCCp
	{
		char	fname[MAX_PATH];
		sprintf_s(fname, MAX_PATH, "DetModel\\CCCP-%04d.txt", g_Parameters.Iter_CCCP);
		ret	= GetDetThres( );
		ret	= WriteModel(fname);
	}

	cout<<"************************************************************************"<<endl;
	return	1;
}

// SMO
int CDetectModel::SMO(double Epsilon)
{
	int			Itr_KKT	= g_Parameters.Max_KKT_Iter;
	double		delta, PreSre, CurSre;
	double		SS, S1, S2, S3;
	int			SetID, Sym1, Sym2, Idx1, Idx2;
	int			IterCnt, SymID, Idx;
	int			ret;

	SetID	= -1;
	IterCnt	= 0;
	CurSre	= 0;
	PreSre	= GetOptScoreByAlpha( );
	ret		= MakeSVMScores( );
	ret		= GetKKTPair(SetID, Sym1, Sym2, Idx1, Idx2);

	// SMO
	while (SetID >= 0)
	{
		S1		= GetKernelScore(Sym1, Sym1, Idx1, Idx1);
		S2		= GetKernelScore(Sym2, Sym2, Idx2, Idx2);
		S3		= GetKernelScore(Sym1, Sym2, Idx1, Idx2);
		SS		= S1 + S2 - 2 * S3;
		ret		= GetDelta(SetID, Sym1, Sym2, Idx1, Idx2, delta, SS);
		ret		= SetKKTScore(SetID, Sym1, Sym2, Idx1, Idx2, delta);
		ret		= GetKKTPair(SetID, Sym1, Sym2, Idx1, Idx2);
		IterCnt	+= 1;

		if (IterCnt	% Itr_KKT == 0)
		{
			CurSre	= GetOptScoreByAlpha( );
			g_Parameters.outCCCP<<setw(8)<<SetID;
			g_Parameters.outCCCP<<setw(8)<<IterCnt;
			g_Parameters.outCCCP.precision(3);
			g_Parameters.outCCCP<<setw(12)<<fixed<<CurSre;
			g_Parameters.outCCCP<<setw(12)<<fixed<<PreSre;
			g_Parameters.outCCCP<<endl;
			if (fabs(CurSre - PreSre) < Epsilon)
			{
				break;
			}
			PreSre	= CurSre;
		}
	}

	// Set the score of support vectors
	NegInst.Score_KKT	= NegInst.Score_SVM + NegInst.Score_Loss;
	for ( int t = 0; t < InstSetCnt; t++ )
	{
		SymID	= SuppInstSet[t].SymID;
		for ( int p = 0; p < (int) SuppInstSet[t].InstIdx.size( ); p++ )
		{
			Idx	= SuppInstSet[t].InstIdx[p];
			SuppInsts[SymID][Idx].Alpha		= InstAlpha[SymID][Idx];
			SuppInsts[SymID][Idx].Score_KKT	= InstSres[SymID][Idx];
			SuppInsts[SymID][Idx].Score_SVM	= InstSres[SymID][Idx] - SuppInsts[SymID][Idx].Score_Loss;
		}
	}

	// get the model for inference
	ret	= GetInfVer( );
	return	1;
}

// find a pair of instances who violate KKT condition
int CDetectModel::GetKKTPair(int &SetID, int &Sym1, int &Sym2, int &Idx1, int &Idx2)
{
	double		MaxDiff;
	double		Score, Alpha;
	int			Index;

	MaxDiff	= g_Parameters.Diff_KKT;
	SetID	= -1;
	for ( int t = 0; t < InstSetCnt; t++ )
	{
		CSVMInstSet		&CurSet	= SuppInstSet[t];
		int				SymID	= CurSet.SymID;
		int				Cnt		= (int) SuppInstSet[t].InstIdx.size( );
		double			MinV, MaxV;
		int				MinI, MaxI;

		MaxV	= NegInst.Score_SVM + Score_Neg[t];
		MaxI	= -1;
		MinV	= (Alpha_Neg[t] > 0) ? MaxV : FLT_MAX;
		MinI	= -1;

		for ( int p = 0; p < Cnt; p++ )
		{
			Index	= SuppInstSet[t].InstIdx[p];
			Score	= InstSres[SymID][Index];
			if (Score > MaxV)
			{
				MaxV	= Score;
				MaxI	= Index;
				continue;
			}

			Alpha	= InstAlpha[SymID][Index];
			if ((Score < MinV) && (Alpha > 0))
			{
				MinV	= Score;
				MinI	= Index;
			}
		}

		if (MaxV - MinV > MaxDiff)
		{
			MaxDiff	= MaxV - MinV;
			SetID	= t;
			Sym1	= SymID;
			Sym2	= SymID;
			Idx1	= MinI;
			Idx2	= MaxI;
		}
	}

	Sym1	= (Idx1 < 0) ? -1 : Sym1;
	Sym2	= (Idx2 < 0) ? -1 : Sym2;
	return	1;
}

// Get move-step for this update
int CDetectModel::GetDelta(int SetID, int Sym1, int Sym2, int Idx1, int Idx2, double &delta, double KerSre)
{
	double	s1	= (Idx1 >= 0) ? InstSres[Sym1][Idx1] : Score_Neg[SetID] + NegInst.Score_SVM;
	double	s2	= (Idx2 >= 0) ? InstSres[Sym2][Idx2] : Score_Neg[SetID] + NegInst.Score_SVM;
	double	a	= s1 - s2;
	double	b	= g_Parameters.SVM_C * KerSre;
	double	c	= (Idx1 >= 0) ? InstAlpha[Sym1][Idx1] : Alpha_Neg[SetID];
	double	d	= (Idx2 >= 0) ? InstAlpha[Sym2][Idx2] : Alpha_Neg[SetID];
	delta		= __max(-c, __min(d, a / b));

	if (Idx1 >= 0)
	{
		InstAlpha[Sym1][Idx1]	+= delta;
	}
	else
	{
		Alpha_Neg[SetID]	+= delta;
	}

	if (Idx2 >= 0)
	{
		InstAlpha[Sym2][Idx2]	-= delta;
	}
	else
	{
		Alpha_Neg[SetID]	-= delta;
	}

	delta	*= g_Parameters.SVM_C;
	return	1;
}

// calculate the objective score
double CDetectModel::GetKernelScore(int Sym1, int Sym2, int Index1, int Index2)
{
	if (Sym1 != Sym2)
	{
		return	0.0;
	}

	if (Sym1 < 0)
	{
		return	g_Parameters.Bias_Neg * g_Parameters.Bias_Neg;
	}

	int		MaxInstCnt	= g_Parameters.MaxViewInstCnt;
	int		Index		= Index1 * MaxInstCnt + Index2;

	return	KerSre[Sym1][Index];
}

// // calculate score of objective function
double CDetectModel::GetOptScoreByAlpha( )
{
	double	S, S1, S2;

	S1	= 0;
	S2	= 0;
	for ( int t1 = 0; t1 < InstSetCnt; t1++ )
	{
		for ( int p1 = -1; p1 < (int) SuppInstSet[t1].InstIdx.size( ); p1++ )
		{
			int			Idx11	= (p1 < 0) ? - 1 : SuppInstSet[t1].InstIdx[p1];
			int			Idx12	= (SuppInstSet[t1].RIdx < 0) ? -1 : SuppInstSet[t1].InstIdx[SuppInstSet[t1].RIdx];
			int			Sym11	= (p1 < 0) ? - 1 : SuppInstSet[t1].SymID;
			int			Sym12	= (SuppInstSet[t1].RIdx < 0) ? - 1 : SuppInstSet[t1].SymID;
			double		Alpha1	= (Idx11 >= 0) ? InstAlpha[Sym11][Idx11] : Alpha_Neg[t1];
			double		LosSre	= (p1 < 0) ? Score_Neg[t1] : SuppInsts[Sym11][Idx11].Score_Loss;

			// part 1
			S1	+= Alpha1 * LosSre;

			// part 2
			for ( int t2 = 0; t2 < InstSetCnt; t2++ )
			{
				for ( int p2 = -1; p2 < (int) SuppInstSet[t2].InstIdx.size( ); p2++ )
				{
					int			Idx21	= (p2 < 0) ? - 1 : SuppInstSet[t2].InstIdx[p2];
					int			Idx22	= (SuppInstSet[t2].RIdx < 0) ? -1 : SuppInstSet[t2].InstIdx[SuppInstSet[t2].RIdx];
					int			Sym21	= (p2 < 0) ? - 1 : SuppInstSet[t2].SymID;
					int			Sym22	= (SuppInstSet[t2].RIdx < 0) ? - 1 : SuppInstSet[t2].SymID;
					double		Alpha2	= (Idx21 >= 0) ? InstAlpha[Sym21][Idx21] : Alpha_Neg[t2];

					S	= 0;
					S	+= GetKernelScore(Sym11, Sym21, Idx11, Idx21);
					S	+= GetKernelScore(Sym12, Sym22, Idx12, Idx22);
					S	-= GetKernelScore(Sym12, Sym21, Idx12, Idx21);
					S	-= GetKernelScore(Sym11, Sym22, Idx11, Idx22);
					S	*= Alpha1 * Alpha2;
					S	*= g_Parameters.SVM_C * 0.5f;
					S2	-= S;
				}
			}
		}
	}

	return	S1 + S2;
}

// Update the scores in SMO stage
int CDetectModel::SetKKTScore(int SetID, int Sym1, int Sym2, int Idx1, int Idx2, double Delta)
{
	int		MaxInstCnt	= g_Parameters.MaxViewInstCnt;
	double	*pKer1, *pKer2;
	double	Score;

	// one instance is negative
	if ((Sym1 < 0) && (Sym2 >= 0))
	{
		Score	= Delta * g_Parameters.Bias_Neg * g_Parameters.Bias_Neg;
		NegInst.Score_SVM	-= Score;

		pKer2	= KerSre[Sym2] + Idx2 * MaxInstCnt;
		for ( int t = 0; t < ViewInstCnt[Sym2]; t++ )
		{
			Score	= Delta * pKer2[t];
			InstSres[Sym2][t]	+= Score;
		}
	}

	// one instance is negative
	if ((Sym1 >= 0) && (Sym2 < 0))
	{
		Score	= Delta * g_Parameters.Bias_Neg * g_Parameters.Bias_Neg;
		NegInst.Score_SVM	+= Score;

		pKer1	= KerSre[Sym1] + Idx1 * MaxInstCnt;
		for ( int t = 0; t < ViewInstCnt[Sym1]; t++ )
		{
			Score	= Delta * pKer1[t];
			InstSres[Sym1][t]	-= Score;
		}
	}

	// two positive instances
	if ((Sym1 >= 0) && (Sym2 >= 0))
	{
		pKer1	= KerSre[Sym1] + Idx1 * MaxInstCnt;
		pKer2	= KerSre[Sym2] + Idx2 * MaxInstCnt;
		for ( int t = 0; t < ViewInstCnt[Sym2]; t++ )
		{
			Score	= Delta * (pKer2[t] - pKer1[t]);
			InstSres[Sym2][t]	+= Score;
		}
	}

	return	1;
}

// Judge whether a instance is in the set
int CDetectModel::InSuppVecSet(CSVMInst &Inst, int Label, int &SetID, int &InstID)
{
	SetID	= -1;
	InstID	= -1;

	// No memory for the instance
	if ((ViewInstCnt[Inst.SymID] == g_Parameters.MaxViewInstCnt) && (NoUseInsts[Inst.SymID].size( ) == 0))
	{
		InstID	= -2;
		return	1;
	}

	for ( int t = 0; t < InstSetCnt; t++ )
	{
		CSVMInstSet		&CurSet	= SuppInstSet[t];
		int				SymID	= CurSet.SymID;

		// whether belong to same data (same patch)
		if ((CurSet.Label != Label) || (CurSet.DataID != Inst.DataID) || (SymID != Inst.SymID) || 
			(CurSet.PosX != Inst.PosX) || (CurSet.PosY != Inst.PosY) || (CurSet.Lvl != Inst.Lvl))
		{
			continue;
		}

		// compare the postions of all nodes
		SetID	= t;
		for ( int s = 0; s < (int) CurSet.InstIdx.size( ); s++ )
		{
			int			Index		= CurSet.InstIdx[s];
			CSVMInst	&CurInst	= SuppInsts[SymID][Index];
			bool		ok			= true;

			if (Inst.ViewID != CurInst.ViewID)
			{
				ok	= false;
				break;
			}

			for ( int p = 0; p < NodeCnt * 2; p++ )
			{
				if (Inst.NodeXY[p] != CurInst.NodeXY[p])
				{
					ok	= false;
					break;
				}
			}

			if (ok)
			{
				InstID	= s;
				return	1;
			}
		}
	}

	return	1;
}

// Remove all redundant instances (alpha = 0)
int CDetectModel::RemoveNonSuppVec( )
{
	vector<int>			NonSet;
	vector<int>			NonSetIdx(InstSetCnt, 0);
	double				SumSre, Alpha;
	int					SymID, Index, Cnt;
	int					p1, p2;
	int					ret;

	// Remove the set of support vectors that belong to same data
	NonSet.clear( );
	for ( int t = 0; t < InstSetCnt; t++ )
	{
		if (SuppInstSet[t].Label > 0)
		{
			continue;
		}

		SymID	= SuppInstSet[t].SymID;
		Alpha	= Alpha_Neg[t];

		if (fabs(Alpha - 1.0f) < g_Parameters.epsilon)
		{
			NonSetIdx[t]	= 1;
			NonSet.push_back(t);
			for ( int p = 0; p < (int) SuppInstSet[t].InstIdx.size( ); p++ )
			{
				Index	= SuppInstSet[t].InstIdx[p];
				InstAlpha[SymID][Index]	= 0;
				NoUseIndex[SymID][Index]	= 0;
				NoUseInsts[SymID].push_back(Index);
			}
		}
	}

	p1	= InstSetCnt;
	p2	= 0;
	Cnt	= (int) NonSet.size( );
	while ((p2 < (int) NonSet.size( )) && (p1 > NonSet[p2]))
	{
		p1--;
		if (NonSetIdx[p1] == 0)
		{
			ret	= SuppInstSet[NonSet[p2]].SetSuppVecSet(SuppInstSet[p1]);
			Score_Neg[NonSet[p2]]	= Score_Neg[p1];
			Alpha_Neg[NonSet[p2]]	= Alpha_Neg[p1];
			Score_Neg[p1]	= 0;
			Alpha_Neg[p1]	= 0;
			p2++;
		}
	}

	for ( int t = InstSetCnt - Cnt; t < InstSetCnt; t++ )
	{
		Score_Neg[t]	= 0;
		Alpha_Neg[t]	= 0;
		SuppInstSet[t].InstIdx.clear( );
		SuppInstSet[t].RIdx		= -1;
		SuppInstSet[t].Label	= 0;
		SuppInstSet[t].DataID	= -1;
		SuppInstSet[t].ViewID	= -1;
		SuppInstSet[t].Lvl		= -1;
		SuppInstSet[t].PosX		= -1;
		SuppInstSet[t].PosY		= -1;
		SuppInstSet[t].Use		= -1;
	}
	InstSetCnt	-= Cnt;

	// Remove support vectors
	for ( int t = 0; t < InstSetCnt; t++ )
	{
		if (SuppInstSet[t].Label > 0)
		{
			continue;
		}

		SymID	= SuppInstSet[t].SymID;
		for ( int p = 0; p < (int) SuppInstSet[t].InstIdx.size( ); p++ )
		{
			Index	= SuppInstSet[t].InstIdx[p];
			if (SuppInsts[SymID][Index].Alpha < g_Parameters.epsilon)
			{
				InstAlpha[SymID][Index]	= 0;
				NoUseIndex[SymID][Index]	= 0;
				NoUseInsts[SymID].push_back(Index);
				SuppInstSet[t].InstIdx.erase(SuppInstSet[t].InstIdx.begin( ) + p);
				p--;
			}
		}

		SumSre	= Alpha_Neg[t];
		for ( int p = 0; p < (int) SuppInstSet[t].InstIdx.size( ); p++ )
		{
			Index	= SuppInstSet[t].InstIdx[p];
			SumSre	+= SuppInsts[SymID][Index].Alpha;
		}

		Alpha_Neg[t]	/= SumSre;
		for ( int p = 0; p < (int) SuppInstSet[t].InstIdx.size( ); p++ )
		{
			Index	= SuppInstSet[t].InstIdx[p];
			SuppInsts[SymID][Index].Alpha	/= SumSre;
			InstAlpha[SymID][Index]			= SuppInsts[SymID][Index].Alpha;
		}
	}

	ret	= GetInfVer( );
	ret	= UpdateState( );
	return	1;
}

// get threshold of model
int CDetectModel::GetDetThres( )
{
	int		SymID, Index;

	Thres	= FLT_MAX;
	for ( int t = 0; t < InstSetCnt; t++ )
	{
		if (SuppInstSet[t].Label > 0)
		{
			SymID	= SuppInstSet[t].SymID;
			Index	= SuppInstSet[t].InstIdx[SuppInstSet[t].RIdx];
			Thres	= (float) __min(Thres, SuppInsts[SymID][Index].Score_SVM);
		}
	}

	return	1;
}

//*******************************************************************************************************************************************//
//**************************************************************    Common    ***************************************************************//
//*******************************************************************************************************************************************//

// convert a feature of a view to its symmetical view
int CDetectModel::GetSymVal(double *pTar, double *pSrc, int Len, vector<int> &VecIdx, vector<double> &VecRate)
{
	for ( int t = 0; t < Len; t++ )
	{
		int	p	= VecIdx[t];
		pTar[p]	= pSrc[t] * VecRate[t];
	}

	return	1;
}

// dot product
double CDetectModel::GetScore_Lin(double *pV1, double *pV2, int Len)
{
	double	Score	= 0;

	for ( int t = 0; t < Len; t++ )
	{
		Score	+= (*pV1++) * (*pV2++);
	}

	return	Score;
}

//*******************************************************************************************************************************************//
//***************************************************************    DEBUG    ***************************************************************//
//*******************************************************************************************************************************************//

int CDetectModel::PrintSVM_CCCP(int IterTime, int CurID, double TimeCost)
{
	double	OptA	= GetOptScoreByAlpha( );

	cout<<setw(4)<<IterTime<<" --- ";
	cout<<setw(6)<<CurID<<" --- ";
	cout.precision(1);
	cout<<setw(6)<<fixed<<TimeCost<<" --- ";
	cout.precision(3);
	cout<<setw(10)<<fixed<<OptA<<endl;

	g_Parameters.outCCCP<<setw(4)<<IterTime<<" --- ";
	g_Parameters.outCCCP<<setw(6)<<CurID<<" --- ";
	g_Parameters.outCCCP.precision(1);
	g_Parameters.outCCCP<<setw(6)<<fixed<<TimeCost<<" --- ";
	g_Parameters.outCCCP.precision(3);
	g_Parameters.outCCCP<<setw(10)<<fixed<<OptA;
	for ( int t = 0; t < SymCnt; t++ )
	{
		g_Parameters.outCCCP<<" --- "<<setw(6)<<ViewInstCnt[t] - NoUseInsts[t].size( );
	}
	g_Parameters.outCCCP<<endl;

	return	1;
}

int CDetectModel::PrintHostVec(double *pV, int Len, string strFileName)
{
	ofstream	outV(strFileName.c_str( ));

	for ( int t = 0; t < Len; t++ )
	{
		outV.precision(2);
		outV<<setw(10)<<fixed<<pV[t];
		if (t % 10 == 9)
		{
			outV<<endl;
		}
	}

	outV.close( );
	outV.clear( );
	return	1;
}

