#pragma once

#include <vector>
#include <string>
#include <iostream>
#include <fstream>
#include <iomanip>
#include "common.h"
#include "OrModel.h"
#include "AndModel.h"
#include "SVMInst.h"
#include "SVMInstSet.h"
#include "InstTree.h"
#include "Parameters.h"

#include "FilterLib/Timer.h"

using	namespace	std;

class CDetectModel
{
	public:
		CDetectModel(void);
		~CDetectModel(void);

	public:
		// basic variables of a model
		vector<CAndModel>			AndModels;					// And Nodes
		vector<COrModel>			OrModels;					// Or Node - Root
		int							Root;						// Root ID, always = 0
		int							NodeCnt;					// The number of and nodes
		int							ViewCnt;					// The number of views (set to 6)
		int							MaxChdCnt;					// the maximum number of children nodes among all and nodes

		vector<int>					ViewToSym;					// 2 Views are symmetrical.
		vector<int>					ViewToPri;					// the correspondence between one view and its symmetrical view
		int							SymCnt;						// the number of identical views (always = number of views / 2)

		vector<int>					HoG_S;						// the resolution of each level (Root: 0; level 1: 1; level 2: 1)
		vector<int>					HoGStep;					// the ratio of resolutions between two levels
		int							MaxL;						// The height of the model
		int							InitS;						// the maximum resolution (1)
		double						Bias_Neg;					// Bias term of negative data
		double						Thres;						// threshold of model

	// Ver 2 - Train
	public:
		vector<double *>			pW_HoG;						// the weights of HoG features in training stage
		vector<double *>			pW_Shp;						// the weights of shape features in training stage
		vector<double>				Bias;						// bias term of each view
		vector<double *>			pW_HoG_Sym;					// the weights of HoG features of the symmetrical views
		vector<double *>			pW_Shp_Sym;					// the weights of shape features of the symmetrical views
		vector<double>				Bias_Sym;					// Bias term of the symmetrical views

	// SVM Training
	public:
		vector<CSVMInstSet>			SuppInstSet;				// all instances which are belong to a data (a patch) are in the same set.
		vector<vector<CSVMInst> >	SuppInsts;					// all instances
		vector<vector<double> >		InstSres;					// the score of all instances
		vector<vector<double> >		InstAlpha;					// the weight of positive instances
		vector<double>				Alpha_Neg;					// the weight of negative instances
		vector<double>				Score_Neg;					// the score of negative instances
		CSVMInst					NegInst;					// the negative instance

		vector<vector<int> >		NoUseInsts;					// The set of instances that are not used
		vector<vector<int> >		NoUseIndex;
		vector<int>					ViewInstCnt;				// the number of instances of each view
		int							InstSetCnt;					// the number of data (1 data = 1 patch)

	// Common
	public:
		vector<vector<int> >		SymIdx_HoG, SymIdx_Shp;		// the mappings of features between two symmetrical views
		vector<vector<double> >		SymRat_HoG, SymRat_Shp;
		vector<int>					LCnts;						// the number of nodes at each level
		vector<int>					LHoG, LHoG_Sym;				// the dimension of HoG features of each view
		vector<int>					LShp, LShp_Sym;				// the dimension of HoG features of each view

	// Memory in Training
	public:
		CInstTree					InstTree;					// for inference
		vector<CSVMInst>			InstSet;					// the results of inference. 
		vector<double *>			FeaSet_HoG, FeaSet_Shp;		// the features of all support vectors
		vector<double *>			KerSre;						// the similarity matrix

	public:
		int		ReadModel(string strModelName);
		int		WriteModel(string strModelName);

		// Init
		int		InitRoots(CDataSet &DataSet, vector<vector<int> > &AspIdx);		// Initial model only with root node
		int		InitParts(int MaxL, int XCnt, int YCnt);						// initial model of 3 levels
		int		MatchPartner( );												// find the symmetrical nodes
		int		GetAreaSize_Min( );												// the minimum area of view
		int		GetAreaSize_Max( );												// the maximum area of view

		// Train
		int		GetNodeInfo( );													// calculate some information of nodes
		int		GetSymIdx( );													// calculate the mappings of features between two symmetrical views

		int		TrainSVM(CDataSet &DataSet);									// train
		int		AllocMemory(CDataSet &DataSet);									// alloc memory for training
		int		FreeMemory(CDataSet &DataSet);									// release memory for training
		int		UpdateState( );													// get a list of free memory

		int		SetPosSet(CDataSet &DataSet, int IterTime);						// Get instances of positive image (maximum overlap or maximum score)
		int		GetTopInsts(CData &Data, double Thres, int &InstCnt);			// Get instances of negative image (score is large than a threshold)
		int		AddInstToSet(int SetID, CSVMInst &CurInst, bool bKerSre);		// add a instance to the set of support instances
		int		GetInfVer( );													// convert concatenated vector to the features of each node 

		int		MakeKerScore(int SymID, int CurIdx);							// calculate the similarity of each instance
		int		MakeSVMScores(bool bShow = true);								// calculate score of each instance in the set of support instances

		int		SMO_LSVM(CDataSet &DataSet, int IterCnt);						// add instances of negative images who violate KKT condition to set, then do the optimization by SMO
		int		SMO(double epsilon);											// SMO
		int		GetKKTPair(int &SetID, int &Sym1, int &Sym2, int &Idx1, int &Idx2);	// find a pair of instances who violate KKT condition
		int		GetDelta(int SetID, int Sym1, int Sym2, int Idx1, int Idx2, double &delta, double KerSre);
		int		SetKKTScore(int SetID, int Sym1, int Sym2, int Idx1, int Idx2, double Delta);	// update the scores in SMO stage
		int		InSuppVecSet(CSVMInst &Inst, int Label, int &SetID, int &InstID);	// judge whether a instance is in the set
		int		RemoveNonSuppVec( );											// remove all redundant instances (alpha = 0)
		int		GetDetThres( );													// get threshold

		double	GetKernelScore(int Sym1, int Sym2, int Idx1, int Idx2);			// get the similarity score from similarity matrix
		double	GetOptScoreByAlpha( );											// calculate score of objective function

		// common
		int		GetSymVal(double *pTar, double *pSrc, int Len, vector<int> &VecIdx, vector<double> &VecRate);	// convert a feature of a view to its symmetical view
		double	GetScore_Lin(double *pV1, double *pV2, int Len);				// dot product
		int		PrintSVM_CCCP(int IterTime, int CurID, double TimeCost);		// debug code
		int		PrintHostVec(double *pV, int Len, string strFileName);			// debug code
};