#pragma once

#include <windows.h>
#include <stdio.h>
#include <math.h>
#include <tchar.h>
#include <assert.h>
#include <typeinfo.h>
#include "TVector.h"

#define	TINY	0.000001f
#define	ESP		0.000001f


template <typename T>
class	TMatrix
{
public:
	int		m_nRow, m_nCol;
	T		**m_pMatrix, *m_pData;	

public:		//Constructor and destructor
	TMatrix()
	{
		m_pMatrix = NULL; m_pData = NULL;
		m_nRow = m_nCol = 0;
	}
	TMatrix(int nRow, int nCol)
	{
		m_pMatrix = NULL; m_pData = NULL;
		m_nRow = m_nCol = 0;
		alloc(nRow, nCol);
	}
	TMatrix(T *pData, int nRow, int nCol)
	{
		int	i, j;
		m_pMatrix = NULL; m_pData = NULL; 
		m_nRow = m_nCol = 0;
		alloc(nRow, nCol);
		for(i = 0; i < nRow; i++)
		{
			for(j = 0; j < nCol; j++)
				m_pMatrix[i][j] = pData[j];
			pData += nCol;
		}
	}		
	TMatrix(const TMatrix &A)
	{
		int	i, j;
		m_pMatrix = NULL; m_pData = NULL;
		m_nRow = m_nCol = 0;
		alloc(A.m_nRow, A.m_nCol);
		for(i = 0; i < m_nRow; i++)
			for(j = 0; j < m_nCol; j++)
				m_pMatrix[i][j] = A.m_pMatrix[i][j];
	}
	TMatrix(TVector<T> &v)
	{
		m_pMatrix = NULL; m_pData = NULL;
		m_nRow = m_nCol = 0;
		alloc(v.size(), 1);
		for(int i = 0; i < m_nRow; i++)
			m_pMatrix[i][0] = v[i];
	}
	~TMatrix()
	{
		free();
	}
	void set(int row, int col, T** pMatrix, T* pData)	//writing for copy a dense matrix defined in sparse matrix class to this class
	{
		m_nRow = row;
		m_nCol = col;
		m_pMatrix = pMatrix;
		m_pData = pData;
	}
	int load(float *pData, int nRow, int nCol)
	{
		int	i, j;
		if(alloc(nRow, nCol) == 0)
			return 0;
		for(i = 0; i < nRow; i++)
		{
			for(j = 0; j < nCol; j++)
				m_pMatrix[i][j] = (T) pData[j];
			pData += nCol;
		}
		return 1;
	}
	int load(float **pData, int nRow, int nCol)
	{
		int	i, j;
		if(alloc(nRow, nCol) == 0)
			return 0;
		for(i = 0; i < nRow; i++)
			for(j = 0; j < nCol; j++)
				m_pMatrix[i][j] = (T) pData[i][j];			
		return 1;
	}
	int load(double *pData, int nRow, int nCol)
	{
		int	i, j;
		if(alloc(nRow, nCol) == 0)
			return 0;
		for(i = 0; i < nRow; i++)
		{
			for(j = 0; j < nCol; j++)
				m_pMatrix[i][j] = (T) pData[j];
			pData += nCol;
		}
		return 1;
	}
	int load(double **pData, int nRow, int nCol)
	{
		int	i, j;
		alloc(nRow, nCol);
		for(i = 0; i < nRow; i++)
			for(j = 0; j < nCol; j++)
				m_pMatrix[i][j] = (T) pData[i][j];	
		return 1;
	}
	int load(BYTE **pData, int nRow, int nCol)
	{
		int	i, j;
		alloc(nRow, nCol);
		for(i = 0; i < nRow; i++)
			for(j = 0; j < nCol; j++)
				m_pMatrix[i][j] = (T) pData[i][j];	
		return 1;
	}
	int load(BYTE *pData, int nRow, int nCol)
	{
		int	i, j;
		if(alloc(nRow, nCol) == 0)
			return 0;
		for(i = 0; i < nRow; i++)
		{
			for(j = 0; j < nCol; j++)
				m_pMatrix[i][j] = (T) pData[j];
			pData += nCol;
		}
		return 1;
	}
	int load(short **pData, int nRow, int nCol)
	{
		int	i, j;
		alloc(nRow, nCol);
		for(i = 0; i < nRow; i++)
			for(j = 0; j < nCol; j++)
				m_pMatrix[i][j] = (T) pData[i][j];	
		return 1;
	}
	int load(short *pData, int nRow, int nCol)
	{
		int	i, j;
		if(alloc(nRow, nCol) == 0)
			return 0;
		for(i = 0; i < nRow; i++)
		{
			for(j = 0; j < nCol; j++)
				m_pMatrix[i][j] = (T) pData[j];
			pData += nCol;
		}
		return 1;
	}	
	//types support: BIN, TXT, SVM (svm light format), MAT, SPARSE (col, row, value)
	int load(const TCHAR *szText, const TCHAR *type = _T("BIN"))
	{
		TCHAR	*pBuffer, *pValue;
		TCHAR	*stopstr;
		HANDLE	hFile;
		int		i, j, size;
		DWORD	dwRead, nLength;

		hFile = CreateFile(szText, GENERIC_READ, FILE_SHARE_READ, NULL, OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, NULL);
		if(hFile == INVALID_HANDLE_VALUE)
			return 0;
		nLength = GetFileSize(hFile, NULL);
		pBuffer = (TCHAR *) GlobalAlloc(GMEM_FIXED, nLength + sizeof(TCHAR));
		if(ReadFile(hFile, pBuffer, nLength, &dwRead, NULL) == 0 || dwRead != nLength)
		{
			CloseHandle(hFile);
			GlobalFree(pBuffer);
			return 0;
		}
		CloseHandle(hFile);		
		if(_tcsicmp(type, _T("BIN")) == 0)			//default format
		{
			m_nRow = *((int *) ((PBYTE) pBuffer));			
			m_nCol = *((int *) ((PBYTE) pBuffer + sizeof(int)));
			size = *((int *) ((PBYTE) pBuffer + sizeof(int) * 2));			
			assert(size == sizeof(T));
			resize(m_nRow, m_nCol);
			memcpy(m_pData, (PBYTE) pBuffer + sizeof(int) * 3, size * m_nCol * m_nRow);
		}
		else if(_tcsicmp(type, _T("TXT")) == 0)
		{
			pBuffer[nLength / sizeof(TCHAR)] = 0;
			i = m_nRow = 0;
			while(pBuffer[i])
			{
				if(pBuffer[i] == _T('\n'))
					m_nRow++;
				i++;
			}
			i--;
			if(pBuffer[i] != _T('\n'))
				m_nRow++;
			i = 0; m_nCol = 0;
			while(pBuffer[i] && pBuffer[i] != _T('\n'))
			{
				if(pBuffer[i] == _T(',') || pBuffer[i] == _T(' ') || pBuffer[i] == _T('\t') || pBuffer[i] == _T(';'))
					m_nCol++;
				i++;
			}
			i--;
			if(pBuffer[i] != _T(',') && pBuffer[i] != _T(' ') && pBuffer[i] != _T('\t') && pBuffer[i] != _T(';'))
				m_nCol++;
			resize(m_nRow, m_nCol);
			for(i = 0; i < m_nRow; i++)
			{
				for(j = 0; j < m_nCol; j++)
				{
					if(i != 0 || j != 0)
						pValue = _tcstok(NULL, _T("\r\n ,;\t"));
					else
						pValue = _tcstok(pBuffer, _T("\r\n ,;\t"));
					if(typeid(T) == typeid(double))
						m_pMatrix[i][j] = (T) _tcstod(pValue, &stopstr);
					else if(typeid(T) == typeid(float))					//float
						m_pMatrix[i][j] = (T) _tcstod(pValue, &stopstr);
					else
						m_pMatrix[i][j] = (T) _ttoi(pValue);
				}
			}
		}
		else if(_tcsicmp(type, _T("MAT")) == 0)
		{
		}
		else if(_tcsicmp(type, _T("SVM")) == 0)	//svm light format
		{
		}
		else if(_tcsicmp(type, _T("SPT")) == 0)	//sparse matrix in text format
		{

		}
		else if(_tcsicmp(type, _T("SPB")) == 0)	//sparse matrix in binary format
		{
			
		}
		else	//unknown format
		{
			GlobalFree(pBuffer);
			return 0;
		}				
		GlobalFree(pBuffer);
		return 1;
	}	
	int save(const TCHAR *szText, const TCHAR *type = _T("BIN"))
	{
		HANDLE	hFile;
		int		i, j;
		TCHAR	buffer[1024];
		DWORD	dwWritten;

		hFile = CreateFile(szText, GENERIC_WRITE, FILE_SHARE_READ, NULL, CREATE_ALWAYS, FILE_ATTRIBUTE_NORMAL, NULL);
		if(hFile == INVALID_HANDLE_VALUE)
			return 0;
		if(_tcsicmp(type, _T("BIN")) == 0)		//save as binary format to keep numerical pricision of numbers
		{
			WriteFile(hFile, &m_nRow, sizeof(m_nRow), &dwWritten, NULL);
			WriteFile(hFile, &m_nCol, sizeof(m_nCol), &dwWritten, NULL);
			i = sizeof(T);		//size of T
			WriteFile(hFile, &i, sizeof(i), &dwWritten, NULL);
			BYTE *pBuffer	= (BYTE *) GlobalAlloc(GMEM_FIXED, sizeof(T) * m_nCol);
			if(pBuffer == NULL)
			{
				CloseHandle(hFile);
				return -1;
			}
			for(i = 0; i < m_nRow; i++)
			{
				memcpy(pBuffer, m_pMatrix[i], sizeof(T) * m_nCol);
				if(!WriteFile(hFile, pBuffer, sizeof(T) * m_nCol, &dwWritten, NULL) || dwWritten != sizeof(T) * m_nCol)
				{
					CloseHandle(hFile);
					GlobalFree(pBuffer);
					return -1;
				}
			}			
			GlobalFree(pBuffer);
		}
		else if(_tcsicmp(type, _T("TXT")) == 0)
		{
			for(i = 0; i < m_nRow; i++)
			{
				for(j = 0; j < m_nCol; j++)
				{
					if(typeid(T) == typeid(double))		// double and
						_stprintf(buffer, _T("%10.4f"), m_pMatrix[i][j]);
					else if(typeid(T) == typeid(float)) // float
						_stprintf(buffer, _T("%10.4f"), m_pMatrix[i][j]);
					else 
						_stprintf(buffer, _T("%10d"), m_pMatrix[i][j]);					
					WriteFile(hFile, buffer, (int) _tcslen(buffer) * sizeof(TCHAR), &dwWritten, NULL);
				}
				_stprintf(buffer, _T("\n"));
				WriteFile(hFile, buffer, (int) _tcslen(buffer) * sizeof(TCHAR), &dwWritten, NULL);
			}
		}
		else if(_tcsicmp(type, _T("MAT")) == 0)	//default name is "DATA"
		{
		}
		else if(_tcsicmp(type, _T("SVM")) == 0)
		{
		}
		else if(_tcsicmp(type, _T("SPT")) == 0)	//sparse matrix in text format
		{
			_stprintf(buffer, _T("%d %d\n"), m_nRow, m_nCol);
			WriteFile(hFile, buffer, (int) _tcslen(buffer) * sizeof(TCHAR), &dwWritten, NULL);
			for(i = 0; i < m_nRow; i++)
			{
				for(j = 0; j < m_nCol; j++)
				{
					if(m_pMatrix[i][j] != 0)
					{
						if(typeid(T) == typeid(double))		// double and
							_stprintf(buffer, _T("%d %d %.6g\n"), i, j, m_pMatrix[i][j]);
						else if(typeid(T) == typeid(float))		// float and
							_stprintf(buffer, _T("%d %d %.5f\n"), i, j, m_pMatrix[i][j]);
						else
							_stprintf(buffer, _T("%d %d %d\n"), i, j, m_pMatrix[i][j]);
						WriteFile(hFile, buffer, (int) _tcslen(buffer) * sizeof(TCHAR), &dwWritten, NULL);
					}
				}
			}
		}
		else if(_tcsicmp(type, _T("SPB")) == 0)	//sparse matrix in binary format
		{
			WriteFile(hFile, &m_nRow, sizeof(m_nRow), &dwWritten, NULL);
			WriteFile(hFile, &m_nCol, sizeof(m_nCol), &dwWritten, NULL);
			BYTE *pBuffer	= (BYTE *) GlobalAlloc(GMEM_FIXED, (sizeof(int) * 2 + sizeof(T)));
			if(pBuffer == NULL)
			{
				CloseHandle(hFile);
				return -1;
			}
			for(i = 0; i < m_nRow; i++)
			{
				for(j = 0; j < m_nCol; j++)
				{
					if(m_pMatrix[i][j] != 0)
					{
						memcpy(pBuffer, &i, sizeof(int));
						memcpy(pBuffer + sizeof(int), &j, sizeof(int));
						memcpy(pBuffer + sizeof(int) * 2, &(m_pMatrix[i][j]), sizeof(T));						
						if(!WriteFile(hFile, pBuffer, sizeof(T) + sizeof(int) * 2, &dwWritten, NULL) || dwWritten != sizeof(T) + sizeof(int) * 2)
						{
							CloseHandle(hFile);
							GlobalFree(pBuffer);
							return -1;
						}
					}
				}
			}
			GlobalFree(pBuffer);
		}
		else		//unknown format
		{
			CloseHandle(hFile);
			return 0;
		}
		CloseHandle(hFile);
		return 1;
	}
	void print()
	{
		int	i, j;
		for(i = 0; i < m_nRow; i++)
		{
			for(j = 0; j < m_nCol; j++)
			{
				if(typeid(T) == typeid(double))			
					_tprintf(_T("%.6g "), m_pMatrix[i][j]);
				else if(typeid(T) == typeid(float))
					_tprintf(_T("%.5f "), m_pMatrix[i][j]);
				else
					_tprintf(_T("%d "), m_pMatrix[i][j]);
			}
			_tprintf(_T("\n"));
		}
		_tprintf(_T("\n"));
	}

private:
	int sign(T v)
	{
		if(v >= 0)
			return 1;		
		return -1;
	}
	T sign(T a, T b)
	{
		return (T) fabs(a) * (T) sign(b);
	}
	void swap(T &a, T &b)
	{
		T	temp	= a; a = b; b = temp;
	}
	T pythag(T a, T b)	//reutrn sqrt(a*a+b*b);
	{
		T	absa, absb;
		absa = (T) fabs(a);
		absb = (T) fabs(b);
		if(absa > absb)
			return absa * (T) sqrt(1.0 + absb * absb / absa / absa);
		return absb == 0 ? 0 : absb * (T) sqrt(1.0 + absa * absa / absb / absb);
	}
public:
	T ** matrix()
	{
		return m_pMatrix;
	}
	int row()
	{
		return m_nRow;
	}
	TVector<T> row(int n)
	{
		TVector<T>	B(m_nCol);
		int			i;
		for(i = 0; i < m_nCol; i++)
			B[i] = m_pMatrix[n][i];
		return B;
	}
	int col()
	{
		return m_nCol;
	}
	TVector<T> col(int n)
	{
		TVector<T>	B(m_nRow);
		int			i;
		for(i = 0; i < m_nRow; i++)
			B[i] = m_pMatrix[i][n];
		return B;
	}
	int alloc(int nRow = 0, int nCol = 0)
	{
		int		nSize, i;
		T		*pData;		
		if(nRow == 0 && nCol == 0)
		{
			m_pMatrix = NULL;
			m_pData = NULL;
			m_nRow = m_nCol = NULL;
			return 0;
		}
		free();
		m_nRow = nRow; m_nCol = nCol;
		m_pMatrix = (T * *) GlobalAlloc(GMEM_FIXED | GMEM_ZEROINIT, m_nRow * sizeof(T * *));
		if(m_pMatrix == NULL)
			return 0;
		nSize = nRow * nCol * sizeof(T);
		m_pData = (T *) GlobalAlloc(GMEM_FIXED | GMEM_ZEROINIT, nSize);
		if(m_pData == NULL)
		{
			GlobalFree(m_pMatrix);
			return 0;
		}
		pData = m_pData;
		for(i = 0; i < m_nRow; i++)
		{
			m_pMatrix[i] = pData;
			pData += m_nCol;
		}
		return nSize;
	}
	void free()
	{
		if(m_pMatrix)
			GlobalFree(m_pMatrix);
		m_pMatrix = NULL;
		if(m_pData)
			GlobalFree(m_pData);		
		m_pData = NULL;
		m_nRow = m_nCol = 0;
	}
	int resize(int nRow, int nCol)
	{
		return alloc(nRow, nCol);
	}
	TMatrix * clone()
	{
		int		i, size;
		TMatrix	*P	= new TMatrix(m_nRow, m_nCol);		
		size = m_nCol * sizeof(T);
		for(i = 0; i < m_nRow; i++)
			memcpy(P->m_pMatrix[i], m_pMatrix[i], size);	
		return P;
	}
	void operator =(const TMatrix &A)
	{
		int		i, size;
		T		**pMatrix;
		resize(A.m_nRow, A.m_nCol);
		size = m_nCol * sizeof(T);
		pMatrix = A.m_pMatrix;
		for(i = 0; i < m_nRow; i++)
			memcpy(m_pMatrix[i], pMatrix[i], size);
	}
	void operator =(const T k)
	{
		int	i, j;		
		for(i = 0; i < m_nRow; i++)
			for(j = 0; j < m_nCol; j++)
				m_pMatrix[i][j] = k;
	}
	TMatrix<double> Double()		//convert a matrix to double
	{
		int				i, j;
		TMatrix<double>	B;
		B.alloc(m_nRow, m_nCol);		
		for(i = 0; i < m_nRow; i++)
			for(j = 0; j < m_nCol; j++)
				B[i][j] = (double) m_pMatrix[i][j];
		return B;
	}	
	TMatrix<float> Float()			//convert a matrix to float
	{
		int				i, j;
		TMatrix<float>	B;
		B.alloc(m_nRow, m_nCol);		
		for(i = 0; i < m_nRow; i++)
			for(j = 0; j < m_nCol; j++)
				B[i][j] = (float) m_pMatrix[i][j];
		return B;
	}	
	TMatrix<int> Int()				//convert a matrix to int
	{
		int				i, j;
		TMatrix<int>	B;
		B.alloc(m_nRow, m_nCol);		
		for(i = 0; i < m_nRow; i++)
			for(j = 0; j < m_nCol; j++)
				B[i][j] = (int) m_pMatrix[i][j];
		return B;
	}
	TMatrix operator *(const TMatrix &A)
	{
		int		i, j, k;		
		TMatrix	B	(m_nRow, A.m_nCol);
		T		sum;
		assert(A.m_nRow == m_nCol);
		for(i = 0; i < m_nRow; i++)
		{
			for(j = 0; j < A.m_nCol; j++)
			{
				B.m_pMatrix[i][j] = 0;
				for(k = 0, sum = 0; k < m_nCol; k++)
					B.m_pMatrix[i][j] += m_pMatrix[i][k] * A.m_pMatrix[k][j];
			}
		}
		return B;
	}
	void operator *=(const TMatrix &A)
	{
		TMatrix	B;
		int		i, j, k;
		assert(m_nCol == A.m_nRow);
		B = *this;
		resize(m_nRow, A.m_nCol);
		for(i = 0; i < m_nRow; i++)
		{
			for(j = 0; j < A.m_nCol; j++)
			{
				m_pMatrix[i][j] = 0;
				for(k = 0, sum = 0; k < m_nCol; k++)
					m_pMatrix[i][j] += B.m_pMatrix[i][k] * A.m_pMatrix[k][j];
			}
		}
	}	
	T * operator [](int row)
	{
		if(row < 0 || row >= m_nRow)
			return NULL;
		return m_pMatrix[row];
	}
	TMatrix operator *(T k)
	{
		TMatrix	B	(m_nRow, m_nCol);
		int		i, j;		
		for(i = 0; i < m_nRow; i++)
			for(j = 0; j < m_nCol; j++)
				B.m_pMatrix[i][j] = m_pMatrix[i][j] * k;
		return B;
	}
	TMatrix operator /(T k)
	{
		TMatrix	B	(m_nRow, m_nCol);
		int		i, j;		
		for(i = 0; i < m_nRow; i++)
			for(j = 0; j < m_nCol; j++)
				B.m_pMatrix[i][j] = m_pMatrix[i][j] / k;
		return B;
	}
	void operator *=(T k)
	{
		int	i, j;
		for(i = 0; i < m_nRow; i++)
			for(j = 0; j < m_nCol; j++)
				m_pMatrix[i][j] *= m_pMatrix[i][j];
	}
	TMatrix operator +(TMatrix &A)
	{
		TMatrix	B;
		int		i, j;
		B.resize(m_nRow, m_nCol);
		for(i = 0; i < m_nRow; i++)
			for(j = 0; j < m_nCol; j++)
				B.m_pMatrix[i][j] = m_pMatrix[i][j] + A.m_pMatrix[i][j];
		return B;
	}
	TMatrix operator -(TMatrix &A)
	{
		TMatrix	B;
		int		i, j;
		B.resize(m_nRow, m_nCol);
		for(i = 0; i < m_nRow; i++)
			for(j = 0; j < m_nCol; j++)
				B.m_pMatrix[i][j] = m_pMatrix[i][j] - A.m_pMatrix[i][j];
		return B;
	}
	void operator +=(TMatrix &A)
	{
		int	i, j;		
		for(i = 0; i < m_nRow; i++)
			for(j = 0; j < m_nCol; j++)
				m_pMatrix[i][j] += A.m_pMatrix[i][j];
	}
	void operator -=(TMatrix &A)
	{
		int	i, j;		
		for(i = 0; i < m_nRow; i++)
			for(j = 0; j < m_nCol; j++)
				m_pMatrix[i][j] -= A.m_pMatrix[i][j];
	}
	void operator +=(T k)
	{
		int	i, j;		
		for(i = 0; i < m_nRow; i++)
			for(j = 0; j < m_nCol; j++)
				m_pMatrix[i][j] += k;
	}
	void operator -=(T k)
	{
		int	i, j;		
		for(i = 0; i < m_nRow; i++)
			for(j = 0; j < m_nCol; j++)
				m_pMatrix[i][j] -= k;
	}
	void inscol(T *pCol, int n)	//before n
	{
		TMatrix	B;
		int		i, j;
		assert(n <= m_nCol);
		B = *this;
		resize(m_nRow, m_nCol + 1);
		for(i = 0; i < m_nRow; i++)
		{
			for(j = 0; j < n; j++)
				m_pMatrix[i][j] = B.m_pMatrix[i][j];
			m_pMatrix[i][j] = pCol[i];
			for(j = n + 1; j < m_nCol; j++)
				m_pMatrix[i][j] = B.m_pMatrix[i][j - 1];
		}
	}
	void insrow(T *pRow, int n)
	{
		TMatrix	B;
		int		i, j;
		assert(n <= m_nRow);
		B = *this;
		resize(m_nRow + 1, m_nCol);
		for(i = 0; i < n; i++)
			for(j = 0; j < m_nCol; j++)
				m_pMatrix[i][j] = B.m_pMatrix[i][j];
		for(j = 0; j < m_nCol; j++)
			m_pMatrix[n][j] = pRow[j];
		for(i = n + 1; i < m_nRow; i++)
			for(j = 0; j < m_nCol; j++)
				m_pMatrix[i][j] = B.m_pMatrix[i - 1][j];
	}
	void delcols(int n)
	{
		int		i, j;
		TMatrix	B;
		assert(n < m_nCol);
		B = *this;
		resize(m_nRow, m_nCol - 1);
		for(i = 0; i < m_nRow; i++)
		{
			for(j = 0; j < n; j++)
				m_pMatrix[i][j] = B.m_pMatrix[i][j];
			for(j = n; j < m_nCol; j++)
				m_pMatrix[i][j] = B.m_pMatrix[i][j + 1];
		}
	}
	void delcols(TVector<int>& cols)		//cols is sorted as increasing order
	{
		int				i, j, k;
		TVector<int>	index;
		assert(cols[cols.size()] < m_nCol);
		index.alloc(m_nCol - cols.size());
		for(i = 0, j = 0, k = 0; i < m_nCol; i++)
		{
			if(i == cols[j])
			{
				j++;
				if(j == cols.size())
					j--;
			}
			else
				index[k++] = i;
		}		
		for(i = 0; i < m_nRow; i++)
			for(j = 0; j < index.size(); j++)
				m_pMatrix[i][j] = m_pMatrix[i][index[j]];
		m_nCol = index.size();
		if(m_nCol == 0)
			free();
	}
	void delrows(int n)
	{
		int		i, j;
		TMatrix	B;
		assert(n < m_nRow);
		B = *this;
		resize(m_nRow - 1, m_nCol);
		for(i = 0; i < n; i++)
			for(j = 0; j < m_nCol; j++)
				m_pMatrix[i][j] = B.m_pMatrix[i][j];
		for(i = n; i < m_nRow; i++)
			for(j = 0; j < m_nCol; j++)
				m_pMatrix[i][j] = B.m_pMatrix[i + 1][j];
	}
	void delrows(TVector<int>& rows)	//waste some memory, but not leaked
	{
		int				i, j, k;
		TVector<int>	index;
		assert(rows[rows.size() - 1] < m_nRow);
		index.alloc(m_nRow - rows.size());		
		for(i = 0, j = 0, k = 0; i < m_nRow; i++)
		{
			if(i == rows[j])
			{
				j++;
				if(j == rows.size())
					j--;
			}
			else
				index[k++] = i;
		}		
		for(i = 0; i < k; i++)
			m_pMatrix[i] = m_pMatrix[index[i]];
		m_nRow = index.size();
		if(m_nRow == 0)
			free();
	}
	void setcol(T *pCol, int n)
	{
		int	i;
		assert(n < m_nCol);
		for(i = 0; i < m_nRow; i++)
			m_pMatrix[i][n] = pCol[i];
	}
	void setcol(TVector<T> &v, int n)
	{
		assert(v.size() == m_nRow);
		setcol(v.data(), n);
	}
	void setrow(T *pRow, int n)
	{
		int	i;
		assert(n < m_nRow);
		for(i = 0; i < m_nCol; i++)
			m_pMatrix[n][i] = pRow[i];
	}
	void setrow(TVector<T> &v, int n)
	{
		assert(v.size() == m_nCol);
		setrow(v.data(), n);
	}
	void swap(T *a, T *b)
	{
		T	temp;
		temp = *a; *a = *b; *b = temp;
	}
	void swapcol(int c1, int c2)
	{
		int	i;
		assert(c1 < m_nCol && c2 < m_nCol);
		for(i = 0; i < m_nRow; i++)
			swap(&m_pMatrix[i][c1], &m_pMatrix[i][c2]);
	}
	void swaprow(int r1, int r2)
	{
		int	i, j;
		assert(r1 < m_nRow && r2 < m_nRow);
		for(i = 0; i < m_nCol; i++)
			swap(&m_pMatrix[r1][i], &m_pMatrix[r2][i]);
	}
	TMatrix crop(int r1, int c1, int r2, int c2)	//include row r1 and col c1
	{
		//exclude row r2 and col c2
		TMatrix	B	(r2 - r1, c2 - c1);
		int		i, j;
		for(i = r1; i < r2; i++)
			for(j = c1; j < c2; j++)
				B.m_pMatrix[i - r1][j - c1] = m_pMatrix[i][j];
		return B;
	}
	static TMatrix eye(int n)
	{
		int		i, j;
		TMatrix	B	(n, n);
		for(i = 0; i < n; i++)
		{
			for(j = 0; j < n; j++)
				B.m_pMatrix[i][j] = 0;
			B.m_pMatrix[i][i] = 1;
		}
		return B;
	}
	static TMatrix zeros(int row, int col)
	{
		TMatrix	B	(row, col);
		return B;
	}	
	static TMatrix ones(int row, int col)
	{
		int		i, j;
		TMatrix	B	(row, col);
		for(i = 0; i < row; i++)
			for(j = 0; j < col; j++)
				B.m_pMatrix[i][j] = 1;
		return B;
	}
	static TMatrix * diag(TMatrix &A)		//fetch A's diagonal
	{
		TMatrix	B;
		int		dim	= m_nRow > m_nCol ? m_nCol : m_nRow;
		B.alloc(dim, 1);
		for(i = 0; i < dim; i++)
			B.m_pMatrix[i][0] = A.m_pMatrix[i][i];				
		return B;
	}
	static TMatrix diag(TVector<T> &v)
	{
		int		i;
		TMatrix	B	(v.size(), v.size());				
		for(i = 0; i < B.m_nRow; i++)
			B.m_pMatrix[i][i] = v[i];
		return B;
	}
	static TMatrix diag(T *pData, int row)
	{
		int		i;
		TMatrix	B	(row, row);				
		for(i = 0; i < row; i++)
			B.m_pMatrix[i][i] = pData[i];
		return B;
	}	
	static TMatrix repmat(T *pData, int nLen, int row, int col)	//duplicate a vector
	{
		TMatrix	A	(nLen *row, col);
		int		i, j;
		for(i = 0; i < row; i++)
			for(j = 0; j < nLen; j++)
				memset(A.m_pMatrix[i * row + j], &pData[j], sizeof(T) * col);
		return A;
	}	
	static TMatrix repmat(TMatrix unit, int row, int col)	//duplicate a matrix
	{
		TMatrix	B	(unit.m_nRow *row, unit.m_nCol *col);
		int		i, j, k;
		for(i = 0; i < row; i++)
		{
			for(j = 0; j < col; j++)
			{
				for(k = 0; k < unit.m_nRow; k++)
					memcpy(&(B.m_pMatrix[i * unit.m_nRow + k][j * unit.m_nCol]),
						unit.m_pMatrix[k],
						sizeof(T) * unit.m_nCol);
			}
		}
		return B;
	}
	void rectify()
	{
		int	i, j;
		for(i = 0; i < m_nRow; i++)
		{
			for(j = 0; j < m_nCol; j++)
				if(fabs(m_pMatrix[i][j]) < TINY)
					m_pMatrix[i][j] = 0;
		}
	}
	void trans()		//transport itself
	{
		TMatrix	B;
		B = *this;
		resize(m_nCol, m_nRow);	//col and row will be exchanged
		int	i, j;
		for(i = 0; i < m_nRow; i++)
			for(j = 0; j < m_nCol; j++)
				m_pMatrix[i][j] = B.m_pMatrix[j][i];
	}
	TMatrix operator ~()	//transport
	{
		TMatrix	B	(m_nCol, m_nRow);
		int		i, j;
		for(i = 0; i < m_nRow; i++)
			for(j = 0; j < m_nCol; j++)
				B.m_pMatrix[j][i] = m_pMatrix[i][j];
		return B;
	}
	TMatrix operator !()	//Gaussian-Jordan elimination
	{
		int		i, j, k, row, col, *indexC, *indexR, *piv;
		T		max, **pMatrix, g;	
		TMatrix	inv;

		assert(m_nRow == m_nCol);
		//is diagonal?
		for(i = 0; i < m_nRow; i++)
			for(j = 0; j < m_nCol; j++)
				if(m_pMatrix[i][j] != 0 && i != j)
					break;
		if(i == m_nRow && j == m_nCol)	//is diagonal matrix
		{
			inv.alloc(m_nRow, m_nCol);
			for(i = 0; i < m_nRow; i++)
				inv[i][i] = 1 / m_pMatrix[i][i];
			return inv;
		}
		inv = *this;
		pMatrix = inv.m_pMatrix;			
		indexC = (int *) GlobalAlloc(GMEM_FIXED | GMEM_ZEROINIT, sizeof(int) * m_nRow * 3);
		indexR = indexC + m_nRow;
		piv = indexR + m_nRow;	
		for(i = 0; i < m_nRow; i++)
		{
			max = 0;			//select pivot
			for(j = 0; j < m_nRow; j++)
			{
				if(piv[j] != 1)	//ѡУоͲѡ
				{
					for(k = 0; k < m_nCol; k++)
					{
						if(piv[k] == 0)
						{
							if(max < fabs(pMatrix[j][k]))
							{
								max = (T) fabs(pMatrix[j][k]);
								row = j;
								col = k;
							}
						}
						else if(piv[k] > 1)
						{
							GlobalFree(indexC);
							return pinv(); //singular matrix							
						}
					}
				}
			}
			piv[col]++;
			if(max < TINY)	//singular matrix
			{
				GlobalFree(indexC);
				return pinv(); //singular matrix							
			}
			//¼еı仯, ϺҪ仯
			indexC[i] = col;
			indexR[i] = row;
			//pivotڶԽswap row, 任Ϻ, pMatrix[col][col]pivot
			if(row != col)
				for(j = 0; j < m_nCol; j++)
					swap(pMatrix[row][j], pMatrix[col][j]);
			//һҪ1! Чڵλϵı仯. ൱ڴλ
			max = pMatrix[col][col];
			pMatrix[col][col] = 1.0f;	
			for(j = 0; j < m_nCol; j++)
				pMatrix[col][j] /= max;
			//pInv->Print();
			for(j = 0; j < m_nRow; j++)
			{
				if(j != col)
				{
					g = pMatrix[j][col];
					//һҪ0! Чڵλϵı仯. ൱ڴλ
					pMatrix[j][col] = 0;	
					for(k = 0; k < m_nCol; k++)
						pMatrix[j][k] -= pMatrix[col][k] * g;
				}
			}
		}
		for(i = m_nRow - 1; i >= 0; i--)	//б任任ȥ
		{
			if(indexC[i] != indexR[i])
			{
				col = indexC[i];
				row = indexR[i];
				for(j = 0; j < m_nRow; j++)
					swap(pMatrix[j][row], pMatrix[j][col]);
			}
		}
		GlobalFree(indexC);
		return inv;
	}
	TMatrix pinv()	//psudo inverse
	{
		TMatrix	inv;
		return inv;
	}
	//˹ȫѡԪȥ
	T det()
	{
		TMatrix	B;
		T		max, det, a, **pMatrix;
		int		i, j, k, row, col, factor;
		assert(m_nRow == m_nCol);
		B = *this;
		pMatrix = B.m_pMatrix;
		for(i = 0; i < m_nRow - 1; i++)
		{
			//1. select Ԫ
			for(j = i, factor = 1, max = pMatrix[i][i], row = i, col = i; j < m_nRow; j++)
			{
				for(k = i; k < m_nCol; k++)
				{
					if(max < pMatrix[j][k])
					{
						max = pMatrix[j][k]; row = j;	col = k;
					}
				}
			}
			if(fabs(max) < TINY)
				return 0;
			if(row != i)	//swap row
			{
				for(k = i; k < m_nCol; k++)
					swap(pMatrix[i][k], pMatrix[row][k]);
				factor *= -1;		//ʽ
			}
			if(col != i)	//swap col
			{
				for(k = i; k < m_nRow; k++)
					swap(pMatrix[k][i], pMatrix[k][col]);
				factor *= -1;	//ʽ
			}		
			//2. ȥ
			for(j = i + 1; j < m_nRow; j++)
			{
				a = pMatrix[j][i] / pMatrix[i][i];
				pMatrix[j][i] = 0;
				for(k = i + 1; k < m_nCol; k++)
					pMatrix[j][k] -= pMatrix[i][k] * a;
			}
			pMatrix[i][i] *= factor;
			//Print();
		}
		for(i = 0, det = 1.0f; i < m_nRow; i++)
			det *= pMatrix[i][i];
		return det;
	}
	TMatrix schm()	//schmit orth
	{
		int			i, j, k;
		T			**pOrth, **pMatrix, *pLength, temp;
		TMatrix		orth;
		orth = *this;
		pOrth = orth.m_pMatrix;
		//1.
		pLength = (T *) GlobalAlloc(GMEM_FIXED | GMEM_ZEROINIT, sizeof(T) * m_nCol);		
		for(i = 0; i < m_nCol; i++)
		{
			for(j = 0; j < m_nRow; j++)			//bi = ai;
				pOrth[j][i] = m_pMatrix[j][i];
			for(j = 0; j < i; j++)	// bi -= (ai,bj)/(bj,bj) * bj
			{
				for(k = 0, temp = 0; k < m_nRow; k++)
					temp += m_pMatrix[k][i] * pOrth[k][j];
				if(pLength[i] < 0.000001 && pLength[i] > -0.000001)
					temp = 0;
				else
					temp = temp / pLength[j];
				for(k = 0; k < m_nRow; k++)
					pOrth[k][i] -= temp * pOrth[k][j];
			}
			for(j = 0; j < m_nRow; j++)
				pLength[i] += pOrth[j][i] * pOrth[j][i];
		}
		//2. Normalize
		for(i = 0; i < m_nCol; i++)
		{
			pLength[i] = sqrt(pLength[i]);
			for(j = 0; j < m_nRow; j++)
				pOrth[j][i] /= pLength[i];
		}
		GlobalFree(pLength);
		//orth.Print();
		return orth;
	}
	void qr(TMatrix &Q, TMatrix &R)
	{
		int		i, j, k;
		T		**pQMatrix, **pRMatrix, scale, sigma, sum, h, f;
		assert(m_nRow == m_nCol);
		R = *this;
		pRMatrix = R.m_pMatrix;		
		Q = eye(m_nRow);
		pQMatrix = Q.m_pMatrix;
		for(i = 0; i < m_nCol - 1; i++)
		{
			for(j = i, scale = 0; j < m_nRow; j++)
				scale += fabs(pRMatrix[j][i]);
			if(scale < 0.00001)	//
				continue;
			for(j = i, sum = 0; j < m_nRow; j++)
			{
				pRMatrix[j][i] /= scale;
				sum += pRMatrix[j][i] * pRMatrix[j][i];
			}
			sigma = sqrt(sum) * sign(pRMatrix[i][i]);	//let u = x - |x|*E1
			pRMatrix[i][i] -= sigma;
			sum = -pRMatrix[i][i] * sigma;		
			for(j = i + 1; j < m_nCol; j++)
			{
				for(k = i, f = 0; k < m_nRow; k++)
					f += pRMatrix[k][i] * pRMatrix[k][j];
				h = f / sum;
				for(k = i; k < m_nRow; k++)
					pRMatrix[k][j] -= pRMatrix[k][i] * h;
			}		
			Q.print(); //Q
			for(j = 0; j < m_nCol; j++)
			{
				for(k = i, f = 0; k < m_nRow; k++)
					f += pRMatrix[k][i] * pQMatrix[k][j];
				h = f / sum;
				for(k = i; k < m_nRow; k++)
					pQMatrix[k][j] -= pRMatrix[k][i] * h;
			}
			//µi
			pRMatrix[i][i] = sigma * scale;
			for(j = i + 1; j < m_nRow; j++)
				pRMatrix[j][i] = 0;			
			Q.print();
		}
		for(i = 0; i < m_nRow; i++)		//transport matrix
			for(j = i + 1; j < m_nCol; j++)
				swap(pQMatrix[i][j], pQMatrix[j][i]);
	}
	TVector<T> mean(int byCol = 1)	//byCol = 1 or 2
	{
		int			i, j;
		T			sum;
		TVector<T>	mean;
		if(byCol == 1)
		{
			mean.alloc(m_nCol);			
			for(i = 0; i < m_nCol; i++)
			{
				for(j = 0, sum = 0; j < m_nRow; j++)
					sum += m_pMatrix[j][i];
				mean[i] = sum / m_nRow;
			}
		}
		else
		{
			mean.alloc(m_nRow);			
			for(i = 0; i < m_nRow; i++)
			{
				for(j = 0, sum = 0; j < m_nCol; j++)
					sum += m_pMatrix[j][i];
				mean[i] = sum / m_nCol;
			}
		}
		return mean;
	}
	TVector<T> sum(int byCol = 1)	//byCol = 1, sum align column, byCol = 2, sum align row
	{
		TVector<T>	s;
		int		i, j;
		if(byCol == 1)	//align column
		{
			s.alloc(m_nCol);
			for(i = 0; i < m_nRow; i++)
			{
				for(j = 0; j < m_nCol; j++)
					s[j] += m_pMatrix[i][j];
			}
		}
		else
		{
			s.alloc(m_nRow);
			for(i = 0; i < m_nCol; i++)
			{
				for(j = 0; j < m_nRow; j++)
					s[j] += m_pMatrix[j][i];
			}
		}
		return s;
	}
	TVector<T> mmin(TVector<int> *index = NULL, int byCol = 1) //because min confilicts with a macro defined in windef.h
	{
		TVector<T>	m;
		T			min;
		int			i, j, k;

		if(byCol == 1)
		{
			m.alloc(m_nCol);
			if(index)
				index->resize(m_nCol);
			for(i = 0; i < m_nCol; i++)
			{
				min = m_pMatrix[0][i];
				k = 0;
				for(j = 1; j < m_nRow; j++)
				{
					if(min > m_pMatrix[j][i])
					{
						k = j;
						min = m_pMatrix[j][i];
					}
				}
				m[i] = min;
				if(index)
					(*index)[i] = k;
			}
		}
		else
		{
			m.alloc(m_nRow);
			if(index)
				index->resize(m_nRow);
			for(i = 0; i < m_nRow; i++)
			{
				min = m_pMatrix[i][0];
				k = 0;
				for(j = 1; j < m_nCol; j++)
				{
					if(min > m_pMatrix[i][j])
					{
						k = j;
						min = m_pMatrix[i][j];
					}
				}
				m[i] = min;
				if(index)
					(*index)[i] = k;
			}
		}
		return m;
	}
	T trace()
	{
		int	i	= 0;
		T	tr;
		assert(m_nRow == m_nCol);
		for(i = 0; i < m_nRow; i++)
			tr += m_pMatrix[i][i];
		return tr;
	}
	TVector<T> mmax(TVector<int> *index = NULL, int byCol = 1) // return index if required
	{
		TVector<T>	m;
		T			max;
		int			i, j, k;

		if(byCol == 1)
		{
			m.alloc(m_nCol);
			if(index)
				index->resize(m_nCol);
			for(i = 0; i < m_nCol; i++)
			{
				max = m_pMatrix[0][i];
				k = 0;
				for(j = 1; j < m_nRow; j++)
				{
					if(max < m_pMatrix[j][i])
					{
						k = j;
						max = m_pMatrix[j][i];
					}
				}
				m[i] = max;
				if(index)
					(*index)[i] = k;
			}
		}
		else
		{
			m.alloc(m_nRow);
			if(index)
				index->resize(m_nRow);
			for(i = 0; i < m_nRow; i++)
			{
				max = m_pMatrix[i][0];
				k = 0;
				for(j = 1; j < m_nCol; j++)
				{
					if(max < m_pMatrix[i][j])
					{
						k = j;
						max = m_pMatrix[i][j];
					}
				}
				m[i] = max;
				if(index)
					(*index)[i] = k;
			}
		}
		return m;
	}
	/*QL algorithm with implicit shifts, to determine the eigenvalues and eigenvectors of a 
	real, symmetric, tridiagonal matrix, or of a real, symmetric matrix previously reduced
	by tred2 x11.2. On input, d[1..n] contains the diagonal elements of the tridiagonal 
	matrix. On output, it returns the eigenvalues. The vector e[1..n] inputs the subdiagonal 
	elements of the tridiagonal matrix, with e[1] arbitrary. On output e is destroyed. 
	When nding only the eigenvalues, several lines may be omitted, as noted in the comments.
	If the eigenvectors of a tridiagonal matrix are desired, the matrix z[1..n][1..n] is 
	input as the identity matrix. If the eigenvectors of a matrix that has been reduced by 
	tred2 are required, then z is input as the matrix output by tred2. In either case, 
	the kth column of z returns the normalized eigenvector corresponding to d[k].*/
	void tqli(T *d, T *e, TMatrix &Z)
	{
		int	m, l, iter, i, k;
		T	s, r, p, g, f, dd, c, b, **pEVMatrix;
		//pEVMatrix(Z)ʱtred2(һΪ), tqliд
		pEVMatrix = Z.m_pMatrix;
		for(i = 1; i < m_nRow; i++)
			e[i - 1] = e[i];		//Convenient to renumber the elements of e. 
		e[m_nRow - 1] = 0;
		for(l = 0; l < m_nRow; l++)
		{
			iter = 0;
			do
			{
				for(m = l; m < m_nRow - 1; m++)
				{
					//Look for a single small subdiagonal element to split the matrix.
					dd = (T) fabs(d[m]) + (T) fabs(d[m + 1]);
					if((T) (fabs(e[m]) + dd) == dd)
						break;
				}
				if(m != l)
				{
					if(iter++ == 30)
						_tprintf(_T("Too many iterations in tqli"));
					g = (d[l + 1] - d[l]) / (2.0f * e[l]); //Form shift.
					r = pythag(g, 1.0f);
					g = d[m] - d[l] + e[l] / (g + sign(r, g)); //This is dm ? ks.
					s = c = 1.0f;
					p = 0;
					for(i = m - 1; i >= l; i--)
					{
						//A plane rotation as in the original QL, followed by Givens
						//rotations to restore tridiagonal form.
						f = s * e[i];
						b = c * e[i];
						e[i + 1] = (r = pythag(f, g));
						if(r == 0)
						{
							d[i + 1] -= p;	//Recover from underflow.
							e[m] = 0;
							break;
						}
						s = f / r;
						c = g / r;
						g = d[i + 1] - p;
						r = (d[i] - g) * s + 2.0f * c * b;
						d[i + 1] = g + (p = s * r);
						g = c * r - b;
						/* Next loop can be omitted if eigenvectors not wanted*/
						for(k = 0; k < m_nRow; k++)
						{
							f = pEVMatrix[k][i + 1];	// Form eigenvectors.
							pEVMatrix[k][i + 1] = s * pEVMatrix[k][i] + c * f;
							pEVMatrix[k][i] = c * pEVMatrix[k][i] - s * f;
						}
					}
					if(r == 0 && i >= l)
						continue;
					d[l] -= p;
					e[l] = g;
					e[m] = 0;
				}
			}
			while(m != l);
		}
	}
	/*Householder reduction of a real, symmetric matrix a[1..n][1..n]. On output, a is replaced
	by the orthogonal matrix Q eecting the transformation. d[1..n] returns the diagonal elements
	of the tridiagonal matrix, and e[1..n] the o-diagonal elements, with e[1]=0. Several
	statements, as noted in comments, can be omitted if only eigenvalues are to be found, in which
	case a contains no useful information on output. Otherwise they are to be included.*/
	void tred2(T *d, T *e)
	{
		int	l, k, j, i;
		T	scale, hh, h, g, f;
		for(i = m_nRow - 1; i >= 1; i--)
		{
			l = i - 1;
			h = scale = 0;
			if(l > 0)
			{
				for(k = 0; k <= l; k++)
					scale += (T) fabs(m_pMatrix[i][k]);
				if(scale < TINY) //Skip transformation.
					e[i] = m_pMatrix[i][l];
				else
				{
					for(k = 0; k <= l; k++)
					{
						m_pMatrix[i][k] /= scale; //Use scaled a's for transformation.
						h += m_pMatrix[i][k] * m_pMatrix[i][k]; //Form . in h.
					}
					f = m_pMatrix[i][l];
					g = (f >= 0.0f ? (T) - sqrt(h) : (T) sqrt(h));
					e[i] = scale * g;
					h -= f * g; //Now h is equation (11.2.4).
					m_pMatrix[i][l] = f - g; //Store u in the ith row of a.
					f = 0;
					for(j = 0; j <= l; j++)
					{
						/* Next statement can be omitted if eigenvectors not wanted */
						m_pMatrix[j][i] = m_pMatrix[i][j] / h; //Store u=H in ith column of a.
						g = 0;			//Form an element of A . u in g.
						for(k = 0; k <= j; k++)
							g += m_pMatrix[j][k] * m_pMatrix[i][k];
						for(k = j + 1; k <= l; k++)
							g += m_pMatrix[k][j] * m_pMatrix[i][k];
						e[j] = g / h; //Form element of p in temporarily unused element of e.
						f += e[j] * m_pMatrix[i][j];
					}
					hh = f / (h + h); //Form K, equation (11.2.11).
					for(j = 0; j <= l; j++)
					{
						//Form q and store in e overwriting p.
						f = m_pMatrix[i][j];
						e[j] = g = e[j] - hh * f;
						for(k = 0; k <= j; k++) //Reduce a, equation (11.2.13).
							m_pMatrix[j][k] -= (f * e[k] + g * m_pMatrix[i][k]);
					}
				}
				//Print();

			}
			else
				e[i] = m_pMatrix[i][l];
			d[i] = h;
		}
		/* Next statement can be omitted if eigenvectors not wanted */
		d[0] = 0;
		e[0] = 0;
		/* Contents of this loop can be omitted if eigenvectors not
						wanted except for statement d[i]=a[i][i]; */
		for(i = 0; i < m_nRow; i++)
		{
			//Begin accumulation of transformationmatrices.
			l = i - 1;
			if(d[i])		//should be d[i] < TINY ???
			{
				//This block skipped when i = 0.
				for(j = 0; j <= l; j++)
				{
					g = 0;
					for(k = 0; k <= l; k++) //Use u and u=H stored in a to form P.Q.
						g += m_pMatrix[i][k] * m_pMatrix[k][j];
					for(k = 0; k <= l; k++)
						m_pMatrix[k][j] -= g * m_pMatrix[k][i];
				}
			}
			d[i] = m_pMatrix[i][i]; //This statement remains.
			m_pMatrix[i][i] = 1.0f; //Reset row and column of a to identity matrix for next iteration. 
			for(j = 0; j <= l; j++)
				m_pMatrix[j][i] = m_pMatrix[i][j] = 0;
		}
	}
	/*Given a matrix a[1..n][1..n], this routine replaces it by a balanced matrix with identical
	eigenvalues. A symmetric matrix is already balanced and is unaected by this procedure. The
	parameter RADIX should be the machine's floating-point radix.*/
	void balanc()
	{
#define RADIX	2.0f
		int	last, j, i;
		T	s, r, g, f, c, sqrdx;
		sqrdx = RADIX * RADIX;
		last = 0;
		while(last == 0)
		{
			last = 1;
			for(i = 0; i < m_nRow; i++)		//Calculate row and column norms.
			{
				r = c = 0;
				for(j = 0; j < m_nRow; j++)
				{
					if(j != i)
					{
						c += fabs(m_pMatrix[j][i]);
						r += fabs(m_pMatrix[i][j]);
					}
				}
				if(c > TINY && r > TINY) //If both are nonzero,
				{
					g = r / RADIX;
					f = 1.0f;
					s = c + r;
					//nd the integer power of the machine radix that comes closest 
					//to balancing the matrix. 
					while(c < g)
					{
						f *= RADIX;
						c *= sqrdx;
					}
					g = r * RADIX;
					while(c > g)
					{
						f /= RADIX;
						c /= sqrdx;
					}
					if((c + r) / f < 0.95f * s)
					{
						last = 0;
						g = 1.0f / f;
						for(j = 0; j < m_nRow; j++)
							m_pMatrix[i][j] *= g;		//Apply similarity transformation.
						for(j = 0; j < m_nRow; j++)
							m_pMatrix[j][i] *= f;
					}
				}
			}
		}
	}
	/* Reduction to Hessenberg form by the elimination method. The real, nonsymmetric matrix
	a[1..n][1..n] is replaced by an upper Hessenberg matrix with identical eigenvalues. 
	Recommended, but not required, is that this routine be preceded by balanc. On output, 
	the Hessenberg matrix is in elements a[i][j] with i . j+1. Elements with i > j+1 are to 
	be thought of as zero, but are returned with random values. */
	void elmhes()
	{
		int	m, j, i;
		T	y, x;
		for(m = 1; m < m_nRow - 1; m++) //m is called r + 1 in the text.
		{
			x = 0;
			i = m;
			for(j = m; j < m_nRow; j++)
			{
				//Find the pivot.
				if(fabs(m_pMatrix[j][m - 1]) > fabs(x))
				{
					x = m_pMatrix[j][m - 1];
					i = j;
				}
			}
			if(i != m)
			{
				//Interchange rows and columns.
				for(j = m - 1; j < m_nRow; j++)
					swap(m_pMatrix[i][j], m_pMatrix[m][j]);
				for(j = 0; j < m_nRow; j++)
					swap(m_pMatrix[j][i], m_pMatrix[j][m]);
			}
			if(x)	//Carry out the elimination.
			{
				for(i = m + 1; i < m_nRow; i++)
				{
					if((y = m_pMatrix[i][m - 1]) != 0)
					{
						y /= x;
						m_pMatrix[i][m - 1] = y;
						for(j = m; j < m_nRow; j++)
							m_pMatrix[i][j] -= y * m_pMatrix[m][j];
						for(j = 0; j < m_nRow; j++)
							m_pMatrix[j][m] += y * m_pMatrix[j][i];
					}
					//Print();
				}
			}
		}
	}
	/*Finds all eigenvalues of an upper Hessenberg matrix a[1..n][1..n]. On input a can be
	exactly as output from elmhes x11.5; on output it is destroyed. The real and imaginary 
	parts of the eigenvalues are returned in wr[1..n] and wi[1..n], respectively.*/
	void hqr(T *wr, T *wi)
	{
		int	nn, m, l, k, j, its, i, mmin;
		T	z, y, x, w, v, u, t, s, r, q, p, anorm;
		//Compute matrix norm for possible use in locating single small subdiagonal element. 
		anorm = 0; 
		for(i = 0; i < m_nRow; i++)
			for(j = max(i - 1, 0); j < m_nRow; j++)
				anorm += fabs(m_pMatrix[i][j]);
		nn = m_nRow - 1;
		t = 0; //Gets changed only by an exceptional shift.
		while(nn >= 0)
		{
			//Begin search for next eigenvalue.
			its = 0;
			do
			{
				for(l = nn; l >= 1; l--)
				{
					//Begin iteration: look for single small subdiagonal element. 
					s = fabs(m_pMatrix[l - 1][l - 1]) + fabs(m_pMatrix[l][l]);
					if(s == 0.0)
						s = anorm;
					if((T) (fabs(m_pMatrix[l][l - 1]) + s) == s)
						break;
				}
				x = m_pMatrix[nn][nn];
				if(l == nn)			//One Root found.
				{
					wr[nn] = x + t;
					wi[nn--] = 0;
				}
				else
				{
					y = m_pMatrix[nn - 1][nn - 1];
					w = m_pMatrix[nn][nn - 1] * m_pMatrix[nn - 1][nn];
					if(l == (nn - 1))	//Two roots found...
					{
						p = 0.5f * (y - x);
						q = p * p + w;
						z = sqrt(fabs(q));
						x += t;
						if(q >= 0.0)	//...a real pair.
						{
							z = p + sign(z, p);
							wr[nn - 1] = wr[nn] = x + z;
							if(z)
								wr[nn] = x - w / z;
							wi[nn - 1] = wi[nn] = 0;
						}
						else
						{
							//...a complex pair.
							wr[nn - 1] = wr[nn] = x + p;
							wi[nn - 1] = -(wi[nn] = z);
						}
						nn -= 2;
					}
					else	//No roots found. Continue iteration.
					{
						if(its == 30)
							_tprintf(_T("Too many iterations in hqr"));
						if(its == 10 || its == 20) //Form exceptional shift.
						{
							t += x;
							for(i = 1; i <= nn; i++)
								m_pMatrix[i][i] -= x;
							s = fabs(m_pMatrix[nn][nn - 1]) + fabs(m_pMatrix[nn - 1][nn - 2]);
							y = x = 0.75f * s;
							w = -0.4375f * s * s;
						}
						++its;
						for(m = (nn - 2); m >= l; m--)
						{
							//Form shift and then look for 2 consecutive small subdiagonal elements.
							z = m_pMatrix[m][m];
							r = x - z;
							s = y - z;
							p = (r * s - w) / m_pMatrix[m + 1][m] + m_pMatrix[m][m + 1]; //Equation (11.6.23).
							q = m_pMatrix[m + 1][m + 1] - z - r - s;
							r = m_pMatrix[m + 2][m + 1];
							s = fabs(p) + fabs(q) + fabs(r); //Scale to prevent overflow or underflow. 
							p /= s;
							q /= s;
							r /= s;
							if(m == l)
								break;
							u = fabs(m_pMatrix[m][m - 1]) * (fabs(q) + fabs(r));
							v = fabs(p) * (fabs(m_pMatrix[m - 1][m - 1]) + fabs(z) + fabs(m_pMatrix[m + 1][m + 1]));
							if((T) (u + v) == v) //Equation (11.6.26).
								break;
						}
						for(i = m + 2; i <= nn; i++)
						{
							m_pMatrix[i][i - 2] = 0;
							if(i != (m + 2))
								m_pMatrix[i][i - 3] = 0;
						}
						for(k = m; k <= nn - 1; k++)
						{
							if(k != m) //Double QR step on rows l to nn and columns m to nn.
							{
								p = m_pMatrix[k][k - 1]; //Begin setup of Householder vector. 
								q = m_pMatrix[k + 1][k - 1];
								r = 0;
								if(k != (nn - 1))
									r = m_pMatrix[k + 2][k - 1];
								if((x = fabs(p) + fabs(q) + fabs(r)) != 0.0)
								{
									p /= x; //Scale to prevent overflow or underflow. 
									q /= x;
									r /= x;
								}
							}
							if((s = sign(sqrt(p * p + q * q + r * r), p)) != 0)
							{
								if(k == m)
								{
									if(l != m)
										m_pMatrix[k][k - 1] = -m_pMatrix[k][k - 1];
								}
								else
									m_pMatrix[k][k - 1] = -s * x;
								p += s;			//Equations (11.6.24).
								x = p / s;
								y = q / s;
								z = r / s;
								q /= p;
								r /= p;
								for(j = k; j <= nn; j++) //Row modication.
								{
									p = m_pMatrix[k][j] + q * m_pMatrix[k + 1][j];
									if(k != (nn - 1))
									{
										p += r * m_pMatrix[k + 2][j];
										m_pMatrix[k + 2][j] -= p * z;
									}
									m_pMatrix[k + 1][j] -= p * y;
									m_pMatrix[k][j] -= p * x;
								}
								mmin = nn < k + 3 ? nn : k + 3;
								for(i = l; i <= mmin; i++)
								{
									//Column modification.
									p = x * m_pMatrix[i][k] + y * m_pMatrix[i][k + 1];
									if(k != (nn - 1))
									{
										p += z * m_pMatrix[i][k + 2];
										m_pMatrix[i][k + 2] -= p * r;
									}
									m_pMatrix[i][k + 1] -= p * q;
									m_pMatrix[i][k] -= p;
								}
							}
						}
					}
				}
			}
			while(l < nn - 1);
		}
	}
	//еֵӦ
	//pEigenvalues[2 * i] ǵiֵʵ
	//pEigenvalues[2 * i +1] ǵiֵ鲿, 鲿Ϊ0, ֵһʵֵ
	void rseig(T *wr, TMatrix &eigenvectors)
	{
		//save(_T("c:\\matrix2.txt"));
		T		*diag, *subdiag;
		eigenvectors = *this;
		diag = (T *) GlobalAlloc(GMEM_FIXED, m_nRow * sizeof(T) * 2);		
		subdiag = diag + m_nRow;
		//ʵԳƾתΪԽǵHessenberg, Լ
		eigenvectors.tred2(diag, subdiag);
		//˫QR㷨, еֵdiagУӦھpEigenvalues
		//
		eigenvectors.tqli(diag, subdiag, eigenvectors);	//tred2ת
		CopyMemory(wr, diag, m_nRow * sizeof(T));
		GlobalFree(diag);	
		//eigenvectors.print();
		//TMatrix d;
		//d = diag(wr, m_nRow);
		//d.print();
		sorteigsD(wr, m_nRow, eigenvectors);	//sorting as descend order
		//eigenvectors.print();
		//d = diag(wr, m_nRow);
		//d.print();
	}
	void raseig(T *wr, T *wi, TMatrix &eigenvalues)
	{
		eigenvalues = *this;
		//1. balance
		eigenvalues.balanc();
		//2. elimination. convert into up Hessenberg matrix
		eigenvalues.elmhes();
		//3. QR algorithm
		eigenvalues.hqr(wr, wi);
	}
	// A = U * E * V', AMxNľUҲMN,
	// EA'AֵƽɵĶԽ; VN.
	// ֵ: Uڱ, wǶԽǾĶԽ(E).
	void svd(TMatrix &U, TMatrix &V, T *w)
	{
		int		i, j, k, flag, l, nm, jj, its;
		T		**pVMatrix, **pMatrix, scale, f, g, h, *up, norm, c, x, y, z, s;
		U = *this;
		pMatrix = U.m_pMatrix;
		//U.print();
		V = eye(m_nCol);
		pVMatrix = V.m_pMatrix;
		up = (T *) GlobalAlloc(GMEM_FIXED, m_nCol * sizeof(T));
		// תΪ˫Խ
		for(i = 0, g = scale = norm = 0; i < m_nCol; i++)
		{
			l = i + 1;
			up[i] = g * scale;			
			g = scale = s = 0;	//  i 
			if(i < m_nRow)
			{
				for(j = i; j < m_nRow; j++)
					scale += fabs(pMatrix[j][i]);
				if(scale)
				{
					for(j = i; j < m_nRow; j++)
					{
						pMatrix[j][i] /= scale;
						s += pMatrix[j][i] * pMatrix[j][i];
					}
					f = pMatrix[i][i];
					g = f > 0 ? -sqrt(s) : sqrt(s);
					h = f * g - s;
					pMatrix[i][i] = f - g;					
					for(j = l; j < m_nCol; j++)
					{
						for(k = i, s = 0; k < m_nRow; k++)
							s += pMatrix[k][i] * pMatrix[k][j];
						f = s / h;
						for(k = i; k < m_nRow; k++)
							pMatrix[k][j] += pMatrix[k][i] * f;
					}
					for(j = i; j < m_nRow; j++)
						pMatrix[j][i] *= scale;
				}
			}
			w[i] = g * scale;	//scale̫С˵Ѿܽӽ0Բˣw[i] = 0
			//U.print();
			//  i 
			s = scale = g = 0;
			if(i < m_nRow && i != m_nCol - 1)	//ΪǶһУԵm_nCol - 1Ϊֹ
			{
				for(j = l; j < m_nCol; j++)
					scale += fabs(pMatrix[i][j]);
				if(scale)
				{
					for(j = l; j < m_nCol; j++)
					{
						pMatrix[i][j] /= scale;
						s += pMatrix[i][j] * pMatrix[i][j];
					}
					f = pMatrix[i][l];
					g = f > 0 ? -sqrt(s) : sqrt(s);
					h = f * g - s;
					pMatrix[i][l] = f - g;					
					for(j = l; j < m_nCol; j++)
						up[j] = pMatrix[i][j] / h;
					for(j = l; j < m_nRow; j++)
					{
						for(k = l, s = 0; k < m_nRow; k++)
							s += pMatrix[i][k] * pMatrix[j][k];						
						for(k = l; k < m_nCol; k++)
							pMatrix[j][k] += s * up[k];
					}
					for(j = l; j < m_nCol; j++)
						pMatrix[i][j] *= scale;
				}
			}
			//U->Print();
			norm = max(norm, fabs(w[i]) + fabs(up[i]));
		}	
		//U.print();
		//V, õʽ Q = Qi * Q, i from min(m_nRow, m_nCol) - 1 to 0
		for(i = m_nCol - 1; i >= 0; i--)
		{
			if(i < m_nCol - 1)
			{
				if(g)
				{
					for(j = l; j < m_nCol; j++)
						pVMatrix[j][i] = pMatrix[i][j] / pMatrix[i][l] / g;
					for(j = l; j < m_nCol; j++)
					{
						for(k = l, s = 0; k < m_nCol; k++)
							s += pMatrix[i][k] * pVMatrix[k][j];
						for(k = l; k < m_nCol; k++)
							pVMatrix[k][j] += s * pVMatrix[k][i];
					}
				}
				for(j = l; j < m_nCol; j++)
					pVMatrix[i][j] = pVMatrix[j][i] = 0;
			}
			pVMatrix[i][i] = 1;
			//V->Print();
			g = up[i];
			l = i;
		}
		//U.
		for(i = min(m_nRow, m_nCol) - 1; i >= 0; i--)
		{
			l = i + 1;
			g = w[i];
			for(j = l; j < m_nCol; j++)	//Ǿ0
				pMatrix[i][j] = 0;
			if(g)
			{
				g = 1.0 / g;
				for(j = l; j < m_nCol; j++)
				{
					for(k = l, s = 0; k < m_nRow; k++)
						s += pMatrix[k][i] * pMatrix[k][j];
					f = s / pMatrix[i][i] * g;
					for(k = i; k < m_nRow; k++)
						pMatrix[k][j] += pMatrix[k][i] * f;
				}
				for(j = i; j < m_nRow; j++)
					pMatrix[j][i] *= g;
			}
			else
			{
				for(j = i; j < m_nRow; j++)
					pMatrix[j][i] = 0;
			}
			pMatrix[i][i] += 1.0;
		}
		//ԽǾA'Aֵ
		for(k = m_nCol - 1; k >= 0; k--)
		{
			for(its = 1; its <= 30; its++)
			{
				flag = 1;
				for(l = k; l >= 0; l--)
				{
					nm = l - 1;
					if((fabs(up[l]) + norm) == norm)
					{
						flag = 0;
						break;
					}
					if((fabs(w[nm]) + norm) == norm)
						break;
				}
				if(flag)
				{
					c = 0;
					s = 1;
					for(i = l; i <= k; i++)
					{
						f = s * up[i];
						up[i] = c * up[i];
						if((fabs(f) + norm) == norm)
							break;
						g = w[i];
						h = pythag(f, g);
						w[i] = h;
						h = 1.0f / h;
						c = g * h;
						s = -f * h;
						for(j = 0; j < m_nRow; j++)
						{
							y = pMatrix[j][nm];
							z = pMatrix[j][i];
							pMatrix[j][nm] = y * c + z * s;
							pMatrix[j][i] = z * c - y * s;
						}
					}
				}
				z = w[k];
				if(l == k)
				{
					if(z < 0)
					{
						w[k] = -z;
						for(j = 0; j < m_nCol; j++)
							pVMatrix[j][k] = -pVMatrix[j][k];
					}
					break;
				}
				if(its == 30)
					_tprintf(_T("error\n"));
				x = w[l];
				nm = k - 1;
				y = w[nm];
				g = up[nm];
				h = up[k];
				f = ((y - z) * (y + z) + (g - h) * (g + h)) / (2.0f * h * y);
				g = pythag(f, 1.0f);
				f = ((x - z) * (x + z) + h * ((y / (f + sign(g, f))) - h)) / x;
				c = s = 1.0f;
				for(j = l; j <= nm; j++)
				{
					i = j + 1;
					g = up[i];
					y = w[i];
					h = s * g;
					g = c * g;
					z = pythag(f, h);
					up[j] = z;
					c = f / z;
					s = h / z;
					f = x * c + g * s;
					g = g * c - x * s;
					h = y * s;
					y *= c;
					for(jj = 0; jj < m_nCol; jj++)
					{
						x = pVMatrix[jj][j];
						z = pVMatrix[jj][i];
						pVMatrix[jj][j] = x * c + z * s;
						pVMatrix[jj][i] = z * c - x * s;
					}
					z = pythag(f, h);
					w[j] = z;
					if(z)
					{
						z = 1.0f / z;
						c = f * z;
						s = h * z;
					}
					f = c * g + s * y;
					x = c * y - s * g;
					for(jj = l; jj < m_nRow; jj++)
					{
						y = pMatrix[jj][j];
						z = pMatrix[jj][i];
						pMatrix[jj][j] = y * c + z * s;
						pMatrix[jj][i] = z * c - y * s;
					}
				}
				up[l] = 0;
				up[k] = f;
				w[k] = x;						
				//_tprintf(_T("iteration: %d\n"), its);
			}
		}
		//U->Print();
		//V->Print();
		GlobalFree(up);
	}

	// Principle Component Analysis
	// feature is row vector, target is n by nDimension matrix, d is dimension of feature, n is sample number
	void pca(TMatrix &projection, TMatrix &mean_m, int nDimension, float fRatio)
	{
		TMatrix		center, cov, eigenvectors;	//covariance matrix
		int			i, j, k;
		T			sum;
		TVector<T>	eigenvalues;
		//1. calc mean
		mean_m = mean();			
		//2. calc covariance matrix: 
		//		(A - means * ones(m_nRow, 1)) * ~(A - means * ones(m_nRow, 1))
		// convariance matrix can be replaced by coefficient matrix, say, normalization
		cov = zeros(m_nCol, m_nCol);		
		center = *this;
		for(i = 0; i < m_nCol; i++)				// mean subtracted
			for(j = 0; j < m_nRow; j++)
				center.m_pMatrix[j][i] -= mean_m[0][i];
		for(i = 0; i < m_nCol; i++)
		{
			for(j = 0; j < m_nCol; j++)
			{
				for(k = 0, sum = 0; k < m_nRow; k++)
					sum += center.m_pMatrix[k][i] * center.m_pMatrix[k][j];
				cov.m_pMatrix[i][j] = sum / m_nRow;
			}
		}		
		//3. calc and sort all eigenvalues and responsing eigenvectors
		//covarance matrix is a symmetric, real matrix
		eigenvalues.alloc(m_nCol);
		cov.rseig(eigenvalues.data(), eigenvectors);
		sorteigsD(eigenvalues.data(), m_nCol, eigenvectors);		
		//4. select main eigenvalues. What strategy used?
		projection = eigenvectors.crop(0, 0, m_nCol, nDimension);	//delete smaller eigenvectors				
	}
	//Using Doolittle algorithm
	// L is a λǾ
	// U is a Ǿ
	// P is Ԫ任
	int doolittle(TMatrix &L, TMatrix &U)
	{
		int		i, j, k;
		T		**pL, **pU, temp;
		assert(m_nRow == m_nCol);
		L = eye(m_nRow);
		U = zeros(m_nRow, m_nCol);
		pL = L.m_pMatrix;
		pU = U.m_pMatrix;
		for(i = 0; i < m_nRow; i++)
		{
			//1. calc U[i,i] to U[i, m_nCol-1]
			for(j = i; j < m_nCol; j++)
			{
				for(k = 0, temp = 0; k < i; k++)
					temp += pL[i][k] * pU[k][j];
				pU[i][j] = m_pMatrix[i][j] - temp;
			}
			if(pU[i][i] <= 0.00001 && pU[i][i] >= -0.00001)
			{
				return 0;	//Singular
			}
			//2. calc L[i,i] to L[m_nRow - 1, i]
			for(j = i + 1; j < m_nRow; j++)
			{
				for(k = 0, temp = 0; k < i; k++)
					temp += pL[j][k] * pU[k][i];
				pL[j][i] = (m_pMatrix[j][i] - temp) / pU[i][i];
			}
			//L->Print(); U->Print();
		}	
		return 1;
	}
	/* Given a matrix a[1..n][1..n], this routine replaces it by the LU decomposition of a 
	rowwise permutation of itself. a and n are input. a is output, arranged as in equation 
	(2.3.14) above; indx[1..n] is an output vector that records the row permutation erected 
	by the partial pivoting; d is output as .1 depending on whether the number of row 
	interchanges was even or odd, respectively. This routine is used in combination with 
	lubksb to solve linear equations or invert a matrix.*/
	int ludcmp(TMatrix &LU, int *indx, T *d)
	{
		int	i, imax, j, k;
		T	big, dum, sum, temp, *vv, *pMatrix;	//vv stores the implicit scaling of each row.
		LU = *this;
		pMatrix = LU.m_pMatrix;
		vv = (T *) GlobalAlloc(GMEM_FIXED, sizeof(T) * m_nRow);
		*d = 1;		// No row interchanges yet.
		for(i = 0; i < m_nRow; i++)
		{
			// Loop over rows to get the implicit scaling information.
			big = 0;
			for(j = 0; j < m_nRow; j++)
				if((temp = fabs(pMatrix[i][j])) > big)
					big = temp;
			if(big == 0)
			{
				_tprintf(_T("Singular matrix in routine ludcmp"));
				return 0;
			}
			//No nonzero largest element.
			vv[i] = 1.0f / big; //Save the scaling.
		}
		for(j = 0; j < m_nRow; j++) //This is the loop over columns of Crout's method.
		{
			for(i = 0; i < j; i++) //This is equation (2.3.12) except for i = j.
			{
				sum = pMatrix[i][j];
				for(k = 0; k < i; k++)
					sum -= pMatrix[i][k] * pMatrix[k][j];
				pMatrix[i][j] = sum;
			}
			big = 0; //Initialize for the search for largest pivot element.
			for(i = j; i < m_nRow; i++)
			{
				//This is i = j of equation (2.3.12) and i = j+1 : ::N
				//of equation (2.3.13). 
				sum = pMatrix[i][j];
				for(k = 0; k < j; k++)
					sum -= pMatrix[i][k] * pMatrix[k][j];
				pMatrix[i][j] = sum;
				if((dum = vv[i] * fabs(sum)) >= big)
				{
					//Is the gure of merit for the pivot better than the best so far?
					big = dum;
					imax = i;
				}
			}
			if(j != imax) //Do we need to interchange rows?
			{
				for(k = 0; k < m_nRow; k++) //Yes, do so...
				{
					dum = pMatrix[imax][k];
					pMatrix[imax][k] = pMatrix[j][k];
					pMatrix[j][k] = dum;
				}
				*d = -(*d);			//...and change the parity of d.
				vv[imax] = vv[j];	//Also interchange the scale factor.
			}
			indx[j] = imax;
			//If the pivot element is zero the matrix is singular (at least to the 
			//precision of the algorithm). For some applications on singular matrices, 
			//it is desirable to substitute TINY for zero.
			if(pMatrix[j][j] == 0)
			{
				pMatrix[j][j] = 0; //TINY; modified by Zhiwei
				return 0;
			}		
			if(j != m_nRow - 1) //Now, nally, divide by the pivot element.
			{
				dum = 1.0f / (pMatrix[j][j]);
				for(i = j + 1; i < m_nRow; i++)
					pMatrix[i][j] *= dum;
			}
		}	//Go back for the next column in the reduction.
		GlobalFree(vv);
		return 1;
	}
	/* Solves the set of n linear equations A*X = B. Here a[1..n][1..n] is input, not as the matrix
	A but rather as its LU decomposition, determined by the routine ludcmp. indx[1..n] is input
	as the permutation vector returned by ludcmp. b[1..n] is input as the right-hand side vector
	B, and returns with the solution vector X. a, n, and indx are not modied by this routine
	and can be left in place for successive calls with dierent right-hand sides b. This routine takes
	into account the possibility that b will begin with many zero elements, so it is e.cient for use
	in matrix inversion.*/
	void lubksb(int *indx, T *b)
	{
		int	i, ii = -1, ip, j;
		T	sum;
		for(i = 0; i < m_nRow; i++)
		{
			// When ii is set to a positive value, it will become the
			// index of the rst nonvanishing element of b. Wenow do the forward substitution, 
			// equation (2.3.6). The only new wrinkle is to unscramble the 
			// permutation as we go.
			ip = indx[i];
			sum = b[ip];
			b[ip] = b[i];
			if(ii != -1)
				for(j = ii; j <= i - 1; j++)
					sum -= m_pMatrix[i][j] * b[j];
			else if(sum)
				ii = i; 
			//A nonzero element was encountered, so from now on we
			//will have to do the sums in the loop above. 
			b[i] = sum;
		}
		for(i = m_nRow - 1; i >= 0; i--)
		{
			//Now we do the backsubstitution, equation (2.3.7).
			sum = b[i];
			for(j = i + 1; j < m_nRow; j++)
				sum -= m_pMatrix[i][j] * b[j];
			b[i] = sum / m_pMatrix[i][i]; //Store a component of the solution vector X.
		} //All done!
	}
	void sorteigsA(T *eigenvalues, int n, TMatrix &eigenvectors)
	{
		int	i, j, k;
		T	max;
		for(i = 0; i < n - 1; i++)
		{
			max = eigenvalues[0];
			k = 0;
			for(j = 1; j < n - i; j++)
			{
				if(max < eigenvalues[j])
				{
					max = eigenvalues[j];
					k = j;
				}
			}
			if(k < n - i - 1)
			{
				swap(eigenvalues[k], eigenvalues[n - i - 1]);
				eigenvectors.swapcol(k, n - i - 1);
			}
		}
	}
	void sorteigsD(T *eigenvalues, int n, TMatrix &eigenvectors)
	{
		int	i, j, k;
		T	min;
		for(i = 0; i < n - 1; i++)
		{
			min = eigenvalues[0];
			k = 0;
			for(j = 1; j < n - i; j++)
			{
				if(min > eigenvalues[j])
				{
					min = eigenvalues[j];
					k = j;
				}
			}
			if(k < n - i - 1)
			{
				swap(eigenvalues[k], eigenvalues[n - i - 1]);
				eigenvectors.swapcol(k, n - i - 1);
			}
		}
	}
	T fnorm()		//forbunis norm
	{
		return 0;
	}
	T norm0()
	{
	}
	T nrom1()
	{
	}
	T norm2()
	{
	}
	T cond2()
	{
	}
	//col = 1, column vector, col = 2, row vector
	//way = 1, Eucilid norm 2
	void normalize(int col = 1, int way = 1)
	{
		int	i, j;
		T	sum;
		if(col == 1)		//column vector
		{
			if(way == 1)
			{
				for(i = 0; i < m_nCol; i++)
				{
					for(j = 0, sum = 0; j < m_nRow; j++)
						sum += m_pMatrix[j][i] * m_pMatrix[j][i];
					sum = sqrt(sum);
					if(sum == 0)
						continue;
					for(j = 0; j < m_nRow; j++)
						m_pMatrix[j][i] /= sum;
				}
			}
			else
			{
			}
		}
		else	//row vector
		{
			if(way == 1)
			{
				for(i = 0; i < m_nRow; i++)
				{
					for(j = 0, sum = 0; j < m_nCol; j++)
						sum += m_pMatrix[i][j] * m_pMatrix[i][j];
					sum = sqrt(sum);
					if(sum == 0)
						continue;
					for(j = 0; j < m_nCol; j++)
						m_pMatrix[i][j] /= sum;
				}
			}
			else
			{
			}
		}
	}
};
