#include "StdAfx.h"
#include "InstTree.h"
#include "Data.h"
#include "InstRect.h"
#include "DetectModel.h"

CInstTree::CInstTree(void)
{
}

CInstTree::~CInstTree(void)
{
}

// Initialization for inference  (for all images)
int CInstTree::Init(CDetectModel &DetModel)
{
	int		Root	= DetModel.Root;
	int		AndCnt	= (int) DetModel.AndModels.size( );
	int		OrCnt	= (int) DetModel.OrModels.size( );
	int		MarX	= g_Parameters.MarX;
	int		MarY	= g_Parameters.MarY;

	//  Alloc memory for each node.
	OrNodes.resize(OrCnt);
	AndNodes.resize(AndCnt);
	InstRects.reserve(g_Parameters.MaxInstCnt);
	for ( int t = 0; t < AndCnt; t++ )
	{
		CAndModel	&CurModel	= DetModel.AndModels[t];
		int			Lvl			= CurModel.Level;
		int			HoG_S		= DetModel.HoG_S[Lvl];

		AndNodes[t].matSre.alloc(MaxY, MaxX);
		AndNodes[t].matSrcX.alloc(MaxY, MaxX);
		AndNodes[t].matSrcY.alloc(MaxY, MaxX);
		AndNodes[t].NodeID	= DetModel.AndModels[t].NodeID;
		AndNodes[t].InitX	= - MarX / 2;
		AndNodes[t].InitY	= - MarY / 2;
		for ( int p = 0; p < HoG_S; p++ )
		{
			AndNodes[t].InitX	*= 2;
			AndNodes[t].InitY	*= 2;
		}
	}

	for ( int t = 0; t < OrCnt; t++ )
	{
		OrNodes[t].NodeID	= DetModel.OrModels[t].NodeID;
	}

	return	1;
}


// Non-Maximum Suppression
int CInstTree::NonMaxSupp(double NMSThres)
{
	vector<pair<double, int> >		RectScores;
	double							Overlap;
	int								RectID;
	int								suppress;

	RectScores.clear( );
	for ( int t = 0; t < (int) InstRects.size( ); t++ )
	{
		RectScores.push_back(make_pair(InstRects[t].Score, t));
	}
	sort(RectScores.begin( ), RectScores.end( ), greater<pair<double, int> >( ));

	NMSRects.clear( );
	for ( int t = 0; t < (int) InstRects.size( ); t++ )
	{
		suppress	= 0;
		RectID		= RectScores[t].second;
		if (InstRects[RectID].RootRect.Width( ) * InstRects[RectID].RootRect.Height( ) < g_Parameters.MinDetectSize / 2)
		{
			continue;
		}

		for ( int p = 0; p < (int) NMSRects.size( ); p++ )
		{
			// overlap: (A&B)/A > 0.5. A: current instances. B: the intances in the result set
			Overlap	= CExternalTools::GetRectOverlap(InstRects[RectID].RootRect, NMSRects[p].RootRect, 1);
			if (Overlap > NMSThres)
			{
				suppress	= 1;
				break;
			}
		}

		if (suppress == 0)
		{
			// add to the result set
			NMSRects.push_back(InstRects[RectID]);
		}
	}

	return	1;
}

// Initialization for inference (for current image)
int CInstTree::InitNode(CDetectModel &DetModel, CData &Data, int Level)
{
	int		Root	= DetModel.Root;
	int		AndCnt	= (int) AndNodes.size( );
	int		BinSize	= g_Parameters.BinSize_HoG;
	
	for ( int l = DetModel.MaxL - 1; l >= 0; l-- )
	{
		for ( int t = 0; t < AndCnt; t++ )
		{
			if (DetModel.AndModels[t].Level != l)
			{
				continue;
			}

			CAndModel	&CurModel	= DetModel.AndModels[t];
			CAndNode	&CurNode	= AndNodes[t];
			int			HoG_S		= DetModel.HoG_S[l];
			int			ExtX		= abs(AndNodes[t].InitX);
			int			ExtY		= abs(AndNodes[t].InitY);
			int			MarX		= g_Parameters.MarX;
			int			MarY		= g_Parameters.MarY;
			int			HoG_L		= Level - HoG_S * Data.InVal;
			int			Step		= 1;

			CurNode.HoG_L	= HoG_L;
			CurNode.SizeX	= __min(MaxX, Data.FeaPyd[HoG_L].Map_W - BlkDimX) - (MarX + AndNodes[t].InitX);
			CurNode.SizeY	= __min(MaxY, Data.FeaPyd[HoG_L].Map_H - BlkDimY) - (MarY + AndNodes[t].InitY);
			CurNode.SizeX	= __min(CurNode.SizeX, Data.FeaPyd[HoG_L].Img_W / BinSize + 2 * ExtX);
			CurNode.SizeY	= __min(CurNode.SizeY, Data.FeaPyd[HoG_L].Img_H / BinSize + 2 * ExtY);
			
			if (l + 1 != DetModel.MaxL)
			{
				for ( int p = DetModel.HoG_S[l]; p < DetModel.HoG_S[l+1]; p++ )
				{
					Step	*= 2;
				}

				for ( int p = 0; p < (int) CurModel.ChildSet.size( ); p++ )
				{
					int			NxtID		= CurModel.ChildSet[p];
					CAndNode	&NxtNode	= AndNodes[NxtID];
					CurNode.SizeX	= __min(CurNode.SizeX, NxtNode.SizeX / Step);
					CurNode.SizeY	= __min(CurNode.SizeY, NxtNode.SizeY / Step);
				}
			}
		}
	}

	return	1;
}

// calculate appearance scores
int CInstTree::MakeAppScore(CDetectModel &DetModel, CData &Data, int Level)
{
	int		ret;

	for ( int t = 0; t < DetModel.NodeCnt; t++ )
	{
		ret	= AndNodes[t].MakeAppScore(DetModel, Data, (*this));
	}

	return	1;
}

// Detect - find the bounding boxes whose score is large than a threshold
int CInstTree::Detect(CDetectModel &DetModel, CData &Data, int InVal)
{
	int		Root	= DetModel.Root;
	int		AndCnt	= (int) AndNodes.size( );
	int		InitS	= DetModel.InitS;
	int		Idx		= 0;
	int		ret;

	InstRects.clear( );

	// extract feature pyramid
	g_Parameters.timer2.Start( );
	ret	= Data.ExtractFeaturePyramid(InVal, InitS);
	g_Parameters.timer2.Stop( );

	// for each level of feature pyramid
	for ( int Index = InitS * InVal; Index < (int) Data.FeaPyd.size( ); Index++ )
	{
		ret	= InitNode(DetModel, Data, Index);
		g_Parameters.timer3.Start( );
		ret	= MakeAppScore(DetModel, Data, Index);
		g_Parameters.timer3.Stop( );
		g_Parameters.timer4.Start( );
		ret	= OrNodes[Root].MakeNodeScore(DetModel, Data, (*this));
		g_Parameters.timer4.Stop( );
		g_Parameters.timer5.Start( );
		ret	= OrNodes[Root].GetInstRects(DetModel, Data, (*this), DetModel.Thres);
		g_Parameters.timer5.Stop( );
	}
	Data.FreePyramid( );

	return	1;
}

// Detect - find the instance with maximum overlap (positive images)
int CInstTree::GetInst_Overlap(CDetectModel &DetModel, CData &Data, CSVMInst &CurInst)
{
	int		Root	= DetModel.Root;
	int		AndCnt	= (int) AndNodes.size( );
	double	MaxS	= 0;
	int		MinSize	= DetModel.GetAreaSize_Min( );
	int		MaxSize	= DetModel.GetAreaSize_Max( );
	int		RSize	= Data.PosRect.Width( ) * Data.PosRect.Height( );
	int		InVal	= g_Parameters.Interval_Train;
	int		InitS	= DetModel.InitS;
	double	Scale, CSize;
	int		ret;

	// Initialization
	ret	= CurInst.Init(DetModel);
	CurInst.Alpha		= 1.0f;
	CurInst.Score_Loss	= 0.0f;
	CurInst.DataID		= Data.DataID;
	CurInst.Label		= 1;
	CurInst.LRCmp		= 0;
	if (RSize < MinSize)
	{
		return	1;
	}

	// extract feature pyramid
	ret	= Data.ExtractFeaturePyramid(InVal, InitS);

	// for each level of feature pyramid
	for ( int Index = InitS * InVal; Index < (int) Data.FeaPyd.size( ); Index++ )
	{
		Scale	= Data.FeaPyd[Index].Scale;
		CSize	= MaxSize / (Scale * Scale);
		if (CSize >= g_Parameters.Overlap_Root * RSize)
		{
			ret	= InitNode(DetModel, Data, Index);
			ret	= MakeAppScore(DetModel, Data, Index);
			ret	= OrNodes[Root].MakeNodeScore(DetModel, Data, (*this));
			ret	= OrNodes[Root].GetInst_Overlap(DetModel, Data, (*this), CurInst, MaxS);
		}
	}
	Data.FreePyramid( );

	return	1;
}

// Detect - find the instance with maximum score (positive images)
int CInstTree::GetInst_Score(CDetectModel &DetModel, CData &Data, CSVMInst &CurInst)
{
	int		Root	= DetModel.Root;
	int		AndCnt	= (int) AndNodes.size( );
	int		MinSize	= DetModel.GetAreaSize_Min( );
	int		MaxSize	= DetModel.GetAreaSize_Max( );
	int		RSize	= Data.PosRect.Width( ) * Data.PosRect.Height( );
	int		InVal	= g_Parameters.Interval_Train;
	int		InitS	= DetModel.InitS;
	double	Scale, CSize;
	int		ret;

	ret	= CurInst.Init(DetModel);
	CurInst.Alpha		= 1.0f;
	CurInst.Score_Loss	= 0.0f;
	CurInst.DataID		= Data.DataID;
	CurInst.Label		= 1;
	CurInst.LRCmp		= 0;
	if (RSize < MinSize)
	{
		return	1;
	}

	ret	= Data.ExtractFeaturePyramid(InVal, InitS);
	for ( int Index = InitS * InVal; Index < (int) Data.FeaPyd.size( ); Index++ )
	{
		Scale	= Data.FeaPyd[Index].Scale;
		CSize	= MaxSize / (Scale * Scale);
		if (CSize >= g_Parameters.Overlap_Root * RSize)
		{
			ret	= InitNode(DetModel, Data, Index);
			ret	= MakeAppScore(DetModel, Data, Index);
			ret	= OrNodes[Root].MakeNodeScore(DetModel, Data, (*this));
			ret	= OrNodes[Root].GetInst_Score(DetModel, Data, (*this), CurInst);
		}
	}
	Data.FreePyramid( );

	return	1;
}

// Detect - find the instances whose score is large than a threshold (negative images)
int CInstTree::GetTopInsts(CDetectModel &DetModel, CData &Data, vector<CSVMInst> &InstSet, int &InstIdx, double Thres)
{
	int		Root	= DetModel.Root;
	int		AndCnt	= (int) AndNodes.size( );
	int		CurCnt	= InstIdx;
	int		ret;

	for ( int Index = DetModel.InitS * Data.InVal; Index < (int) Data.FeaPyd.size( ); Index++ )
	{
		ret	= InitNode(DetModel, Data, Index);
		ret	= MakeAppScore(DetModel, Data, Index);
		ret	= OrNodes[Root].MakeNodeScore(DetModel, Data, (*this));
		ret	= OrNodes[Root].GetTopInsts(DetModel, Data, (*this), InstSet, InstIdx, Thres);

		if (InstIdx - CurCnt > g_Parameters.MaxPerInstCnt)
		{
			break;
		}
	}

	return	1;
}

// Print all positions whose score is large than threshold
int	CInstTree::PrintLocSet(CData &Data, int &PosCnt)
{
	vector<pair<double, int> >		RectScores;
	ofstream						outRects;
	double							Thres		= - DBL_MAX;
	int								MaxCnt		= 12;
	int								MaxInstCnt	= 5000;
	char							fname[MAX_PATH];
	int								Idx;

	sprintf_s(fname, MAX_PATH, "LocSet_HoG\\%06d.txt", Data.DataID);
	outRects.open(fname);

	if ((int) NMSRects.size( ) > MaxCnt)
	{
		vector<CInstRect>		RInsts;

		RInsts.clear( );
		Thres	= NMSRects[MaxCnt].Score;
		for ( int t = 0; t < (int) InstRects.size( ); t++ )
		{
			if (InstRects[t].Score > Thres)
			{
				RInsts.push_back(InstRects[t]);
			}
		}
		RInsts.swap(InstRects);
	}

	RectScores.clear( );
	for ( int t = 0; t < (int) InstRects.size( ); t++ )
	{
		RectScores.push_back(make_pair(InstRects[t].Score, t));
	}
	sort(RectScores.begin( ), RectScores.end( ), greater<pair<double, int> >( ));

	MaxInstCnt	= __min(MaxInstCnt, (int) InstRects.size( ));
	PosCnt		+= MaxInstCnt;
	outRects<<MaxInstCnt<<endl;
	for ( int t = 0; t < MaxInstCnt; t++ )
	{
		Idx	= RectScores[t].second;
		outRects.precision(6);
		outRects<<setw(6)<<InstRects[Idx].ViewID;
		outRects<<setw(6)<<InstRects[Idx].Lvl;
		outRects<<setw(6)<<InstRects[Idx].PosX;
		outRects<<setw(6)<<InstRects[Idx].PosY;
		outRects<<setw(12)<<fixed<<InstRects[Idx].Score;
		outRects<<endl;
	}

	outRects<<(int) NMSRects.size( )<<endl;
	for ( int t = 0; t < (int) NMSRects.size( ); t++ )
	{
		outRects.precision(6);
		outRects<<setw(6)<<NMSRects[t].ViewID;
		outRects<<setw(6)<<NMSRects[t].Lvl;
		outRects<<setw(6)<<NMSRects[t].PosX;
		outRects<<setw(6)<<NMSRects[t].PosY;
		outRects<<setw(12)<<fixed<<NMSRects[t].Score;
		outRects<<endl;
	}

	outRects.close( );
	outRects.clear( );
	return	1;
}

// print the detection results
int	CInstTree::PrintInstRects(string strFileName, ofstream &outRes)
{
	for ( int t = 0; t < (int) NMSRects.size( ); t++ )
	{
		outRes<<strFileName.c_str( )<<" ";
		outRes.precision(6);
		outRes<<fixed<<NMSRects[t].Score<<" ";
		outRes<<NMSRects[t].RootRect.left + 1<<" ";
		outRes<<NMSRects[t].RootRect.top + 1<<" ";
		outRes<<NMSRects[t].RootRect.right<<" ";
		outRes<<NMSRects[t].RootRect.bottom<<endl;
	}

	return	1;
}

int CInstTree::DrawInstRects(CDetectModel &DetModel, CData &Data, CInstRect &CurRet)
{
	vector<Color>	&Colors		= g_Parameters.Colors;
	CImageEx		*pImage;
	RGBQUAD			*pBits;
	int				ret;

	for ( int l = 0; l < DetModel.MaxL; l++ )
	{
		pImage	= new CImageEx( );
		pImage->m_pBitmap = new Bitmap(Data.ImW, Data.ImH, PixelFormat32bppPARGB);
		pBits = (RGBQUAD*) pImage->GetBits();
		for ( int y = 0; y < Data.ImH; y++ )
		{
			for ( int x = 0; x < Data.ImW; x++, pBits++ )
			{
				pBits->rgbRed	= int(Data.SrcImg.at(x, y, 0, 0) + 0.5);
				pBits->rgbGreen	= int(Data.SrcImg.at(x, y, 0, 1) + 0.5);
				pBits->rgbBlue	= int(Data.SrcImg.at(x, y, 0, 2) + 0.5);
				pBits->rgbReserved	= 255;
			}
		}
		pImage->FreeBits( );

		Graphics		graphics(pImage->m_pBitmap);
		SolidBrush		myBrush(Color(255, 0, 0));
		Pen				myPen(&myBrush, 3);

		//myPen.SetColor(Color(255, 0, 0));
		//myBrush.SetColor(Color(255, 0, 0));
		myPen.SetColor(g_Parameters.Colors[CurRet.ViewID]);
		myBrush.SetColor(g_Parameters.Colors[CurRet.ViewID]);
		for ( int t = 0; t < DetModel.NodeCnt; t++ )
		{
			if (DetModel.AndModels[t].Level == l)
			{
				ret	= graphics.DrawRectangle(&myPen, CurRet.Rects[t].left, CurRet.Rects[t].top, CurRet.Rects[t].Width( ), CurRet.Rects[t].Height( ));
			}
		}

		char fname[MAX_PATH];
		sprintf_s(fname, MAX_PATH, "DEBUG\\%04d-%03d.jpg", Data.DataID, l);
		ret	= pImage->Save(fname);
		delete	pImage;
	}

	return	1;
}