#include "StdAfx.h"
#include "OrNode.h"
#include "AndNode.h"
#include "InstTree.h"
#include "InstRect.h"
#include "DetectModel.h"

COrNode::COrNode(void)
{
}

COrNode::~COrNode(void)
{
}

// calculate scores of all nodes
int COrNode::MakeNodeScore(CDetectModel &DetModel, CData &Data, CInstTree &InstTree)
{
	COrModel	&CurModel	= DetModel.OrModels[NodeID];
	int			ret;

	for ( int t = 0; t < (int) CurModel.ChildSet.size( ); t++ )
	{
		int			NxtID		= CurModel.ChildSet[t];
		CAndNode	&NxtNode	= InstTree.AndNodes[NxtID];

		ret = NxtNode.MakeNodeScore(DetModel, Data, InstTree);
	}

	return	1;
}

// Detect - find the bounding boxes whose score is large than a threshold
int COrNode::GetInstRects(CDetectModel &DetModel, CData &Data, CInstTree &InstTree, double Thres)
{
	COrModel	&CurModel	= DetModel.OrModels[NodeID];
	CInstRect	CurRet;
	int			ret;

	ret	= CurRet.Init(DetModel);
	for ( int t = 0; t < (int) CurModel.ChildSet.size( ); t++ )
	{
		int			NxtID		= CurModel.ChildSet[t];
		CAndNode	&NxtNode	= InstTree.AndNodes[NxtID];
		CAndModel	&NxtModel	= DetModel.AndModels[NxtID];

		for ( int y = 0; y < NxtNode.SizeY; y++ )
		{
			for ( int x = 0; x < NxtNode.SizeX; x++ )
			{
				if (NxtNode.matSre[y][x] > Thres)
				{
					ret	= InstTree.AndNodes[NxtID].GetInstRects(DetModel, Data, InstTree, CurRet, x, y);
					InstTree.InstRects.push_back(CurRet);
				}
			}
		}
	}

	return	1;
}

// Detect - find the instance with maximum overlap (positive images)
int COrNode::GetInst_Overlap(CDetectModel &DetModel, CData &Data, CInstTree &InstTree, CSVMInst &CurInst, double &MaxS)
{
	COrModel	&CurModel	= DetModel.OrModels[NodeID];
	int			ret;

	for ( int t = 0; t < (int) CurModel.ChildSet.size( ); t++ )
	{
		int			NxtID		= CurModel.ChildSet[t];
		CAndNode	&NxtNode	= InstTree.AndNodes[NxtID];
		CAndModel	&NxtModel	= DetModel.AndModels[NxtID];

		for ( int y = 0; y < NxtNode.SizeY; y++ )
		{
			for ( int x = 0; x < NxtNode.SizeX; x++ )
			{
				ret	= InstTree.AndNodes[NxtID].GetInst_Overlap(DetModel, Data, InstTree, CurInst, MaxS, x, y);
			}
		}
	}

	return	1;
}

// Detect - find the instance with maximum score (positive images)
int COrNode::GetInst_Score(CDetectModel &DetModel, CData &Data, CInstTree &InstTree, CSVMInst &CurInst)
{
	COrModel	&CurModel	= DetModel.OrModels[NodeID];
	int			ret;

	for ( int t = 0; t < (int) CurModel.ChildSet.size( ); t++ )
	{
		int			NxtID		= CurModel.ChildSet[t];
		CAndNode	&NxtNode	= InstTree.AndNodes[NxtID];
		CAndModel	&NxtModel	= DetModel.AndModels[NxtID];

		for ( int y = 0; y < NxtNode.SizeY; y++ )
		{
			for ( int x = 0; x < NxtNode.SizeX; x++ )
			{
				if ((CurInst.ViewID < 0) || (NxtNode.matSre[y][x] > CurInst.Score_SVM))
				{
					ret	= InstTree.AndNodes[NxtID].GetInst_Score(DetModel, Data, InstTree, CurInst, x, y);
				}
			}
		}
	}

	return	1;
}

// Detect - find the instances whose score is large than a threshold (negative images)
int COrNode::GetTopInsts(CDetectModel &DetModel, CData &Data, CInstTree &InstTree, vector<CSVMInst> &InstSet, int &InstIdx, double Thres)
{
	COrModel		&CurModel	= DetModel.OrModels[NodeID];
	vector<int>		InstX, InstY, InstV;
	vector<int>		Index;
	int				Cnt;
	int				ret;

	Cnt	= 0;
	Index.clear( );
	InstX.clear( );
	InstY.clear( );
	InstV.clear( );
	for ( int t = 0; t < (int) CurModel.ChildSet.size( ); t++ )
	{
		int			NxtID		= CurModel.ChildSet[t];
		CAndNode	&NxtNode	= InstTree.AndNodes[NxtID];
		CAndModel	&NxtModel	= DetModel.AndModels[NxtID];

		for ( int y = 0; y < NxtNode.SizeY; y++ )
		{
			for ( int x = 0; x < NxtNode.SizeX; x++ )
			{
				if (NxtNode.matSre[y][x] > Thres)
				{
					Index.push_back(Cnt);
					InstX.push_back(x);
					InstY.push_back(y);
					InstV.push_back(NxtID);
					Cnt++;
				}
			}
		}
	}

	// Rearranges the sequence of instances at random
	random_shuffle(Index.begin( ), Index.end( ));
	for ( int t = 0; t < Cnt; t++ )
	{
		int			Idx			= Index[t];
		int			NxtID		= InstV[Idx];
		CAndNode	&NxtNode	= InstTree.AndNodes[NxtID];
		CAndModel	&NxtModel	= DetModel.AndModels[NxtID];

		ret	= NxtNode.GetTopInsts(DetModel, Data, InstTree, InstSet[InstIdx], InstX[Idx], InstY[Idx]);
		InstIdx++;

		if (InstIdx >= (int) InstSet.size( ))
		{
			break;
		}
	}

	return	1;
}