#pragma once

#include <vector>
#include <string>
#include <iostream>
#include <fstream>
#include <iomanip>
#include <gdiplus.h>
#include <algorithm>
#include "FilterLib\\Timer.h"

using	namespace	std;
using	namespace	Gdiplus;

class CParameters
{
	public:
		CParameters(void);
		~CParameters(void);

	public:
		vector<Color>		Colors;				// Colors for drawing
		vector<Color>		RColors;
		int					SetColors( );
		int					SetRColors( );
		float				epsilon;

	public:
		string				strDataPathName;			// the folder name of dataset
		string				strClassName;				// the name of category, such as "car"
		string				strModelFileName;			// the filename of model
		string				strTrainSetFileName;		// train set
		string				strTestSetFileName;			// test set

	public:
		// Pyramid
		int					Interval_Train;				// the number of levels between scale=1.0 and scale=0.5
		int					Interval_Test;				// the number of levels between scale=1.0 and scale=0.5
		int					CellNumber_LowBound;		// the minimum size of the node in the top of feature pyramid

		// Position
		int					MarX, MarY;					// Margin = 8

		// Shape
		int					Dim_Shape;					// (dx, dy, dx*dx, dy*dy)=4

		// HoG
		int					BinSize_HoG;				// the size of cell of HoG (=8)
		int					Dim_HoG;					// 32 Dim

	public:
		int					MaxV;						// the number of views (set to 3)
		int					MaxL;						// the height of model
		int					MaxR;						// resolution 
		int					MaxDetectSize;				// the maximum area of model
		int					MinDetectSize;				// the minimum area of model

		int					Max_CCCP_Iter;				// the number of CCCP (25)
		int					Max_SMO_Iter;				// the times of visiting negative images in one CCCP (1)
		int					Max_KKT_Iter;				// the maximum iterations in one SMO
		float				Diff_KKT;					// the threshold. SMO terminates when increase is smaller than the threshold

		float				SVM_C;						// C
		double				SMO_epsilon;
		float				Bias_Node;					// the weight of bias term of each view
		float				Bias_Neg;					// the weight of bias term of negative data

		int					MaxPerInstCnt;				// the maximum number of instances added in one SMO
		int					MaxViewInstCnt;				// the maximum number of support vectors of each view
		int					MaxInstSetCnt;				// the maximum number of data in set of support vectors (data = patch)
		int					MaxInstCnt;

	public:
		int					NegPicCnt_Start;			// the number of negative images used in the first CCCP
		float				NegPicCnt_Ratio;			// the ratio of the number of negative images used this CCCP to them used last CCCP

	public:
		float				Overlap_NMS;				// threshold for NMS = 0.5
		float				Overlap_Root;				// the overlap of a positive bounding box should be larger than the threshold
		float				NullInstScore;

	// for debugging
	public:
		ofstream			outCCCP;
		ofstream			outTime;
		int					Iter_CCCP;
		CTimer				timer1, timer2, timer3, timer4, timer5;
		float				Time_All, Time_SMO, Time_Infer, Time_Ker, Time_Other;
};

extern	CParameters		g_Parameters;