#include "StdAfx.h"
#include "PascalData.h"

CPascalData::CPascalData(void)
{
}

CPascalData::~CPascalData(void)
{
}

void CPascalData::UnifyString(string &strName)
{
	while (strName[0] == ' ')
	{
		strName.erase(strName.begin( ));
	}

	int	len	= (int) strName.length( );
	while ((len > 0) && (strName[len-1] == ' '))
	{
		strName.erase(strName.end( ) - 1);
		len	= (int) strName.length( );
	}

	for ( int i = 0; i < (int) len; i++ )
	{
		strName[i]	= tolower(strName[i]);
	}
}


int	CPascalData::ReadAnnotation(string strFileName)
{
	ifstream	inAnno(strFileName.c_str( ));
	string		line;
	int			ObjCount, PointCount;

	getline(inAnno, strImageFileName, '\n');
	inAnno>>Height>>Width>>Depth;

	inAnno>>ObjCount;
	Objects.resize(ObjCount);
	for ( int t = 0; t < ObjCount; t++ )
	{
		getline(inAnno, line, '\n');
		getline(inAnno, line, '\n');
		getline(inAnno, Objects[t].strTagName, '\n');
		getline(inAnno, Objects[t].strPoseStatus, '\n');

		UnifyString(Objects[t].strTagName);
		UnifyString(Objects[t].strPoseStatus);

		inAnno>>Objects[t].Truncated;
		inAnno>>Objects[t].Difficult;

		inAnno>>PointCount;
		Objects[t].points.resize(PointCount);
		for ( int p = 0; p < PointCount; p++ )
		{
			inAnno>>Objects[t].points[p].x;
			inAnno>>Objects[t].points[p].y;
			Objects[t].points[p].x	-= 1;
			Objects[t].points[p].y	-= 1;
		}
	}

	inAnno.close( );
	return	1;
}

bool CPascalData::InCategoryNameSet(string strObjectName, vector<string> &CateSet)
{
	for ( int t = 0; t < (int) CateSet.size( ); t++ )
	{
		if (CateSet[t] == strObjectName)
		{
			return	true;
		}
	}

	return	false;
}

int CPascalData::GetUseObject(vector<string> &CateSet)
{
	CRect	Rect;

	Rects.clear( );
	for ( int t = 0; t < (int) Objects.size(); t++ )
	{
		// remove the objects which are not in this category or diffcult to detect
		if ((Objects[t].Difficult == 1) /*|| (Objects[t].Truncated == 1)*/ || (!InCategoryNameSet(Objects[t].strTagName, CateSet)))
		{
			Objects.erase(Objects.begin( ) + t);
			t--;
			continue;
		}

		Rect.left	= INT_MAX;
		Rect.right	= -1;
		Rect.top	= INT_MAX;
		Rect.bottom	= -1;
		for ( int p = 0; p < (int) Objects[t].points.size( ); p++ )
		{
			Rect.left	= __min(Objects[t].points[p].x, Rect.left);
			Rect.right	= __max(Objects[t].points[p].x + 1, Rect.right);
			Rect.top	= __min(Objects[t].points[p].y, Rect.top);
			Rect.bottom	= __max(Objects[t].points[p].y + 1, Rect.bottom);
		}

		Rects.push_back(Rect);
	}

	return	1;
}

int CPascalData::SavePosRect(string strRectFileName, CRect PosRect)
{
	ofstream	outRect(strRectFileName.c_str( ));

	outRect<<setw(6)<<PosRect.left;
	outRect<<setw(6)<<PosRect.right;
	outRect<<setw(6)<<PosRect.top;
	outRect<<setw(6)<<PosRect.bottom;

	outRect.close( );
	return	1;
}

// crop the object from image
int CPascalData::Clip(int RectID)
{
	CImg<double>	SrcImg;
	char			fname[MAX_PATH];
	CRect			CropRect;

	CropRect.left	= int(Rects[RectID].left   - Rects[RectID].Width( )  * 0.5);
	CropRect.right	= int(Rects[RectID].right  + Rects[RectID].Width( )  * 0.5 + 0.5);
	CropRect.top	= int(Rects[RectID].top    - Rects[RectID].Height( ) * 0.5);
	CropRect.bottom	= int(Rects[RectID].bottom + Rects[RectID].Height( ) * 0.5 + 0.5);
	CropRect.left	= __max(CropRect.left, 0);
	CropRect.top	= __max(CropRect.top, 0);
	CropRect.right	= __min(CropRect.right, Width);
	CropRect.bottom	= __min(CropRect.bottom, Height);

	PosRect.left	= Rects[RectID].left   - CropRect.left;
	PosRect.right	= Rects[RectID].right  - CropRect.left;
	PosRect.top		= Rects[RectID].top    - CropRect.top;
	PosRect.bottom	= Rects[RectID].bottom - CropRect.top;

	sprintf_s(fname, MAX_PATH, "%s\\Images\\%s", g_Parameters.strDataPathName.c_str( ), strImageFileName.c_str( ));
	CExternalTools::myLoadImage(fname, SrcImg);

	CropImg.assign(CropRect.Width( ), CropRect.Height( ), 1, 3);
	for ( int y = 0; y < CropImg.dimy( ); y++ )
	{
		for ( int x = 0; x < CropImg.dimx( ); x++ )
		{
			int	xx	= x + CropRect.left;
			int	yy	= y + CropRect.top;
			while ((xx < 0) || (xx >= Width))
			{
				xx	= (xx < 0) ? -xx : xx;
				xx	= (xx >= Width) ? 2*Width-2-xx : xx;
			}
			while ((yy < 0) || (yy >= Height))
			{
				yy	= (yy < 0) ? -yy : yy;
				yy	= (yy >= Height) ? 2*Height-2-yy : yy;
			}
			CropImg.at(x, y, 0, 0)	= SrcImg.at(xx, yy, 0, 0);
			CropImg.at(x, y, 0, 1)	= SrcImg.at(xx, yy, 0, 1);
			CropImg.at(x, y, 0, 2)	= SrcImg.at(xx, yy, 0, 2);
		}
	}

	return	1;
}

// left-right flip
int CPascalData::Flip( )
{
	CRect	PreRect	= PosRect;
	int		W		= CropImg.dimx( );
	int		H		= CropImg.dimy( );

	PosRect.left	= W - PreRect.right;
	PosRect.right	= W - PreRect.left;
	CropImg.mirror( );
	return	1;
}

int CPascalData::Save(string strFileName)
{
	char	fname[MAX_PATH];
	int		ret;

	sprintf_s(fname, MAX_PATH, "%s.jpg", strFileName.c_str( ));
	CExternalTools::mySaveImage(fname, CropImg);
	sprintf_s(fname, MAX_PATH, "%s.txt", strFileName.c_str( ));
	ret	= SavePosRect(fname, PosRect);
	return	1;
}