#include "StdAfx.h"
#include "SVMDetect.h"

CSVMDetect::CSVMDetect(void)
{
}

CSVMDetect::~CSVMDetect(void)
{
}

int CSVMDetect::ReadModel(string strModelName)
{
	int		ret;

	ret	= DetModel.ReadModel(strModelName);
	ret	= DetModel.GetNodeInfo( );
	ret	= DetModel.GetSymIdx( );
	return	1;
}

int CSVMDetect::FreeModel( )
{
	return	1;
}

int CSVMDetect::InitClassSet( )
{
	CategorySet.clear( );
	CategorySet.push_back(g_Parameters.strClassName);
	return	1;
}

// Detect
int CSVMDetect::DetectSet(string strImageSetFileName)
{
	vector<string>		FileNameSet;
	CInstTree			InstTree;
	ofstream			outRes;
	char				fname[MAX_PATH];
	int					DataCnt;
	int					ret;

	sprintf_s(fname, "comp3_det_test_%s.txt", g_Parameters.strClassName.c_str( ));
	outRes.open(fname);

	ret		= CExternalTools::ReadStringSet(strImageSetFileName, FileNameSet);
	DataCnt	= (int) FileNameSet.size( );
	ret		= InstTree.Init(DetModel);
	for ( int t = 0; t < DataCnt; t++ )
	{
		CData		Data;

		g_Parameters.timer1.Reset( );
		g_Parameters.timer2.Reset( );
		g_Parameters.timer3.Reset( );
		g_Parameters.timer4.Reset( );
		g_Parameters.timer5.Reset( );

		// Detect
		g_Parameters.timer1.Start( );
		sprintf_s(fname, MAX_PATH, "%s\\Images\\%s.jpg", g_Parameters.strDataPathName.c_str( ), FileNameSet[t].c_str( ));
		Data.DataID	= t;
		ret	= Data.ReadSourceImage(fname);
		ret	= InstTree.Detect(DetModel, Data, g_Parameters.Interval_Test);
		ret	= InstTree.NonMaxSupp(g_Parameters.Overlap_NMS);
		ret	= InstTree.PrintInstRects(FileNameSet[t], outRes);
		Data.FreeImageSpace( );
		g_Parameters.timer1.Stop( );

		cout.precision(1);
		cout<<"Image "<<setw(6)<<t+1;
		cout<<" - "<<setw(6)<<fixed<<g_Parameters.timer1.GetCumTime( );
		cout<<" - "<<setw(6)<<fixed<<g_Parameters.timer2.GetCumTime( );
		cout<<" - "<<setw(6)<<fixed<<g_Parameters.timer3.GetCumTime( );
		cout<<" - "<<setw(6)<<fixed<<g_Parameters.timer4.GetCumTime( );
		cout<<" - "<<setw(6)<<fixed<<g_Parameters.timer5.GetCumTime( );
		cout<<endl;
	}

	outRes.close( );
	return	1;
}

// Generate the training set
int CSVMDetect::GenDataSet(string strDataSetFileName)
{
	vector<string>		PosNameSet, NegNameSet;
	vector<string>		FileNameSet;
	char				fname[MAX_PATH];
	int					PosIdx	= 0;

	PosNameSet.clear( );
	NegNameSet.clear( );
	CExternalTools::ReadStringSet(strDataSetFileName, FileNameSet);
	for ( int t = 0; t < (int) FileNameSet.size( ); t++ )
	{
		CPascalData		AnnoObj;
		int				ret;

		sprintf_s(fname, MAX_PATH, "%s\\Annotations\\%s.txt", g_Parameters.strDataPathName.c_str( ), FileNameSet[t].c_str( ));
		ret		= AnnoObj.ReadAnnotation(fname);
		ret		= AnnoObj.GetUseObject(CategorySet);

		if (AnnoObj.Rects.size( ) > 0)
		{
			for ( int p = 0; p < (int) AnnoObj.Rects.size( ); p++ )
			{
				ret	= AnnoObj.Clip(p);
				if (ret > 0)
				{
					sprintf_s(fname, MAX_PATH, "PosSet\\%06d", PosIdx);
					ret	= AnnoObj.Save(fname);
					PosNameSet.push_back(fname);
					PosIdx++;

					ret	= AnnoObj.Flip( );
					sprintf_s(fname, MAX_PATH, "PosSet\\%06d", PosIdx);
					ret	= AnnoObj.Save(fname);
					PosNameSet.push_back(fname);
					PosIdx++;
				}
			}
		}

		if (AnnoObj.Rects.size( ) == 0)
		{
			NegNameSet.push_back(FileNameSet[t]);
		}

		cout<<setw(6)<<t<<"/"<<setw(6)<<(int) FileNameSet.size( )<<"\r";
	}

	CExternalTools::WriteStringSet("PosSet.txt", PosNameSet);
	CExternalTools::WriteStringSet("NegSet.txt", NegNameSet);
	cout<<"Generate Data Set --- FINISHED"<<endl;
	return	1;
}

// Read the training set
int CSVMDetect::ReadSet( )
{
	vector<string>		PosSet;
	vector<string>		NegSet;
	char				fname[MAX_PATH];
	int					InitS	= DetModel.InitS;
	int					ret;

	DataSet.Clear( );
	CExternalTools::ReadStringSet("PosSet.txt", PosSet);
	DataSet.PosCnt	= (int) PosSet.size( );
	DataSet.PosSet.resize(DataSet.PosCnt);
	DataSet.UseIdx_Pos.assign(DataSet.PosCnt, 1);
	for ( int t = 0; t < DataSet.PosCnt; t++ )
	{
		DataSet.PosSet[t].Label		= 1;
		DataSet.PosSet[t].DataID	= t;
		ret	= DataSet.PosSet[t].ReadSrcImgWithRect(PosSet[t]);
		DataSet.PosSet[t].FreeImageSpace( );
		cout<<setw(4)<<t<<"/"<<setw(4)<<DataSet.PosCnt<<"\r";
	}

	CExternalTools::ReadStringSet("NegSet.txt", NegSet);
	DataSet.NegCnt	= (int) NegSet.size( );
	DataSet.NegSet.resize(DataSet.NegCnt);
	DataSet.NegIdx.resize(DataSet.NegCnt);
	for ( int t = 0; t < DataSet.NegCnt; t++ )
	{
		DataSet.NegIdx[t]			= t;
		DataSet.NegSet[t].Label		= -1;
		DataSet.NegSet[t].DataID	= t;
		sprintf_s(fname, MAX_PATH, "%s\\Images\\%s.jpg", g_Parameters.strDataPathName.c_str( ), NegSet[t].c_str( ));
		ret	= DataSet.NegSet[t].ReadSourceImage(fname);
		DataSet.NegSet[t].FreeImageSpace( );
		cout<<setw(4)<<t<<"/"<<setw(4)<<DataSet.NegCnt<<"\r";
	}

	return	1;
}

// Initialize the model for detection
int CSVMDetect::InitSVM( )
{
	vector<vector<int> >	AspectIdx;
	vector<int>				Aspects;
	int						XCnt	= 3;
	int						YCnt	= 3;
	int						MaxL	= g_Parameters.MaxL;
	int						ret;

	ret	= ReadSet( );
	cout<<"Read Cropped Data --- Finished!"<<endl;
	ret	= DataSet.SplitByAspect(AspectIdx, g_Parameters.MaxV);
	ret	= DetModel.InitRoots(DataSet, AspectIdx);
	ret	= DetModel.WriteModel("PASCAL_Root_Init.txt");
	cout<<"Init Model (Root) --- Done "<<endl;

	ret	= DetModel.ReadModel("PASCAL_Root_Init.txt");
	ret	= DetModel.InitParts(MaxL, XCnt, YCnt);
	ret	= DetModel.WriteModel("PASCAL_Part_Init.txt");
	cout<<"Init Model (Part) --- Done "<<endl;

	return	1;
}

// Train the detection model
int CSVMDetect::TrainSVM( )
{
	int			ret;

	ret	= ReadSet( );
	cout<<"Read LSVM Data --- Finished"<<endl;

	ret	= DetModel.TrainSVM(DataSet);
	ret	= DetModel.WriteModel(g_Parameters.strModelFileName.c_str( ));
	return	1;
}