#include "StdAfx.h"
#include "SVMInst.h"
#include "Data.h"
#include "DetectModel.h"
#include "InstTree.h"

CSVMInst::CSVMInst(void)
{
}

CSVMInst::~CSVMInst(void)
{
}

int CSVMInst::Init(CDetectModel &DetModel)
{
	int		NodeCnt		= DetModel.NodeCnt;
	int		ViewCnt		= DetModel.ViewCnt;

	NodeXY.assign(NodeCnt * 2, -1);

	Score_Loss	= 0.0;
	Score_SVM	= 0.0;
	Score_KKT	= 0.0;
	Alpha		= 1.0;
	Index		= -1;

	Label		= -1;
	PosX		= -1;
	PosY		= -1;
	Lvl			= -1;
	DataID		= -1;
	ViewID		= -1;
	SymID		= -1;
	LRCmp		= 0;

	return	1;
}

// Alloc Memory for one instance
int CSVMInst::AllocMemory(CDetectModel &DetModel)
{
	int		NodeCnt		= DetModel.NodeCnt;
	int		ViewCnt		= DetModel.ViewCnt;
	int		MaxL_HoG	= 0;
	int		MaxL_Shp	= 0;

	for ( int t = 0; t < ViewCnt; t++ )
	{
		MaxL_HoG	= __max(MaxL_HoG, DetModel.LHoG[t]);
		MaxL_Shp	= __max(MaxL_Shp, DetModel.LShp[t]);
	}

	pHoG	= (double *) malloc(MaxL_HoG * sizeof(double));
	pShp	= (double *) malloc(MaxL_Shp * sizeof(double));
	return	1;
}

int CSVMInst::FreeMemory( )
{
	free(pHoG);
	free(pShp);
	return	1;
}

// Get hog features of a instance
int CSVMInst::GetAppFea(CDetectModel &DetModel, CData &Data, CInstTree &InstTree)
{
	int		Dim_HoG	= g_Parameters.Dim_HoG;

	memset(pHoG, 0, DetModel.LHoG[SymID] * sizeof(double));
	for ( int t = 0; t < DetModel.NodeCnt; t++ )
	{
		CAndModel	&CurModel	= DetModel.AndModels[t];
		CAndNode	&CurNode	= InstTree.AndNodes[t];
		int			cx			= NodeXY[2 * t + 0];
		int			cy			= NodeXY[2 * t + 1];
		int			HoG_S		= DetModel.HoG_S[CurModel.Level];
		int			HoG_L		= Lvl - HoG_S * Data.InVal;
		FeaMapType	&FeaMap		= Data.FeaPyd[HoG_L];
		int			Off_HoG		= CurModel.Off_HoG;
		int			Index		= 0;
		
		if ((cx < 0) || (cy < 0))
		{
			continue;
		}

		for ( int p = 0; p < Dim_HoG; p++ )
		{
			for ( int y = 0; y < CurModel.SizeY; y++ )
			{
				for ( int x = 0; x < CurModel.SizeX; x++ )
				{
					pHoG[Off_HoG + Index]	= FeaMap.Map_HoG.at(x + cx, y + cy, p);
					Index++;
				}
			}
		}

	}

	return	1;
}

// Get shape features
int CSVMInst::GetShpFea(CDetectModel &DetModel, CInstTree &InstTree, int NodeID, int x, int y)
{
	int		NodeCnt		= DetModel.NodeCnt;
	int		MaxL		= DetModel.MaxL;

	NodeXY.assign(NodeCnt * 2, -1);
	NodeXY[2*NodeID+0]	= x;
	NodeXY[2*NodeID+1]	= y;
	memset(pShp, 0, DetModel.LShp[SymID] * sizeof(double));

	for ( int t = 0; t < DetModel.NodeCnt; t++ )
	{
		CAndModel	&CurModel	= DetModel.AndModels[t];
		int			Lvl	= CurModel.Level;
		int			cx	= NodeXY[2 * t + 0];
		int			cy	= NodeXY[2 * t + 1];
		int			px, py, ox, oy;

		if ((cx < 0) || (cy < 0))
		{
			continue;
		}

		cx	*= DetModel.HoGStep[Lvl];
		cy	*= DetModel.HoGStep[Lvl];
		for ( int p = 0; p < (int) CurModel.ChildSet.size( ); p++ )
		{
			int			NxtID		= CurModel.ChildSet[p];
			CAndModel	&NxtModel	= DetModel.AndModels[NxtID];

			px	= InstTree.AndNodes[NxtID].matSrcX[cy][cx];
			py	= InstTree.AndNodes[NxtID].matSrcY[cy][cx];
			ox	= cx + NxtModel.PosX - px;
			oy	= cy + NxtModel.PosY - py;

			NodeXY[2*NxtID+0]	= px;
			NodeXY[2*NxtID+1]	= py;

			pShp[NxtModel.Off_Shp + 0]	= - 1.0f * ox * ox;
			pShp[NxtModel.Off_Shp + 1]	= - 1.0f * ox;
			pShp[NxtModel.Off_Shp + 2]	= - 1.0f * oy * oy;
			pShp[NxtModel.Off_Shp + 3]	= - 1.0f * oy;
		}
	}

	for ( int t = 0; t < DetModel.NodeCnt; t++ )
	{
		if ((NodeXY[2 * t + 0] >= 0) && (NodeXY[2 * t + 0] >= 0))
		{
			NodeXY[2 * t + 0]	+= InstTree.AndNodes[t].InitX + g_Parameters.MarX;
			NodeXY[2 * t + 1]	+= InstTree.AndNodes[t].InitY + g_Parameters.MarY;
		}
	}

	return	1;
}

int CSVMInst::GetLRCmp(CDetectModel &DetModel)
{
	double	LSre	= 0;
	double	RSre	= 0;

	for ( int NodeID = 0; NodeID < (int) DetModel.AndModels.size( ); NodeID++ )
	{
		if (DetModel.AndModels[NodeID].ViewID != ViewID)
		{
			continue;
		}

		CAndModel		&CurModel	= DetModel.AndModels[NodeID];
		int				Off_HoG		= CurModel.Off_HoG;
		int				Dim_HoG		= CurModel.Dim_HoG;

		if (CurModel.NodeID < CurModel.Partner)
		{
			for ( int p = 0; p < Dim_HoG; p++ )
			{
				if (pHoG[Off_HoG + p] < 0.8)
				{
					LSre	+= pHoG[Off_HoG + p];
				}
			}
		}

		if (CurModel.NodeID > CurModel.Partner)
		{
			for ( int p = 0; p < Dim_HoG; p++ )
			{
				if (pHoG[Off_HoG + p] < 0.8)
				{
					RSre	+= pHoG[Off_HoG + p];
				}
			}
		}
	}

	LRCmp	= LSre - RSre;
	return	1;
}